<?php
// Date and time formatter
//
class AdministrateDateTime {
	
	// Format single date
	public static function date( $date, $date_format, $show_month, $show_year ) {
		if( $date_format == "mdy" ) {
			// Jan 01, 2015
			$format  = $show_month ? "M " : "";
			$format .= "d";
			$format .= $show_year ? ", Y" : "";
		}
		else {
			// 01 Jan 2015
			$format  = "d";
			$format .= $show_month ? " M" : "";
			$format .= $show_year ? " Y" : "";
		}
		return date( $format, $date );
	}
	
	// Format a date range
	public static function date_range( $dates, $date_format, $show_year ) {
		if( !$dates ) {
			$dates = array( 'start' => time(), 'end' => time() );
		}
		$start_date = $dates['start'];
		$end_date   = $dates['end'];

		$one_date = (bool) ($start_date == $end_date || $end_date === false);
		$has_diff_month = (bool) ( date('M', $start_date) != date('M', $end_date) );
		$has_diff_year  = (bool) ( date('Y', $start_date) != date('Y', $end_date) );

		if( $date_format == "mdy" ) {
			/*
				Jan 01 -- 10, 2015
				Jan 01 -- Dec 10, 2015
				Jan 01, 2015 -- Dec 10, 2016
			*/
			$t = self::date(
				$start_date, $date_format,
				true,
				( $has_diff_year || $one_date ) && $show_year
			);
		
			if( !$one_date ) {
				$t .= " &ndash; ";
				$t .= self::date( 
					$end_date, $date_format,
					$has_diff_month || $has_diff_year,
					$show_year
				);
			}
		}
		else {
			/*
				01 -- 10 Jan 2015
				01 Jan -- 10 Dec 2016
				01 Jan 2015 -- 10 Sec 2016
			*/
			$t = self::date(
				$start_date, $date_format,
				$has_diff_month || $one_date || $has_diff_year,
				( $has_diff_year || $one_date ) && $show_year
			);
		
			if( !$one_date ) {
				$t .= " &ndash; ";
				$t .= self::date(
					$end_date, $date_format,
					true,
					$show_year
				);
			}
		}
		return $t;
	}
	
	// Format single time
	public static function time( $time, $show_24h_times ) {
		if( !$time ) {
			return "&ndash;";
		}
		
		$time = date_parse( $time );
		$timestamp = mktime($time["hour"], $time["minute"], 0, 0, 0, 0);
		
		if( $show_24h_times ) {
			// 20:00h
			return date( 'H:i', $timestamp )."h";
		}
		else {
			// 8PM
			return date( 'g:iA', $timestamp );
		}
	}
	
	// Format a time range
	public static function time_range( $times, $show_24h_times ) {
		$t = self::time( $times["start"], $show_24h_times );
		
		if( $times["start"] && $times["end"] && ( $times["end"] != $times["start"]) ) {
			$t .= " &ndash; ";
			$t .= self::time( $times["end"], $show_24h_times );
		}
		return $t;
	}

	public static function date_time($time_string, $output_tz='UTC', $format='M d, Y g:iA') {
		$timezone = new DateTimeZone('UTC');
		$date = new DateTime($time_string, $timezone);

		// If the timezone is numeric, we assume it's an offset in number of hours, otherwise
		// assume that it is an ISO timezone format and attempt to convert using DateTime.
		if(is_numeric($output_tz)) {

			// The offset in minutes is abs of the number of hours * 60.
			$offset_minutes = abs($output_tz) * 60;
			$interval = new DateInterval('PT' . $offset_minutes . 'M');

			// If it's positive, add, otherwise sub.
			if($output_tz > 0) {
				$date->add($interval);
			} else {
				$date->sub($interval);
			}
		} else {

			try {
				$output_timezone = new DateTimeZone($output_tz);
				$date->setTimezone($output_timezone);
			} catch(Exception $e) {
				// Nothing really to do here. We couldn't convert by offset, and couldn't
				// convert by string, so just output the datetime in UTC that we generated
				// at the beginning.
			}
		}

		return $date->format($format);
	}
}
