<?php
// Caching algorithm for Weblink and Wordpress plugin
//
//
class AdministrateCacheAlgorithm {
	private $_cache = null;

	public function __construct($cache) {
		$this->_cache = $cache;
	}

	//  Find cached objects
	public function find_objects($method, $args = array()) {
		if ( !$this->_cache->is_call_cacheable($method) ) {
			return null;
		}

		$entity = $this->_cache->get_call_type($method);
		$entity_params = $this->_cache->get_call_parameters($entity);

		// Multiple object requested
		if( $this->_cache->is_result_multiple($method) ) {
			return $this->_find_multiple_objects($method, $args, $entity_params);
		}

		// Single object requested
		else {
			return $this->_find_single_object($method, $args, $entity_params);
		}
	}

	// Record API objects
	public function record_objects($method, $args, $api_data) {
		if ( !$this->_cache->is_call_cacheable($method) ) {
			return;
		}

		$entity = $this->_cache->get_call_type($method);
		$entity_params = $this->_cache->get_call_parameters($entity);

		// Multiple objects requested
		if( $this->_cache->is_result_multiple($method) ) {
			$this->_record_multiple_objects($method, $args, $entity_params, $api_data);
		}

		// Single object requested
		else {
			$this->_record_single_object($method, $args, $entity_params, $api_data);
		}
	}

	// Find single object
	private function _find_single_object($method, $args, $entity_params) {
		return $this->_cache->fetch_single_object($method, $args[0], $entity_params);
	}

	// Find multiple objects
	private function _find_multiple_objects($method, $args, $entity_params) {

		// Fetch API call results form database
		$id_str = $this->_cache->fetch_api_call($method, $args, $entity_params);

		// Do we have cache records for this call?
		if ($id_str) {
			$object = $this->_cache->fetch_multiple_objects($method, $id_str, $entity_params);
			$id_array = explode(",",$id_str);

			if( is_object($object) && count($object->{$entity_params['list_name']}) < count($id_array) ) {
				// If this is a partial cache hit, force referesh
				return null;
			}
			return $object;
		}
		return null;
	}

	// Record single object
	private function _record_single_object($method, $args, $entity_params, $data) {
		$this->_cache->record_object($method, $args[0], $data);
	}

	// Record multiple objects
	private function _record_multiple_objects($method, $args, $entity_params, $data) {
		$rows = array();
		$id_array = array();

		// Record each object as separate cache entry
		if (!is_null($data->{$entity_params['list_name']})) {
			foreach ($data->{$entity_params['list_name']} as $row) {
				$id = $row->{$entity_params['id_column']};
				$id_array[] = $id;

				$this->_cache->record_object($method, $id, $row);
			}
		} elseif(is_null($data->{$entity_params['list_name']})) {

			if(!is_array($data)) {
				// Don't attempt to cache data if there is none.
				return;
			}

			foreach($data as $row) {
				$id = $row->{$entity_params['id_column']};
				$id_array[] = $id;
				$this->_cache->record_object($method, $id, $row);
			}
		}

		// Record API call result
		$this->_cache->record_api_call($method, $args, $id_array);
	}

}
