<?php
// Administrate API call validation class
//
class AdministrateCallValidator {
	
	// Valid API calls
	private static $_valid_api_calls = array(
		'getAllDiscounts' => array(
			// No arguments
		),
		'sendInvoice' => array( 
			FILTER_VALIDATE_INT 
		),
		'getEventByID' => array( 
			FILTER_VALIDATE_INT 
		),
		'getNumDelegates' => array(
			FILTER_VALIDATE_INT
		),
		'getCourseByCode' => array (
			FILTER_SANITIZE_STRING
		),
		'getUserByEmail' => array(
			FILTER_SANITIZE_STRING
		),
		'getPaymentIntegrationConfig' => array(
			FILTER_SANITIZE_STRING
		),
		'getEvents' => array(
			'CourseCode' => FILTER_SANITIZE_STRING,
			'CourseCategoryID' => FILTER_VALIDATE_INT,
			'CourseSubCategoryID' => FILTER_VALIDATE_INT,
			'EventID' => FILTER_VALIDATE_INT,
			'EventStartDate' => FILTER_SANITIZE_STRING,
			'EventLocationName' => FILTER_SANITIZE_STRING,
			'IncludePrices' => FILTER_VALIDATE_BOOLEAN,
			'IncludeCurrentOnline' => FILTER_VALIDATE_BOOLEAN,
			'IncludeSessions' => FILTER_VALIDATE_BOOLEAN,
			'LocationRegionID' => FILTER_VALIDATE_INT,
			'SortBy' => FILTER_SANITIZE_STRING,
			'InvoiceCountry' => FILTER_SANITIZE_STRING
		),
		'getCourses' => array(
			'CourseID' => FILTER_VALIDATE_INT,
			'Disposition' => FILTER_SANITIZE_STRING,
			'IncludeCancelled' => FILTER_SANITIZE_STRING,
			'IncludeCustomFields' => FILTER_SANITIZE_STRING,
			'CourseCategoryID' => FILTER_VALIDATE_INT,
			'CourseSubCategoryID' => FILTER_VALIDATE_INT
		),
		'getCategories' => array(),
		'addUser' => array(
			'IsIndividual' => FILTER_VALIDATE_BOOLEAN,
			'Email' => FILTER_VALIDATE_EMAIL,
			'DontMail' => FILTER_VALIDATE_BOOLEAN,
			'Password' => FILTER_SANITIZE_STRING,
			'Company' => FILTER_SANITIZE_STRING,
			'FirstName' => FILTER_SANITIZE_STRING,
			'LastName' => FILTER_SANITIZE_STRING,
			'JobTitle' => FILTER_SANITIZE_STRING,
			'Department' => FILTER_SANITIZE_STRING,
			'Address1' => FILTER_SANITIZE_STRING,
			'Address2' => FILTER_SANITIZE_STRING,
			'Address3' => FILTER_SANITIZE_STRING,
			'City' => FILTER_SANITIZE_STRING,
			'County' => FILTER_SANITIZE_STRING,
			'PostCode' => FILTER_SANITIZE_STRING,
			'CountryCode' => FILTER_SANITIZE_STRING,
			'Tel' => FILTER_SANITIZE_STRING,
			'Mobile' => FILTER_SANITIZE_STRING,
			'HearAboutUs' => FILTER_SANITIZE_STRING,
			'HearAboutUsText' => FILTER_SANITIZE_STRING,
			'ContactID' => FILTER_SANITIZE_STRING,
			'CustomDate1' => FILTER_SANITIZE_STRING,
			'CustomDate2' => FILTER_SANITIZE_STRING,
			'CustomDropDown1' => FILTER_SANITIZE_STRING
		),
		'addDelegate' => array(
			'WebsiteUserID' => FILTER_VALIDATE_INT,
			'Email' => FILTER_VALIDATE_EMAIL,
			'FirstName' => FILTER_SANITIZE_STRING,
			'LastName' => FILTER_SANITIZE_STRING,
			'JobTitle' => FILTER_SANITIZE_STRING,
			'Department' => FILTER_SANITIZE_STRING,
			'Address1' => FILTER_SANITIZE_STRING,
			'Address2' => FILTER_SANITIZE_STRING,
			'Address3' => FILTER_SANITIZE_STRING,
			'City' => FILTER_SANITIZE_STRING,
			'County' => FILTER_SANITIZE_STRING,
			'PostCode' => FILTER_SANITIZE_STRING,
			'Country' => FILTER_SANITIZE_STRING,
			'Tel' => FILTER_SANITIZE_STRING,
			'Fax' => FILTER_SANITIZE_STRING,
			'Mobile' => FILTER_SANITIZE_STRING
		),
		'placeOrder' => array(
			'orderDetails' => array(
				'IsPending' =>	FILTER_VALIDATE_BOOLEAN,
				'Date' => FILTER_SANITIZE_STRING,
				'UserID' => FILTER_VALIDATE_INT,
				'PaymentMethod' => FILTER_SANITIZE_STRING,
				'Currency' => FILTER_SANITIZE_STRING,
				'Region' => FILTER_SANITIZE_STRING,
				'Email' => FILTER_VALIDATE_EMAIL,
				'Company' => FILTER_SANITIZE_STRING,
				'FirstName' => FILTER_SANITIZE_STRING,
				'LastName' => FILTER_SANITIZE_STRING,
				'Phone' => FILTER_SANITIZE_STRING,
				'Address1' => FILTER_SANITIZE_STRING,
				'Address2' => FILTER_SANITIZE_STRING,
				'Address3' => FILTER_SANITIZE_STRING,
				'City' => FILTER_SANITIZE_STRING,
				'Territory' => FILTER_SANITIZE_STRING,
				'PostalCode' => FILTER_SANITIZE_STRING,
				'Country' => FILTER_SANITIZE_STRING,
				'Notes' => FILTER_SANITIZE_STRING,
				'AltAddresses' => array(),
				'PromoCode' => FILTER_SANITIZE_STRING,
			),
			'paymentDetails' => array(),
			'cartData' => array()
		)
	);
	
	// Ensure API method is available
	public static function validate($method, $args) {
		
		// Assert method exists
		if( !array_key_exists($method, self::$_valid_api_calls) ) {
			throw new Exception('Method ' . $method . ' not found.');
		}
		
		// Arguments of getEvents are contained in 'Filter' key
		if( $method == "getEvents" ) {
			if( !array_key_exists('Filter', $args) ) {
				throw new Exception('Method getEvents expects arguments in parameter Filter.');
			}
			$args = $args['Filter'];
		}
		
		// Argument of getPaimentConfig is in 'Provider' key
		if( $method == "getPaymentIntegrationConfig" ) {
			if( !array_key_exists('provider', $args) ) {
				throw new Exception('Method getPaymentIntegrationConfig expects argument in parameter Provider.');
			}
			$args[0] = $args['provider'];
			unset($args['provider']);
		}
		
		// All API calls accept single arguments in the form array(argument_value)
		// The call sendInvoice is the only exception - it accepts a plain value
		/*if( $method == "sendInvoice" ) {
			// Enforce the format array(argument_value) before validation
			$args = array($args);
		}*/
		
		// Arguments for this method
		$valid_args = self::$_valid_api_calls[$method];
		
		// Single-argument methods are called without key (only value)
		if( count($valid_args)==1 ) {
			if( count($args) != 1 ) {
				throw new Exception('Method ' .$method. ' expects ' .count($valid_args). ' arguments.');
			}
			// No argument name, only value is passed for single aruments
			self::validate_field($method, "", $args[0], $valid_args[0]);
			return;
		}
		
		// Argument array provided
		foreach( $args as $name => $value ) {
			
			// Verify argument name
			$name_validated = false;
			
			foreach( $valid_args as $valid_name => $valid_value ) {
				if( $name == $valid_name) {
					
					// Verify argument type
					$name_validated = true;
					self::validate_field($method, $name, $value, $valid_value );
					break;
				}
			}
			
			// Argument not matched by any valid arugment
			if( !$name_validated ) {
				throw new Exception( 'Argument ' . $name . ' not valid in ' . $method . ' checked against ' . var_export($valid_args, true));
			}
		}
		
	}
	
	public static function validate_field($method, $field_name, $field, $filter) {

		if( is_array($filter) ) {
			// TODO: Validate sub-fields recursively
			return;
		}

		if($filter == FILTER_VALIDATE_BOOLEAN) {

			// Check if fields to be validated as bools are string representations of T or F.
			if(in_array(strtolower($field), array('true', 'false'))) {
				return;
			}

			// if it's definitely a bool...
			if(is_bool($field)) {
				return;
			}
		}

		// Validate single field
		else if( !filter_var($field, $filter) ) {
				throw new Exception( 'Argument '.$field_name.' for ' . $method . ' must be filter_id=' . var_export($filter, true) . ', was given ' . var_export($field, true));
		}
	}
}
?>
