<?php
// Administrate API Event
class AdministrateAPIEvent extends AdministrateAPIObject {

	private $course = null;

	private static $fields = array(
		'id'				=>	array(
			'api_field'	=>	'EventID'
		),
		'default_currency'	=>	array(
			'api_field'	=>	'DefaultCurrencyCode'
		),
		'prices'			=>	array(
			'api_field'	=>	'EventPrices'
		),
		'start_date'		=>	array(
			'api_field'	=>	'EventStartDate'
		),
		'end_date'			=>	array(
			'api_field'	=>	'EventEndDate'
		),
 		'start_time'		=>	array(
 			'api_field'	=>	'StartTime'
 		),
 		'end_time'			=>	array(
 			'api_field'	=>	'EndTime'
 		),
		'classroom_start_date_time' => array(
			'api_field' => 'ClassroomStart'
		),
		'classroom_end_date_time' => array(
			'api_field' => 'ClassroomEnd'
		),
		'location'			=>	array(
			'api_field'	=>	'EventLocationName'
		),
		'location_id'       => array(
			'api_field' =>  'LocationID'
		),
		'course_code'		=>	array(
			'api_field'	=>	'CourseCode'
		),
		'course_title'		=>	array(
			'api_field'	=>	'CourseTitle'
		),
		'sold_out'			=>	array(
			'api_field'	=>	'SoldOut'
		),
		'course_categories'	=>	array(
			'api_field'	=>	'CourseCategories'
		),
		'is_provisional'	=>	array(
			'api_field'	=>	'IsProvisional'
		),
		'is_online'			=>	array(
			'api_field'	=>	'IsOnline'
		),
		'region_id'			=>	array(
			// REST property name for Region ID
			'api_field'	=>	'RegionID'
		),
		'location_region_id'	=> array(
			// SOAP property name for Region ID
			'api_field' => 'LocationRegionID'
		),
		'max_places'		=>	array(
			'api_field'	=>	'MaxPlaces'
		),
		'num_delegates'	=>	array(
			'api_field'	=>	'Delegates'
		),
		'event_type' => array(
			'api_field'	=>	'EventDisposition'
		),
		'sessions'	=>	array(
			'api_field'	=>	'EventChildrenSessions'
		),
		'is_eu_digital_good'	=>	array(
			'api_field'	=>	'IsEuDigitalGood'
		),
		'tax_name'	=>	array(
			'api_field'	=>	'TaxName'
		),
		'tax_id'	=>	array(
			'api_field'	=>	'TaxID'
		),
		'venue_name'	=>	array(
			'api_field'	=>	'VenueName'
		),
		'venue_description'	=>	array(
			'api_field'	=>	'VenueDescription'
		),
		'persistent'	=> array(
			'api_field'	=>	'Persistent'
		),
		'access_duration' => array(
			'api_field'	=>	'LmsExpireDays'
		),
		'registration_deadline' => array(
			'api_field' => 'RegistrationDeadline'
		),
		'finance_code' => array(
			'api_field' => 'FinanceCode'
		)
	);

	// Get copy of fields
	protected static function _get_api_map() {
		return self::$fields;
	}

	// Get the event's course code
	public function get_course_code() {
		return $this->_get_field('course_code');
	}

	// Get the event's course title
	public function get_course_title() {
		return $this->_get_field('course_title');
	}

	// Is the event persistent?
	public function is_persistent() {
		return ($this->get_start_date() == false) && ($this->get_end_date() == false);
	}

	public function get_access_duration() {
		return $this->_get_field('access_duration');
	}

	public function get_registration_deadline() {
		return $this->_get_field('registration_deadline');
	}

	// Get the event's course
	public function get_course() {
		$this->_set_course();
		return $this->course;
	}

	// Get the event's course summary
	public function get_course_summary() {
		$this->_set_course();
		return $this->course->get_summary();
	}

	// Get the event's course schedule
	public function get_course_schedule() {
		$this->_set_course();
		return $this->course->get_schedule();
	}

	// Get the event's course inclusions
	public function get_course_inclusions() {
		$this->_set_course();
		return $this->course->get_inclusions();
	}

	// Get the event's course custom_text_8
	public function get_course_custom_text_8() {
		$this->_set_course();
		return $this->course->get_custom_text_8();
	}

	// Get the event's course method
	public function get_course_method() {
		$this->_set_course();
		return $this->course->get_method();
	}

	// Get the event's course prerequisites
	public function get_course_prerequisites() {
		$this->_set_course();
		return $this->course->get_prerequisites();
	}

	// Get the event's course topics
	public function get_course_topics() {
		$this->_set_course();
		return $this->course->get_topics();
	}

	// Get the event's course benefits
	public function get_course_benefits() {
		$this->_set_course();
		return $this->course->get_benefits();
	}

	// Get the event's course duration
	public function get_course_duration() {
		$this->_set_course();
		return $this->course->get_duration();
	}

	public function get_venue_name() {
		return $this->_get_field('venue_name');
	}

	public function get_venue_description() {
		return $this->_get_field('venue_description');
	}

	public function get_classroom_start_date_time() {
		return $this->_get_field('classroom_start_date_time');
	}

	public function get_classroom_end_date_time() {
		return $this->_get_field('classroom_end_date_time');
	}

	public function get_classroom_start_date() {
		$start = new DateTime($this->get_classroom_start_date_time());
		$start->setTime(0, 0, 0);
		return $start->format('U');
	}

	public function get_classroom_end_date() {
		$end = new DateTime($this->get_classroom_end_date_time());
		$end->setTime(0, 0, 0);
		return $end->format('U');
	}

	public function get_classroom_start_time() {
		$start = new DateTime($this->get_classroom_start_date_time());
		return $start->format('H:i');
	}

	public function get_classroom_end_time() {
		$end = new DateTime($this->get_classroom_end_date_time());
		return $end->format('H:i');
	}

	private function _event_has_classroom_element() {
		return !is_null($this->get_classroom_start_date_time()) && !is_null($this->get_classroom_end_date_time());
	}

	// Get the event's dates
	public function get_dates($classroom = false) {
		if($classroom && $this->_event_has_classroom_element()) {
			return array(
				'start' => $this->get_classroom_start_date(),
				'end' => $this->get_classroom_end_date()
			);
		}

		return array(
			'start'	=>	$this->get_start_date(),
			'end'	=>	$this->get_end_date()
		);
	}

	// Get the event's start date
	public function get_start_date() {
		return strtotime($this->_get_field('start_date'));
	}

	// Get the event's end date
	public function get_end_date() {
		return strtotime($this->_get_field('end_date'));
	}

	// Get the event's times
	public function get_times($classroom = false) {
		if($classroom) {
			return array(
				'start' => $this->get_classroom_start_time(),
				'end' => $this->get_classroom_end_time()
			);
		}

		return array(
			'start'	=>	$this->get_start_time(),
			'end'	=>	$this->get_end_time()
		);
	}

	// Get the event's start time
	public function get_start_time() {
		return $this->_get_field('start_time');
	}

	// Get the event's end time
	public function get_end_time() {
		return $this->_get_field('end_time');
	}

	// Whether or not the event is today
	public function is_today() {
		return ($this->get_start_date() == strtotime('today'));
	}

	// Get the finance code
	public function get_finance_code() {
		return $this->_get_field('finance_code');
	}

	// Get the event location
	public function get_location() {
		return $this->_get_field('location');
	}

	public function get_location_id() {
		return $this->_get_field('location_id');
	}

	// Get the region ID
	public function get_region_id() {
		// Try REST property for region ID
		$id = $this->_get_field('region_id');
		if( !$id ) {
			// If not available, try SOAP property for region ID
			$id = $this->_get_field('location_region_id');
		}
		return $id;
	}

	// Get prices
	public function get_prices() {
		// Fetch prices
		if (!property_exists($this, 'prices'))
		{
			$this->prices = array();
			if ($this->_get_field('prices')) {
				foreach ($this->_get_field('prices') as $price) {
  					array_push($this->prices, new AdministrateAPIEventPrice($price));
				}
			}
		}
		// Return prices
		return $this->prices;
	}

	public function get_tax_name() {
		return $this->_get_field('tax_name');
	}

	public function get_tax_id() {
		return $this->_get_field('tax_id');
	}

	public function is_eu_digital_good() {
		return $this->_get_field('is_eu_digital_good');
	}

	// Get price by currency
	public function get_price_by_currency($currency) {
		$best_price = null;

		foreach($this->get_prices() as $price) {
			if(
				$price->get_currency() == $currency
				&& ((is_object($best_price)) && ($price->get_net() < $best_price->get_net()) || is_null($best_price))
				&& (time() < strtotime($price->get_price_level_expiry()) || is_null($price->get_price_level_expiry()))
			) {
				$best_price = $price;
			}
		}

		if(!is_null($best_price)) {
			return $best_price;
		}

		return false;
	}

	// Get default currency
	public function get_default_currency() {
		$default_currency = $this->_get_field('default_currency');

		if( $this->get_price_by_currency( $default_currency ) ) {
			return $default_currency;
		}

		$prices = $this->get_prices();
		return $prices[0]->get_currency();
	}

	// Whether or not the event is sold out
	public function is_sold_out() {
		// Marked as sold out, or no available places
		return ($this->_get_field('sold_out') || ($this->get_num_places() === 0));
	}

	// Is event multisession
	public function is_multi_session() {
		$sessions = $this->get_sessions();
		return (bool) ( is_array($sessions) && ( count($sessions) > 1 ) );
	}

	// Get event type (Normal, Multi-Session, Compound)
	public function get_type() {
		return $this->_get_field('event_type');
	}

	// Get event sessions
	public function get_sessions() {
		if( property_exists($this, "sessions") ) {
			return $this->sessions;
		}

		$sessions_data = $this->_get_field('sessions');
		$this->sessions = array();

		if( $sessions_data ) {
			foreach( $sessions_data as $s ) {
				$this->sessions[] = new AdministrateAPIEventSession($s);
			}
		}
		return $this->sessions;
	}

	// Get course categories
	public function get_categories() {
		$categories = array();
		foreach ($this->_get_field('course_categories') as $category) {
			array_push($categories, new AdministrateAPICourseCategory($category));
		}
		return $categories;
	}

	// Get course subcategories
	public function get_subcategories() {
		$subcategories = array();
		/*foreach ($this->_get_field('course_categories') as $category) {
			array_push($categories, new AdministrateAPICourseCategory($category));
		}*/
		return $categories;

	}

	// Is event in subcategory
	public function is_in_subcategory($sub_id) {
		foreach ($this->get_categories() as $category) {
			foreach ($category->get_subcategories() as $subcategory) {
				if ($subcategory->get_id() == $sub_id) {
						return true;
					}
				}
			}
			return false;
	}

	// Whether or not the event is provisional
	public function is_provisional() {
		return $this->_get_field('is_provisional');
	}

	// Whether or not the event is online
	public function is_online() {
		return $this->_get_field('is_online');
	}

	// Max number of available places
	// This can be null ( no set limit ), 0 ( no places ), or positive number ( places available )
	public function get_max_places() {
		return $this->_get_field('max_places');
	}

	// Number of currently available places
	public function get_num_places() {
		if( !property_exists($this, 'num_places') ) {

			if( $this->get_max_places() === null ) {
				$this->num_places = null;
			}
			else {
				$this->num_places = intval( max($this->get_max_places() - $this->get_num_delegates(), 0) );
			}
		}
		return $this->num_places;
	}

	// Is registration enabled for this event
	public function registration_available() {
		// The number of places can be null ( no set limit ), 0 ( no places ), or positive number ( places available )
		// Registration is enabled for null and positive number. It's disabled only for 0 (no places).
		return (bool) ($this->get_num_places()!==0);
	}

	// Get the number of delegates
	public function get_num_delegates() {
		return $this->_get_field('num_delegates');
	}

	// Get the number of months to the event stat date
	public function get_months_until_start() {
		$dates = $this->get_dates();
		$start_timestamp = $dates['start'];
		$current_timestamp = time();
		$i = 1;

		if( $current_timestamp >= $start_timestamp ) {
			return 0;
		}

		while( strtotime("+{$i} MONTH", $current_timestamp) <= $start_timestamp ) {
			$i++;
		}
		return $i;
	}

	// Set the course
	private function _set_course() {
		if (is_null($this->course)) {
 			$this->course = Administrate::api()->get_course($this->get_course_code());
		}
	}

}
