<?php

// Allow the endpoint to be hit on it's own (e.g. not as a real relay response) for
// verification that it is accessible to the outside world. Display message and quit.
if(!array_key_exists('api', $_REQUEST)) {
	echo 'Authorize.NET relay response endpoint is visible and accessible.';
	exit();
}

// Include configs and initialise the main plugin class.
include_once('../../../../../../wp-config.php');
global $_ADMINISTRATE_CONFIG, $wpdb;
$adm = new AdministratePlugin($_ADMINISTRATE_CONFIG, $wpdb);

// Load the MD5 from config and attempt verification if it's set.
$md5 = $adm->get_option('authorizenet_md5', 'checkout');
if(($md5 != "") && ($md5 != "disabled")) {

	// Verify the received hash, re-hash and check the expected outcome.
	$hash_received = $_REQUEST['x_MD5_Hash'];
	$hash_str = $md5 . $_REQUEST['api'] . $_REQUEST['x_trans_id'] . $_REQUEST['x_amount'];
	$hash_expected = strtoupper(md5($hash_str));

	if ($hash_received != $hash_expected) {

		// Potentially fraudulent transaction
		die("This transaction cannot be authenticated.");
	}
}

// Load up the order.
require_once('../../../wordpress-components/widgets/checkout/AdministrateWidgetCheckoutOrder.php');
$order = new AdministrateWidgetCheckoutOrder(
	$adm,
	array('order_id' => $_REQUEST['x_cust_id'])
);

// Set payment type to credit card.
$order->set_payment_type('C');

// Bail out if no order found.
if(!$order) {
	throw new Exception("Relay response sent against non-existent order.");
}

// Complete the order. 'use_sales_opportunities' is a string 'true' or 'false'
$order->complete(true, array(), $adm->get_option('use_sales_opportunities', 'checkout'));


// Show order completion page
if($order->get_event_id()) {
	$redirect_url =
		$_ADMINISTRATE->get_registration_url($order->get_event_id())
		. "&administrate_checkout_step=" . constant('ADM_CHECKOUT_STEP_PLACE')
		. "&administrate_checkout_order_id=" . $order->get_id();
} else {
	$redirect_url =
		$_ADMINISTRATE->get_registration_url(null, false, $order->get_track_id())
		. "&administrate_checkout_step=" . constant('ADM_CHECKOUT_STEP_PLACE')
		. "&administrate_checkout_order_id=" . $order->get_id();
}

// Usage of heredoc so that we can interpolate the URL's.
// This is returned to Authorize.NET who then present it to the user, redirecting them back to the clients site.
echo <<<HTML
	<html>
	<head>
		<script type="text/javascript">window.location.href="{$redirect_url}"</script>
	</head>
	<body>
		Thanks for your payment. <a href="{$redirect_url}">Click here</a> if you have not been automatically redirected.
	</body>
	</html>
HTML;

exit();
