<?php

include_once '../../ErrorHandler.php';
include_once '../../../weblink-components/config.php';
ErrorHandler::setupErrorHandling($_CONFIG['sentry_dsn']);

if(ErrorHandler::isEnabled()) {
	function error_handler($errno, $errstr, $errfile, $errline)
	{
		if($errno == E_ERROR || $errno == E_WARNING) {
			ErrorHandler::handleError($errno, $errstr, $errfile, $errline);
		}
		return false;
	}
	set_error_handler("error_handler");
}


try {
	$raw_post_data = file_get_contents('php://input');
	$raw_post_array = explode('&', $raw_post_data);
	$post_vars = array();
	foreach ($raw_post_array as $keyval) {
		$keyval = explode('=', $keyval);
		if (count($keyval) == 2)
			$post_vars[$keyval[0]] = urldecode($keyval[1]);
	}

	$req = 'cmd=_notify-validate';
	if (function_exists('get_magic_quotes_gpc')) {
		$get_magic_quotes_exists = true;
	}
	foreach ($post_vars as $key => $value) {
		if ($get_magic_quotes_exists == true && get_magic_quotes_gpc() == 1) {
			$value = urlencode(stripslashes($value));
		} else {
			$value = urlencode($value);
		}
		$req .= "&$key=$value";
	}

	$custom = unserialize(urldecode($post_vars['custom']));

	// we abuse the invoice variable to store whether this was a sandbox or live request
	if (array_key_exists('paypal_mode', $custom) && strtolower($custom['paypal_mode']) == 'sandbox') {
		$curl_url = 'https://www.sandbox.paypal.com/cgi-bin/webscr';
	} else {
		$curl_url = 'https://www.paypal.com/cgi-bin/webscr';
	}

	$ch = curl_init($curl_url);
	curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
	curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $req);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	curl_setopt($ch, CURLOPT_FORBID_REUSE, 1);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Connection: Close'));

	$res = curl_exec($ch);
	curl_close($ch);

	if (!($res)) {

		// If we can't get a response from PP to validate the IPN response then
		// just return a 500 so that PayPal is aware that an error has occurred.
		throw new Exception("Could not connect to $curl_url to validate IPN");
	}

	if (strcmp($res, "VERIFIED") == 0) {

		include_once('../../../weblink-components/options.php');
		include_once('../../administrate-common.php');
		include_once('../../../weblink-components/AdministrateWebLink.php');
		include_once('../../../weblink-components/AdministrateMysqlDB.php');
		include_once('../../../weblink-components/AdministrateDao.php');

		$wl = new AdministrateWebLink($_CONFIG, $_CONFIG['service']['debug'], false, $custom['api_key']);

		require_once('../../../weblink-components/widgets/checkout/AdministrateWidgetCheckoutOrder.php');

		$order = new AdministrateWidgetCheckoutOrder(
			$wl,
			array('order_id' => $post_vars['item_number']),
			false // Ignore API key when generating SELECT to grab this order
		);

		if (!$order->exists()) {

			// Order doesn't exist, throw a 404 back to PP.
			throw new Exception("IPN sent against non-existent order.");
		}

		// Complete the order. 'use_sales_opportunities' is a string 'true' or 'false'
		$order->complete(true, array(), $custom['use_sales_opportunities'] == 'true', $custom['api_key']);

	} else {

		// If we couldn't validate the IPN response then just return
		// a 500 so that PayPal is aware that an error has occurred.
		throw new Exception("IPN validation returned $res");
	}

} catch(Exception $e) {

	/*
	 * We want to catch exceptions and report them if we're on prod,
	 * otherwise just log them out to the console. This ensures that we don't
	 * return the ACTUAL 500's to PayPal, so that we don't risk them shutting
	 * down our clients integrations.
	 */
	if(ErrorHandler::isEnabled()) {
		ErrorHandler::handleException($e);
	} else {
		error_log($e->getMessage());
	}
}
