/*
	Event slider
*/
function administrate_event_slider($, dom_slider) {	
	var slider = $(dom_slider);
	
	// Setup styles and initialize
	//
	slider.css("overflow-x","hidden");
	slider.css("overflow-y","hidden");
	slider.find(".administrate-event-slider2-nav").css("display","table");
	slider.find("form").css("display","block");	
	
	var slider_content = slider.find(".administrate-event-slider2-content");
	var no_events_item = $("<li></li>");
	
	no_events_item.addClass("administrate-no-events-message");
	no_events_item.attr("data-event-currencies", "all");
	no_events_item.attr("data-event-default-currency", "all");
	no_events_item.css("text-align", "center");
	slider_content.append(no_events_item);
	slider_content.css("height", "100px");
	
	var slider_w         = 0;
	var slider_content_w = 0;
	
	var button_prev     = slider.find(".prev");
	var button_next     = slider.find(".next");
	var currency_select = slider.find(".administrate-currency-select");
	
	var event_positions = [];
	var event_index     = 0;
	
	var slider_offset        = 0;
	var slider_target_offset = 0;
	var slider_max_offset    = 0;
	var slider_released      = true;
	var slider_timer         = null;
	
	// Resize event handler
	//
	var scroll_resize = function() {
		slider_w         = slider.width();
		item_width       = 0;
		event_positions  = [];
		
		// Number of visible events
		var n_slots = 7;
		
		if( slider_w >= 1100 ) {
			n_slots = 5;
		}
		else if( slider_w >= 700 ) {
			n_slots = 4;
		}
		else if( slider_w >= 600 ) {
			n_slots = 3;
		}
		else if( slider_w >= 500 ) {
			n_slots = 2;
		}
		else {
			n_slots = 1;
		}
		
		item_width = (slider_w / n_slots) - 30;
		
		// Store positions for each event
		$.each( slider_content.find("li:visible"), function() {
			if( ! $(this).hasClass("administrate-no-events-message") ) {
				event_positions.push( $(this).position().left );
				$(this).find("p").css("width", item_width+"px");
			}
		});
		
		// No events message
		if( event_positions.length == 0 ) {
			no_events_item.html("No events to show");
			no_events_item.css("width", slider_w+"px");
		}
		else {
			no_events_item.html("");
			no_events_item.css("width", "0px");
		}
		
		slider_content_w  = slider_content.width();
		slider_max_offset = Math.max(slider_content_w-slider_w,0);
	}
	
	slider.show();
	scroll_resize();
	$(window).on( "resize", scroll_resize);
	
	// Start scrolling
	//
	var scroll_animate = function( scroll_direction ) {
		
		var next_event = function() {
			var next_exists = true;
			event_index += scroll_direction;
			
			if( event_index >= event_positions.length ) {
				event_index = event_positions.length-1;
				next_exists = false;
			}
			else if( event_index < 0 ) {
				event_index = 0;
				next_exists = false;
			}
				
			slider_target_offset = event_positions[ event_index ];
			
			if( slider_target_offset > slider_max_offset) {
				slider_target_offset = slider_max_offset;
				event_index -= scroll_direction;
			}
			
			return next_exists;
		};
		
		var scroll_stop = function() {
			if( slider_timer !== null ) {
				clearInterval( slider_timer );
				slider_timer = null;
			}
		};
		
		slider_released = false;
		scroll_stop();
		
		if( !next_event() ) {
			return;
		}
		if( slider_timer !== null ) {
			return;
		}
		
		slider_timer = setInterval( function() {
			slider_offset += scroll_direction*10;
			
			if( scroll_direction*slider_offset > scroll_direction*slider_target_offset ) {
				slider_offset = slider_target_offset;
				
				if( slider_released || !next_event() ) {
					scroll_stop();
				}
				
				if( slider_offset >= slider_max_offset ) {
					slider_offset = slider_max_offset;
					scroll_stop();
				}
			}
			slider_content.css( "left", (-1*slider_offset)+"px" );
		},15);
	};
	
	// Event handlers for previous/next buttons
	//
	button_prev.on( "mousedown", function() { scroll_animate(-1); });
	button_next.on( "mousedown", function() { scroll_animate(+1); });
	
	var scroll_release = function() { slider_released = true; }
	
	button_prev.on( "mouseup", scroll_release);
	button_prev.on( "mouseout", scroll_release);
	button_next.on( "mouseup", scroll_release);
	button_next.on( "mouseout", scroll_release);
	
	button_prev.on( "click", function(e) { e.preventDefault(); });
	button_next.on( "click", function(e) { e.preventDefault(); });
	
	// Currency change event handler
	//
	var currency_change = function() {
		var currency = currency_select.val();
		var ie9 = Boolean( navigator.userAgent.indexOf("MSIE 9")!=-1 );
		
		function fade_out(ready_fn) {
			if(ie9) {
				slider_content.hide();
				ready_fn();
			}
			else{
				slider_content.fadeOut(500, ready_fn);
			}
		}
		
		function fade_in(ready_fn) {
			if(ie9) {
				slider_content.show();
				ready_fn();
			}
			else{
				slider_content.fadeIn(500, ready_fn);
			}
		}
		
		fade_out( function() {
			$.each( slider_content.find("li"), function() {
				var currencies_list        = $(this).attr("data-event-currencies").split(",");
				var event_default_currency = $(this).attr("data-event-default-currency");
				var item = $(this);
				
				if( (currency=="all") || (currencies_list[0]=="all") || ($.inArray(currency, currencies_list) != -1) ){
					$(this).show();
				}
				else {
					$(this).hide();
				}
				
				// Show active price
				$(this).find(".administrate-price").each(function() {
					var price = $(this);
					var price_currency = price.attr("data-event-currency");
					
					if( 
						(price_currency == "all") || 
						( (currency == "all") && (price_currency == event_default_currency) ) ||
						(price_currency == currency)
					) {
						price.show();
						
						if( price_currency != "all") {
							var link = item.find(".administrate-event-table-register-link");
							var href = link.attr("href");
							if(href) {
								var href_currency = href.replace(/event_currency=[A-Z]+$/, "event_currency=" + price_currency );
								link.attr("href", href_currency);
							}
						}
					}
					else {
						price.hide();
					}
				});
			});
			
			scroll_release();
			slider_offset = 0;
			event_index = 0;
			next_exists = true;
			slider_content.css( "left", "0px" );
			
			fade_in( function() { scroll_resize(); } );
		});
	};
	
	currency_select.on("change", currency_change);
	
	// Show sessions container
	//
	var session_links = slider_content.find(".administrate-session-link");
	
	session_links.on( "mouseover", function(e) {
		var widget    = $(this).closest(".administrate-event-slider-out");
		var event_id  = $(this).closest("li").attr("data-event-id");
		var container = $( "#sessions-" + event_id );
		
		var widget_pos = widget.offset();
		var pos = {left:e.pageX-widget_pos.left, top:e.pageY-widget_pos.top};
		
		var container_w = container.width();
		var container_h = container.height();
		
		container.css( {"left": (pos.left+20) + "px", "top": (pos.top-50) + "px"} );
		container.show();
	});
	
	session_links.on( "mouseout", function(e) {
		var event_id = $(this).closest("li").attr("data-event-id");
		var container = $( "#sessions-" + event_id );
		container.fadeOut("fast");
	});
}
