<?php
// Event Session Test
//
use PHPUnit\Framework\TestCase;
require_once('common/api/objects/AdministrateAPIObject.php');
require_once('common/api/objects/AdministrateAPIEventSession.php');

class AdministrateAPIEventSessionTest extends TestCase {
	
	private function mock_api_data() {
		$mock = new stdClass();
		$mock->SessionID = 5;
		$mock->Title     = "Test Session";
		$mock->StartDate = mktime(0,0,0,1,2,2015);
		$mock->EndDate   = mktime(0,0,0,3,4,2016);
		$mock->StartTime = "10:00:00";
		$mock->EndTime   = "11:10:20";
		return $mock;
	}
	
	public function test_title() {
		$test_session = new AdministrateAPIEventSession( $this->mock_api_data() );
		$this->assertEquals( "Test Session", $test_session->get_title() );
	}
	
	public function test_dates() {
		$test_session = new AdministrateAPIEventSession( $this->mock_api_data() );
		
		$this->assertTrue( $test_session->is_multi_day() );
		$this->assertTrue( $test_session->has_times() );
		$this->assertTrue( $test_session->has_all_times() );
		$this->assertEquals( mktime(0,0,0,1,2,2015), $test_session->get_start_date() );
		$this->assertEquals( mktime(0,0,0,3,4,2016), $test_session->get_end_date() );
	}
	
	public function test_times() {
		$test_session = new AdministrateAPIEventSession( $this->mock_api_data() );
		
		$this->assertTrue( $test_session->has_times() );
		$this->assertTrue( $test_session->has_all_times() );
		
		$this->assertEquals( "10:00:00", $test_session->get_start_time() );
		$this->assertEquals( "11:10:20", $test_session->get_end_time() );
	}
	
	private function mock_api_data_without_times() {
		$mock = new stdClass();
		$mock->SessionID = 5;
		$mock->Title     = 'Happy Session';
		$mock->StartDate = mktime(0,0,0,1,2,2015);
		$mock->EndDate   = mktime(0,0,0,3,4,2016);
		$mock->StartTime = null;
		$mock->EndTime   = null;
		return $mock;
	}
	
	public function test_without_times() {
		$test_session = new AdministrateAPIEventSession( $this->mock_api_data_without_times() );
		$this->assertTrue( $test_session->is_multi_day() );
		$this->assertFalse( $test_session->has_times() );
		$this->assertFalse( $test_session->has_all_times() );
		$this->assertFalse( $test_session->get_start_time() );
		$this->assertFalse( $test_session->get_end_time() );
	}
	
	public function test_with_start_time() {
		$mock_data = $this->mock_api_data_without_times();
		$mock_data->StartTime = "10:15";
		$test_session = new AdministrateAPIEventSession( $mock_data );
		
		$this->assertTrue( $test_session->has_times() );
		$this->assertFalse( $test_session->has_all_times() );
		$this->assertEquals( "10:15", $test_session->get_start_time() );
		$this->assertFalse( $test_session->get_end_time() );
	}
}
