<?php
// Date/Time formatter test
//
use PHPUnit\Framework\TestCase;
require_once('common/AdministrateDateTime.php');

class AdministrateDateTimeTest extends TestCase {
	
	public function test_date_mdy() {
		$t = AdministrateDateTime::date( mktime(0,0,0,1,2,2015), "mdy", true, true );
		$this->assertEquals( "Jan 02, 2015", $t );
		
		$t = AdministrateDateTime::date( mktime(0,0,0,1,2,2015), "mdy", true, false );
		$this->assertEquals( "Jan 02", $t );
		
		$t = AdministrateDateTime::date( mktime(0,0,0,1,2,2015), "mdy", false, false );
		$this->assertEquals( "02", $t );
	}
	
	public function test_date_dmy() {
		$t = AdministrateDateTime::date( mktime(0,0,0,1,2,2015), "dmy", true, true );
		$this->assertEquals( "02 Jan 2015", $t );
		
		$t = AdministrateDateTime::date( mktime(0,0,0,1,2,2015), "dmy", true, false );
		$this->assertEquals( "02 Jan", $t );
		
		$t = AdministrateDateTime::date( mktime(0,0,0,1,2,2015), "dmy", false, false );
		$this->assertEquals( "02", $t );
	}
	
	public function test_date_range_mdy() {
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,1,2,2015) );
		$t = AdministrateDateTime::date_range( $d, "mdy", true, true );
		$this->assertEquals( "Jan 02, 2015", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,1,20,2015) );
		$t = AdministrateDateTime::date_range( $d, "mdy", true, true );
		$this->assertEquals( "Jan 02 &ndash; 20, 2015", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,2,20,2015) );
		$t = AdministrateDateTime::date_range( $d, "mdy", true, true );
		$this->assertEquals( "Jan 02 &ndash; Feb 20, 2015", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,2,20,2016) );
		$t = AdministrateDateTime::date_range( $d, "mdy", true, true );
		$this->assertEquals( "Jan 02, 2015 &ndash; Feb 20, 2016", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,1,2,2016) );
		$t = AdministrateDateTime::date_range( $d, "mdy", true, true );
		$this->assertEquals( "Jan 02, 2015 &ndash; Jan 02, 2016", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,1,20,2016) );
		$t = AdministrateDateTime::date_range( $d, "mdy", true, true );
		$this->assertEquals( "Jan 02, 2015 &ndash; Jan 20, 2016", $t );
	}
	
	public function test_date_range_dmy() {
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,1,2,2015) );
		$t = AdministrateDateTime::date_range( $d, "dmy", true, true );
		$this->assertEquals( "02 Jan 2015", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,1,20,2015) );
		$t = AdministrateDateTime::date_range( $d, "dmy", true, true );
		$this->assertEquals( "02 &ndash; 20 Jan 2015", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,2,20,2015) );
		$t = AdministrateDateTime::date_range( $d, "dmy", true, true );
		$this->assertEquals( "02 Jan &ndash; 20 Feb 2015", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,2,20,2016) );
		$t = AdministrateDateTime::date_range( $d, "dmy", true, true );
		$this->assertEquals( "02 Jan 2015 &ndash; 20 Feb 2016", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,1,2,2016) );
		$t = AdministrateDateTime::date_range( $d, "dmy", true, true );
		$this->assertEquals( "02 Jan 2015 &ndash; 02 Jan 2016", $t );
		
		$d = array( 'start' => mktime(0,0,0,1,2,2015), 'end' => mktime(0,0,0,1,20,2016) );
		$t = AdministrateDateTime::date_range( $d, "dmy", true, true );
		$this->assertEquals( "02 Jan 2015 &ndash; 20 Jan 2016", $t );
	}
	
	public function test_time() {
		$t = AdministrateDateTime::time( "20:15:00", false );
		$this->assertEquals( "8:15PM", $t );
		
		$t = AdministrateDateTime::time( null, false );
		$this->assertEquals( "&ndash;", $t );
		
		$t = AdministrateDateTime::time( false, false );
		$this->assertEquals( "&ndash;", $t );
	}
	
	public function test_time_24h() {
		$t = AdministrateDateTime::time( "20:15:00", true );
		$this->assertEquals( "20:15h", $t );
		
		$t = AdministrateDateTime::time( null, true);
		$this->assertEquals( "&ndash;", $t );
		
		$t = AdministrateDateTime::time( false, true );
		$this->assertEquals( "&ndash;", $t );
	}
	
	public function test_time_range() {
		$d = array( 'start' => "10:01:02", 'end' => "10:01:02" );
		$t = AdministrateDateTime::time_range( $d, false );
		$this->assertEquals( "10:01AM", $t );
		
		$d = array( 'start' => "10:01:02", 'end' => null );
		$t = AdministrateDateTime::time_range( $d, false );
		$this->assertEquals( "10:01AM", $t );
		
		$d = array( 'start' => null, 'end' => "10:01:02" );
		$t = AdministrateDateTime::time_range( $d, false );
		$this->assertEquals( "&ndash;", $t );
		
		$d = array( 'start' => null, 'end' => null );
		$t = AdministrateDateTime::time_range( $d, false );
		$this->assertEquals( "&ndash;", $t );
		
		$d = array( 'start' => "10:01:02", 'end' => "15:02:02" );
		$t = AdministrateDateTime::time_range( $d, false );
		$this->assertEquals( "10:01AM &ndash; 3:02PM", $t );
	}
}
