<?php
use PHPUnit\Framework\TestCase;
require_once('common/api/MappedItemCalculator.php');

class MappedItemTaxCalculatorTest extends TestCase {

	protected function setUp() {
		$this->calculator = new MappedItemCalculator();
	}

	public function test_mapped_item_of_100_and_20_tax_returns_20() {
		$mock_item = array(
			'subtotal' => 100,
		);
		$tax_rate = 1.2;

		$expected_tax = 20;
		$this->assertEquals($expected_tax, $this->calculator->calculate_tax($mock_item, $tax_rate));
	}

	public function test_mapped_item_of_200_and_50_tax_returns_100() {
		$mock_item = array(
			'subtotal' => 200,
		);
		$tax_rate = 1.5;

		$expected_tax = 100;
		$this->assertEquals($expected_tax, $this->calculator->calculate_tax($mock_item, $tax_rate));
	}

	public function test_mapped_item_with_is_taxable_false_returns_0_for_tax_always() {
		$mock_item = array(
			'subtotal' => 200,
			'is_taxable' => False
		);
		$tax_rate = 1.5;
		$this->assertEquals(0, $this->calculator->calculate_tax($mock_item, $tax_rate));
	}

	public function test_mapped_item_with_50_percent_discount_takes_into_account_that_discount_when_calculating_tax() {
		$mock_item = array(
			'subtotal' => 200,
			'is_discountable' => True,
			'discount' => 100,
			'is_taxable' => True
		);
		$tax_rate = 1.5;
		$expected_tax = 50;
		$this->assertEquals($expected_tax, $this->calculator->calculate_tax($mock_item, $tax_rate));
	}

}
