<?php
//  MySQL database acces
//
class AdministrateMysqlDB extends AdministrateDB {
	private $_connection = false;
	private $_last_result = 0;
	
	public function __construct($db_name, $user, $password, $table_prefix = "", $host = "localhost", $port = false) {
		// Construct parent
		parent::__construct($table_prefix);
		
		// Set database information
		$this->db_name = $db_name;
		$this->user = $user;
		$this->password = $password;
		
		$this->host = $host;
		$this->port = $port;
		
		// Format host url
		$db_url = $this->host;
			
		if( $this->port ) {
			$db_url .= ":".$this->port;
		}
		
		// Connect
		$this->_connection = new mysqli($db_url, $this->user, $this->password, $this->db_name) or $this->_warn();
	}

	//  Executes a query
	public function query($qry) {
		if ($this->_connection->connect_error) {
			return false;
		}
		
		$qry = trim($qry);
		$this->_set_last_query($qry);
		$this->_last_result = $this->_connection->query($qry);
		
		if(!$this->_last_result) {
			$this->_warn();
		}
		
		return $this->_last_result;
	}
	
	//  Executes a query and format result set
	public function query_fetch($qry) {
		$mysqli_result = $this->query($qry);
		
		if(!$mysqli_result) {
			$this->_warn();
			return false;
		}
		
		$rows = $mysqli_result->fetch_all(MYSQLI_ASSOC);
		
		if( empty($rows) ) {
			return false;
		}
		
		return count($rows)==1 ? $rows[0] : $rows;
	}
	
	//  Prepare statement
	public function prepare($qry, $param_types = "") {
		if ($this->_connection->connect_error) {
			return false;
		}
		
		$mysqli_pepared = $this->_connection->prepare($qry);
		
		if(!$this->_connection->prepare($qry)) {
			$this->_warn();
			return false;
		}
		return array($qry, $param_types, $mysqli_pepared);
	}
	
	//  Execute prepared
	public function prepared_exec($prep_statement, $params = array()) {
		if ($this->_connection->connect_error) {
			return false;
		}
		
		$qry = $prep_statement[0];
		$param_types = $prep_statement[1];
		$mysqli_pepared = $prep_statement[2];
		
		$this->_set_last_query($qry);
		//$this->_log("prepared_exec");
		//$this->_log("prepared_exec, param_types=".$param_types.", params=". var_export($params, true));

		$args_array = array( $param_types );
		
		for($i=0; $i<count($params); $i++) {
			$args_array[] = &$params[$i];
		}
		
		call_user_func_array(array($mysqli_pepared, "bind_param"), $args_array);
		$status = $mysqli_pepared->execute();
		$this->_last_result = $mysqli_pepared->get_result();
		return $status;
	}
	
	//  Execute prepared and get result set
	public function prepared_fetch($prep_statement, $params = array()) {
		$this->prepared_exec($prep_statement, $params);
		
		$mysqli_pepared = $prep_statement[2];
		
		if(!$this->_last_result) {
			$this->_warn();
			return false;
		}
		
		$rows = $this->_last_result->fetch_all(MYSQLI_ASSOC);
		
		if( empty($rows) ) {
			return false;
		}
		
		return count($rows)==1 ? $rows[0] : $rows;
	}
	
	//  Gets the number of records returned by a query
	public function get_last_num_records() {
		if(!is_object($this->_last_result)) {
			$this->_warn("get_last_num_records, result=".var_export($this->_last_result,true));
			return 0;
		}
		return $this->_last_result->num_rows;
	}
	
	// Get last insert id
	public function get_last_insert_id() {
		return $this->_connection->insert_id;
	}
	
	// Get last error
	public function get_last_error() {
		if ($this->_connection->connect_error) {
			return $this->_connection->connect_error;
		}
		return $this->_connection->error;
	}
	
	/*
	
	Value Formatters
	
	*/
	
	public function format_string($val, $strip_slashes=true) {
		return $this->_connection->real_escape_string($strip_slashes ? stripslashes($val) : $val);
	}
	
	public function format_date($val) {
		return date('Y-m-d', $val);
	}
	
}
