<?php
//  Include required classes
require_once('AdministrateMysqlDB.php');

//  Administrate Service controller class
class AdministrateWebLink extends Administrate {
	
	//  Properties
	protected $keyDelimiter = "_";
	public $db = false;
	protected $dbPrefix = 'wl_';
	protected $path = false;
	protected $url = false;
	public $debug = false;
	protected $config = array();
	protected $options = array();
	protected $optionValues = array();
	protected $transferableSubfieldProperties = array(
		'required',
		'max_length',
		'default',
		'hint'
	);
	protected $tableMaps = array();
	protected $jsEnabled = false;
	protected $controller;
	protected $service;
	
	//  Properties
	protected $namespace = "administrate";
	protected $tables = array(
		
		//  Orders table
		'orders' => array(
			'order_id' => array(
				'type'		=>	'INT',
				'length'	=>	11,
				'null'		=>	false
			),
			'order_status' => array(
				'type'		=>	'ENUM',
				'options'	=>	array('I', 'P', 'C'), // I = incomplete, P = pending (invoiced), C = complete
				'null'		=>	false,
				'default'	=>	'I'
			),
			'order_payment_type' => array(
				'type'		=>	'ENUM',
				'options'	=>	array('I', 'C'), // I = invoice, C = credit card
				'null'		=>	false,
				'default'	=>	'I'
			),
			'order_time_started' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false
			),
			'order_time_completed' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false,
				'default'	=>	0
			),
			'order_event_id' => array(
				'type'		=>	'INT',
				'length'	=>	11,
				'null'		=>	true
			),
			'order_track_id' => array(
				'type'		=>	'INT',
				'length'	=>	11,
				'null'		=>	true
			),
			'order_item_ids'    => array(
				'type'      => 'VARCHAR',
				'length'    => '255',
				'null'      => true
			),
			'order_session_id' => array(
				'type'		=>	'CHAR',
				'length'	=>	32,
				'null'		=>	false
			),
			'order_max_step' => array(
				'type'		=>	'TINYINT',
				'length'	=>	1,
				'null'		=>	false,
				'default'	=>	1
			),
			'order_currency' => array(
				'type'		=>	'CHAR',
				'length'	=>	3,
				'null'		=>	false,
				'default'	=>	'GBP'
			),
			'order_discount' => array(
				'type'		=>	'TEXT',
				'null'		=>	true
			),
			'order_buyer_details' => array(
				'type'		=>	'TEXT',
				'null'		=>	true,
				'serialize'	=>	array(
					'first_name',
					'last_name',
					'company',
					'email',
					'phone'
				)
			),
			'order_invoice_address' => array(
				'type'		=>	'TEXT',
				'null'		=>	true,
				'serialize'	=>	array(
					'address',
					'city',
					'territory',
					'postal_code',
					'country'
				)
			),
			'order_num_attendees' => array(
				'type'		=>	'TINYINT',
				'length'	=>	1,
				'null'		=>	false,
				'default'	=>	1
			),
			'order_attendee_details' => array(
				'type'		=>	'TEXT',
				'null'		=>	true,
				'serialize'	=>	array(
					'first_name',
					'last_name',
					'email'
				),
				'multiple'	=>	true
			),
			'order_processor_transaction_id' => array(
				'type'		=>	'VARCHAR',
				'length'	=>	64,
				'null'		=>	true
			),
			'order_api_invoice_id' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false,
				'default'	=>	0
			),
			'order_path_objectives' => array(
				'type'		=>	'TEXT',
				'null'		=>	true
			)
		),
		
		//  Logs table
		'logs' => array(
			'log_id' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false
			),
			'log_msg' => array(
				'type'		=>	'TEXT',
				'null'		=>	false
			),
			'log_time' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false
			),
			'log_order_id' => array(
				'type'		=>	'INT',
				'length'	=>	11,
				'null'		=>	false,
				'default'	=>	0
			)
		)
	);
	protected $subdomainPlaceholder = '<subdomain>';
	
	//  Constructor
	public function __construct($config, $debug = false, $routing = true, $api_key = null) {
		$this->config = $config;
		$this->debug = $debug;
		Administrate::log()->set_debug($this->debug);

		$this->db = new AdministrateMysqlDB(
			$this->config['service']['db']['name'],
			$this->config['service']['db']['user'],
			$this->config['service']['db']['password'],
			'',
			$this->config['service']['db']['host']
		);
		
		//  Set table names
		foreach ($this->tables as $key=>$fields) {
			$this->tableMaps[$key] = $this->dbPrefix . $this->add_namespace($key);	
		}
		
		//  Load and initialize the API
		$this->init_api($api_key);

		if($routing == false) {
			return;
		}

		//  Strip the trailing slash in URI if it exists
		$uri = trim($_SERVER['REQUEST_URI']);
		if (isset($_SERVER['QUERY_STRING']) && !empty($_SERVER['QUERY_STRING'])) {
			$uri = substr($uri, 0, (strlen($_SERVER['QUERY_STRING'])+1)*-1);	
		}
		if (substr($uri, -1) != '/') {
			$uri .= '/';	
		}
		
		//  Parse out the controller and action
		$dirs = explode('/', $uri);
		$controller = $dirs[count($dirs)-3];
		$action = $dirs[count($dirs)-2];
		
		//  Save the service URL
		$this->url = substr($uri, 0, (strlen($controller) + strlen($action) + 2) * -1);

		$use_https = (
			array_key_exists('HTTPS',$_SERVER)
			|| (array_key_exists('HTTP_X_FORWARDED_PROTO', $_SERVER) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https')
		);

		$this->absUrl = 'http' . ($use_https ? 's' : '') . '://' . $_SERVER['SERVER_NAME'] . $this->url;
		
		//  Include the controller
		require_once('AdministrateController.php');
		$this->controller = new AdministrateWebLinkController($this);
		
		//  Decode GET arguments
		$decoded_params = array();
		foreach( $_GET as $key => $value ) {
			// Some array parameters are merged into GET[], skip decoding for them
			$decoded_params[$key] = is_array($value) ? $value : urldecode($value);
		}

		//  Output the result
		echo $this->run($action, $decoded_params);

	}
	
	//  Display the price widget
	public function display_price_widget($event, $fieldName, $defaultCurrency = '', $showCurrencyIndicator = true, $currencyIndicator = 'symbol') {
		$this->controller->display_price_widget($event, $fieldName, $defaultCurrency, $showCurrencyIndicator);
	}
	
	//  Display the currency selector
	public function display_currency_selector($events, $defaultCurrency = '', $fieldName = '') {
		$this->controller->display_currency_selector($events, $defaultCurrency, $fieldName);	
	}
	
	//  Display the currency prices
	public function display_currency_prices($event, $pricingBasis = false, $defaultCurrency = '', $showCurrencyIndicator = true, $currencyIndicator = 'symbol') {
		$this->controller->display_currency_prices($event, $pricingBasis, $defaultCurrency, $showCurrencyIndicator, $currencyIndicator);
	}
	
	//  Get currency symbol
	public function get_currency_symbol($currency) {
		return $this->controller->get_currency_symbol($currency);	
	}
	
	//  Format a currency
	public function format_currency($amount, $currency, $showCurrencyIndicator = true, $currencyIndicator = 'symbol') {
		return $this->controller->format_currency($amount, $currency, $showCurrencyIndicator, $currencyIndicator);	
	}
	
	//  Format a datetime
	public function format_datetime($ts, $format = 'M j @ g:ia') {
		return date($format, $ts);
	}
	
	//  Get the registration URL of an event
	public function get_registration_url($eventId=0, $currency=false, $track_id=null) {
		return '#';
	}
	
	//  Get the course url structure
	public function get_course_url_structure() {
		if (!property_exists($this, 'courseUrlStructure')) {
			$this->courseUrlStructure = $this->get_option('course_url_structure');
		}
		return $this->courseUrlStructure;	
	}
	
	//  Get the course URL structure parts
	public function get_course_url_parts() {
		if (!property_exists($this, 'courseUrlParts')) {
			$this->courseUrlParts = explode('-', $this->get_course_url_structure());
		}
		return $this->courseUrlParts;
	}
	
	//  Get the course page
	public function get_course_page() {
		if (!property_exists($this, 'coursePage')) {
			$this->coursePage = $this->get_option('course_page');
		}
		return $this->coursePage;
	}
	
	//  Get the course page URL
	public function get_course_page_base_url() {
		if (!property_exists($this, 'coursePageBaseUrl')) {
			$this->coursePageBaseUrl = substr(get_permalink($this->get_course_page()), strlen(site_url()));
		}
		return $this->coursePageBaseUrl;
	}
	
	//  Whether or not to use pretty course URLs
	public function use_custom_course_urls() {
		if (!property_exists($this, 'customCourseUrls')) {
			$this->customCourseUrls = ((strlen($this->get_course_url_structure()) > 0) && (strlen(get_option('permalink_structure')) > 0));
		}
		return $this->customCourseUrls;	
	}
	
	//  Get the info URL of a course
	public function get_course_url($course, $subcategory = false, $category = false, $forceGenerate = false) {
		return '#';
	}
	
	//  Get the info URL of a subcategory
	public function get_subcategory_url($subcategory, $category = false, $forceGenerate = false) {
		
		//  If there is a custom structure, generate it
		if ($this->use_custom_course_urls()) {
		
			//  If the subcategory object wasn't passed in, query for it
			if (!is_object($subcategory)) {
				$subcategory = Administrate::api()->get_subcategory($subcategory);		
			}
			
			//  See if there is already a saved filter path
			require_once($this->get_path('/AdministrateCourseFilter.php'));
			$filter = new AdministrateCourseFilter(
				$this, 
				array(
					'filter_object_type'	=>	'subcategory',
					'filter_object_id'		=>	$subcategory->get_id(),
					'filter_hidden'			=>	0
				)
			);
			
			//  If the filter exists, return the cached path
			if ($filter->exists() && !$forceGenerate) {
				return $filter->get_object_path();
				
			//  Otherwise continue to generate it
			} else {
				
				//  Set the params
				$params = array();
				if ($category) {
					$params['category'] = $category;
				} else if (isset($_REQUEST[$this->add_namespace(array('category', 'id'))])) {
					$params['category'] = Administrate::api()->get_category($_REQUEST[$this->add_namespace(array('category', 'id'))]);
				} else {
					$params['category']	= $subcategory->get_parent_category();
				}
				$params['subcategory'] = $subcategory;
				
				//  Return the subcategory URL
				return $this->get_course_page_url($params);

			}
		
		//  Otherwise just pass the course ID
		} else {
			return $this->get_course_page_url(array('subcategory'=>$subcategory));
		}
	
	}
	
	//  Get the info URL of a category
	public function get_category_url($category, $forceGenerate = false) {
	
		//  If there is a custom structure, generate it
		if ($this->use_custom_course_urls()) {
		
			//  If the category isn't an object, query for it
			if (!is_object($category)) {
				$category = Administrate::api()->get_category($category);		
			}
		
			//  See if there is already a saved filter path
			require_once($this->get_path('/AdministrateCourseFilter.php'));
			$filter = new AdministrateCourseFilter(
				$this, 
				array(
					'filter_object_type'	=>	'category',
					'filter_object_id'		=>	$category->get_id(),
					'filter_hidden'			=>	0
				)
			);
			
			//  If the filter exists, return the cached path
			if ($filter->exists() && !$forceGenerate) {
				return $filter->get_object_path();
			
			//  Or else generate the URL
			} else {
				return $this->get_course_page_url(array('category'=>$category));
			}
		
		//  Or else just return the normal URL
		} else {
			return $this->get_course_page_url(array('category'=>$category));	
		}
	
	}
	
	//  Get a course page URL
	public function get_course_page_url($params = array()) {
		
		//  Set the base URL
		$url = $this->get_course_page_base_url();
		
		//  If there is a custom structure, generate it
		if ($this->use_custom_course_urls()) {
		
			//  Include the course filter class
			require_once($this->get_path('/AdministrateCourseFilter.php'));
			
			//  Split the structure into parts
			$urlStructureParts = $this->get_course_url_parts();
			
			//  Loop through URL structure backwards to find the first identifier available
			$objectSuffix = '';
			for ($i = count($urlStructureParts)-1; $i >= 0; --$i) {	
				
				//  If the part matches the passed key, generate the URL from here
				if (isset($params[$urlStructureParts[$i]])) {
					
					//  Get the associated filter
					if ($urlStructureParts[$i] == 'course') {
						$objectId = $params[$urlStructureParts[$i]]->get_code();
					} else {
						$objectId = $params[$urlStructureParts[$i]]->get_id();
					}
					$filter = new AdministrateCourseFilter(
						$this, 
						array(
							'filter_object_type'	=>	$urlStructureParts[$i], 
							'filter_object_id'		=>	$objectId, 
							'filter_hidden'			=>	0
						)
					); 
					
					if ($filter->exists()) {
						$objectSuffix = $filter->get_url_string() . '/' . $objectSuffix;
					}
					
				} 
				
			}
			
			//  Add the suffix to the URL
			$url .= $objectSuffix;
		
		//  Otherwise just add the query string
		} else {
			
			//  Add the appropriate query string prefix
			if (strpos($url, '?') > -1) {
				$url .= '&';	
			} else {
				$url .= '?';
			}
			
			//  Loop through params and add them to the query string		
			foreach ($params as $key=>$val) {
				if ($key == 'course') {
					$objectId = $params[$key]->get_code();
				} else {
					$objectId = $params[$key]->get_id();
				}
				$url .= $this->add_namespace(array($key, 'id')) . '=' . $objectId;
			}
		
		}
		
		//  Return the generated URL
		return $this->get_site_url($url);
		
	}
	
	//  Get event field values based on passed list (array) or field names
	public function get_event_fields($event, $fields) {
		
		//  If an array of fields wasn't passed, try unserializing it
		if (!is_array($fields)) {
			$fields = explode(',',$fields);
		}
		if (empty($fields)) {
			$fields = array();
		}
		
		//  Add the requested fields to the values  
		$values = array();
		if (in_array('code', $fields)) {
			$values['code'] = $event->get_course_code();
		}
		if (in_array('summary', $fields)) {
			$values['summary'] = $event->get_course_summary();
		}
		if (in_array('schedule', $fields)) {
			$values['schedule'] = $event->get_course_schedule();
		}
		if (in_array('location', $fields)) {
			$values['location'] = $event->get_location();
		}
		if (in_array('inclusions', $fields)) {
			$values['inclusions'] = $event->get_course_inclusions();
		}
		if (in_array('custom_text_8', $fields)) {
			$values['custom_text_8'] = $event->get_course_custom_text_8();
		}
		if (in_array('method', $fields)) {
			$values['method'] = $event->get_course_method();
		}
		if (in_array('prerequisites', $fields)) {
			$values['prerequisites'] = $event->get_course_prerequisites();
		}
		if (in_array('topics', $fields)) {
			$values['topics'] = $event->get_course_topics();
		}
		if (in_array('benefits', $fields)) {
			$values['benefits'] = $event->get_course_benefits();
		}
		if (in_array('duration', $fields)) {
			$values['duration'] = $event->get_course_duration();
		}
		
		//  Return the values
		return $values;
		
	}
	
	//  Get course field values based on passed list (array) or field names
	public function get_course_fields(&$course, $fields) {
		
		//  If an array of fields wasn't passed, try unserializing it
		if (!is_array($fields)) {
			$fields = unserialize(base64_decode($fields));	
		}
		if (empty($fields)) {
			$fields = array();	
		}
		
		//  Add the requested fields to the values  
		$values = array();
		if (in_array('code', $fields)) {
			$values['code'] = $course->get_code();	
		}
		if (in_array('summary', $fields)) {
			$values['summary'] = $course->get_summary();	
		}
		if (in_array('schedule', $fields)) {
			$values['schedule'] = $course->get_schedule();	
		}
		if (in_array('inclusions', $fields)) {
			$values['inclusions'] = $course->get_inclusions();
		}
		if (in_array('method', $fields)) {
			$values['method'] = $course->get_method();	
		}
		if (in_array('prerequisites', $fields)) {
			$values['prerequisites'] = $course->get_prerequisites();	
		}
		if (in_array('topics', $fields)) {
			$values['topics'] = $course->get_topics();	
		}
		if (in_array('benefits', $fields)) {
			$values['benefits'] = $course->get_benefits();	
		}
		if (in_array('custom_text_8', $fields)) {
			$values['custom_text_8'] = $course->get_custom_text_8();
		}
		if (in_array('duration', $fields)) {
			$values['duration'] = $course->get_duration();	
		}
		
		//  Return the values
		return $values;
		
	}
	
	//  Get the number of places available for an event
	public function get_event_num_places(&$event) {
		return Administrate::api()->get_num_places($event);
	}
	
	//  Get API URL
	public function get_api_url($mode, $domain) {
		$serviceConfig = $this->get_config('service');
		//return str_replace($this->subdomainPlaceholder, $subdomain, $serviceConfig['urls'][$mode]);
		return str_replace($this->domainPlaceholder, $domain, $serviceConfig['urls'][$mode]);
	}
	
	//  Initialize the API
	public function init_api($api_key = null) {

		// WebLink API key
		if(is_null($api_key)) {
			$weblink_api_key = $this->db->format_string($this->get_option('api_key'));
		} else {
			$weblink_api_key = $api_key;
		}

		//  Get API credentials form database
		$query = "
			SELECT rest_url,soap_url,username,password,UNIX_TIMESTAMP(last_cache_sweep) as lcs
			FROM weblink
			WHERE api_key = '{$weblink_api_key}'
		";
		$record = $this->db->query_fetch($query);		

		// Set API credentials
		$api_credentials = new AdministrateCredentials(
			$record['rest_url'],
			$record['soap_url'],
			$record['username'], $record['password'],
			$this->debug,
			str_replace('api/v2', 'beta/api', $record['rest_url'])
		);
		Administrate::api()->set_credentials($api_credentials);

		// Create cache and perform cache sweep
		//
		$api_cache = new AdministrateCache($this->db, "", $weblink_api_key);
		Administrate::api()->set_cache($api_cache);
		
		if( $api_cache->delete_expired($record['lcs']) ) {
			$query = "
				UPDATE weblink
				SET last_cache_sweep = NOW()
				WHERE api_key = '{$weblink_api_key}'
			";
			$record = $this->db->query($query);	
		}
		
	}
	
	//  Get the orders table
	public function get_orders_table() {
		return $this->get_table('orders');	
	}
	
	//  Get the logs table
	public function get_logs_table() {
		return $this->get_table('logs');	
	}
	
	//  Get a table
	public function get_table($key) {
		return $this->tableMaps[$key];
	}
	
	//  Get the table field definitions
	public function get_table_fields($key) {
		return $this->tables[$key];	
	}
	
	//  Filter text output
	public function filter_text_output($str) {
		$str = trim($str);
		if ($str != strip_tags($str)) {	
			return $str;
		} else {
			return nl2br($str);
		}
	}
	
	//  Encode base64/serialize a string
	protected function _encode($str) {
		return base64_encode(serialize($str));
	}
	
	//  Decode base64/unserialize a string
	protected function _decode($str) {
		return unserialize(base64_decode($str));	
	}

	/**
	 * Returns the required $text in the language required.
	 *
	 * @param $text string - The text to be translated.
	 * @return string - The translated text.
	 */
	public function get_text($text) {
		return $text;
	}

	/**
	 * Echoes out the required $text in the language required.
	 *
	 * @param $text string - The text to be translated.
	 * @return null
	 */
	public function echo_text($text) {
		echo $text;
	}

	//  Get an option
	public function get_option($name, $group=null, $sub_group=null) {
		if( !array_key_exists($name, $this->config['options']) ) {
			return '';
		}
		return $this->config['options'][$name];
	}

	/**
	 * Gets an option as a boolean. Useful in weblink where params may be scraped as strings.
	 *
	 * @param $name string - The option to lookup.
	 * @param null $group string - (optional) The group of the option.
	 * @param null $sub_group - (optional) The sub group of the option.
	 * @return bool
	 */
	public function get_bool_option($name, $group=null, $sub_group=null) {
		return $this->str_to_boolean(
			$this->get_option(
				$name,
				$group,
				$sub_group
			)
		);
	}

	//  Get integer option
	public function get_int_option($name, $group=null, $sub_group=null) {
		return intval( $this->get_option($name, $group, $sub_group) );
	}
	
	//  Get a group of options
	public function get_options($group) {
		return $this->config['options'][$group];	
	}
	
	//  Get config
	public function get_config($key) {
		return $this->config[$key];	
	}
	
	//  Get the service path
	public function get_path($path) {
		return $this->config['service']['path'] . $path;
	}
	
	//  Get the service URL
	public function get_url($path) {
		return $this->url . $path;
	}
	
	//  Get the absolute URL
	public function get_abs_url($path) {
		
		//  Parse the URL
		$parts = parse_url($path);
		
		//  Parse the params
		$params = array();
		if (array_key_exists('query', $parts)) {
			parse_str($parts['query'], $params);
		}
		
		//  Merge the params with existing $_GET params
		$params = array_merge($_GET, $params);
		
		//  Generate the URL and return it
		$url = $this->absUrl . $parts['path'];
		if (!empty($params)) {
			$url .= '?' . http_build_query($params);
		}

		return $url;
		
	}
	
	//  Add the plugin prefix
	public function add_namespace($str, $delimiter = false) {
		if (!is_array($str)) {
			$str = array($str);	
		}
		if (!$delimiter) {
			$delimiter = $this->get_key_delimiter();	
		}
		$newStr = $this->get_namespace();
		foreach ($str as $tmpStr) {
			$newStr .= $delimiter . $tmpStr;	
		}
		if ($delimiter) {
			$newStr = str_replace($this->get_key_delimiter(), $delimiter, $newStr);	
		}
		return $newStr;
	}
	
	//  Strip out the namespace from a string
	public function strip_namespace($str, $delimiter = false) {
		return substr($str, strlen($this->add_namespace('', $delimiter)));	
	}
	
	//  Get the namespace
	public function get_namespace() {
		return $this->namespace;	
	}
	
	//  Get the key delimiter
	public function get_key_delimiter() {
		return $this->keyDelimiter;	
	}
	
	// Deprecated
	public function to_boolean($str) {
		return $this->str_to_boolean($str);
	}
	
	//  Convert string to boolean
	public function str_to_boolean($str) {
		return (
			(intval($str) === 1) || 
			(strtolower($str) === 'true') ||
			($str === true)
		);
	}
	
	//  Display the widget
	public function display_widget($key, $params = array()) {
		echo $this->controller->run($key, $params);	
	}
	
	//  Run the widget
	public function run($key, $params = array()) {
		return $this->controller->run($key, $params);	
	}
	
	//  Display error messages
	public function display_errors($errors = array(), $header = "") {
		if (!empty($errors)) {
			if (!is_array($errors)) {
				$errors = array($errors);
			}
			include($this->get_path('/widgets/views/errors.php'));
		}
	}
	
	//  Include script
	public function include_script($path) {
		echo '<script type="text/javascript" src="' . $path . '"></script>';
	}
	
	//  Include CSS
	public function include_style($path) {
		echo '<link rel="stylesheet" type="text/css" href="' . $path . '">';
	}
	
	//  Get a data label
	public function get_data_label($table, $field, $serializedField = false) {
		if ($serializedField) {
			return $this->config['data_labels'][$table][$field][$serializedField];	
		} else {
			return $this->config['data_labels'][$table][$field];
		}
	}
	
	//  Convert a string to a key-compatible string
	public function str_to_key($str) {
		return trim(preg_replace('/[^a-z0-9]+/', '_', strtolower($str)));	
	}
	
	//  Pluralize a word
	public function pluralize_str($num, $singular, $plural = false) {
		if ($num == 1) {
			return $singular;	
		} else {
			if ($plural) {
				return $plural;
			} else {
				if (substr($singular, -1) == 'y') {
					return $substr($singular, 0, -1) . 'ies';
				} else if (substr($singular, -1) == 'x') {
					return $singular . 'es';
				} else {
					return $singular . 's';
				}
			}
		}
	}
	
	// Echo prices
	public function t_prices($event, $basis = false, $currency='', $show_currency_indicator=true, $currency_indicator='', $currency_list=false) {
		ob_start();
			$this->controller->display_currency_prices($event, $basis, $currency, $show_currency_indicator, $currency_indicator, $currency_list);
			$contents = ob_get_contents();
		ob_end_clean();
		return $contents;
	}
	
	// Course widget url
	public function get_widget_url_course($course_id) {
		// Get url for Weblink
		return $this->get_abs_url('widgets/course/?course_code='. urlencode($course_id) );
	}
	
	// Checkout widget url
	public function get_widget_url_checkout($event_id=null, $event_currency=false, $track_id=null, $objectives=null) {
		if(!is_null($event_id)) {
			$url = 'widgets/checkout/?event_id=' . $event_id;
		} else {
			$url = 'widgets/checkout/?track_id=' . $track_id;
		}

		if( $event_currency ) {
			$url .= "&event_currency=".$event_currency;
		}
		
		// Get url for Weblink
		return $this->get_abs_url($url);
	}

	public function get_widget_url_track($track_id = false) {
		$url = 'widgets/learning_track/';
		if($track_id) {
			$url .= '?track_id=' . $track_id;
		}

		return $this->get_abs_url($url);
	}

	public function get_widget_url_track_list() {
		$url = 'widgets/learning_track_list';
		return $this->get_abs_url($url);
	}
	
	// Is object hidden
	public function is_hidden($object_type, $id) {
		return false;
	}
	
	
	
	// Resource url
	public function get_resource_url($resource_file) {
		// Get resource url for Weblink
		return $this->get_abs_url("/weblink-components/resources/".$resource_file);
	}

	public function get_product() {
		return 'weblink';
	}
}
