<?php

// Define checkout steps so that we can
// view them in a textual fashion.
define('ADM_CHECKOUT_STEP_COURSE', 1);
define('ADM_CHECKOUT_STEP_ATTENDEE', 2);
define('ADM_CHECKOUT_STEP_TERMS', 3);
define('ADM_CHECKOUT_STEP_CONFIRM', 4);
define('ADM_CHECKOUT_STEP_PLACE', 5);
define('ADM_CHECKOUT_STEP_IPN', 6);		// PayPal IPN
define('ADM_CHECKOUT_STEP_RR', 7);		// Authorize.Net RR
define('ADM_CHECKOUT_BEGIN_STRIPE', 8);

//  Include required classes
require_once('AdministrateDao.php');

require_once('AdministrateWidgetCheckoutPaymentProcessorPayPal.php');
require_once('AdministrateWidgetCheckoutPaymentProcessorSagePay.php');
require_once('AdministrateWidgetCheckoutPaymentProcessorAuthorizeNet.php');
require_once('AdministrateWidgetCheckoutPaymentProcessorWPM.php');

//  Administrate checkout widget
class AdministrateWidgetCheckout extends AdministrateWidget {

	//  Configuration
	protected $key = 'checkout';
	private $orderKey = 'order_id';
	private $eventKey = 'event_id';
	private $trackKey = 'track_id';
	private $stepKey = 'step';
	private $jsonFlag = '<!-- ADMINISTRATE_JSON_START -->';
	private $jsonFlagEnd = '<!-- ADMINISTRATE_JSON_END -->';

	private $stepFields = array(
		1	=>	array(
			'order_currency',
			'order_num_attendees',
			'order_discount',
			'order_max_step'
		),
		2	=>	array(
			'order_buyer_details',
			'order_invoice_address',
			'order_attendee_details',
			'order_max_step'
		),
		3	=>	array(
			'tac',
			'order_max_step'
		),
		4	=>	array(
			'order_max_step'
		),
		5	=>	array(
			'order_status',
			'order_payment_type',
			'order_processor_transaction_id',
			'order_max_step'
		)
	);

	/* @var $order AdministrateWidgetCheckoutOrder */
	private $order = false;

	/* @var $paymentProcessor AdministrateWidgetCheckoutPaymentProcessor */
	private $paymentProcessor = false;
	private $paymentProcessorName = false;

	/* @var $params array */
	protected $params = array();

	/* @var $validParams array - List of parameters which this widget should respect. Required for sanitising input */
	private $validParams = array(
		'namespace',
		'debug',
		'api_key',
		'cache_timeout',
		'tax_label',
		'basis',
		'currency',
		'currency_list',
		'currency_indicator',
		'show_currency_indicator',
		'use_sales_opportunities',
		'order_notify_email',
		'max_attendees',
		'default_country',
		'terms',
		'course_fields',
		'step1_course_fields',
		'step4_course_fields',
		'billing_address',
		'payment_by_invoice',
		'invoice_instructions',
		'invoice_send_email',
		'payment_by_cc',
		'payment_processor',
		'paypal_mode',
		'sagepay_mode',
		'authorizenet_mode',
		'sagepay_email_message',
		'cc_instructions',
		'cc_send_email',
		'conversion_tracking',
		'event_id',
		'track_id',
		'event_currency',
		'show_times',
		'show_classroom_dates',
		'show_promo_codes',
		'step',
		'order_error_message',
		'force_email_check'
	);

	//  Constructor
	public function __construct(&$service, $params = array()) {

		//  Call parent constructor
		parent::__construct($service, $params);

		//  Save the global variable names w/ namespacing
		$this->orderKey = $this->add_namespace($this->orderKey);
		$this->eventKey = $this->add_namespace($this->eventKey);
		$this->stepKey = $this->add_namespace($this->stepKey);

		// Discount JSON output
		$this->discount_json();

		//  Start the session
		if(session_id() == '') {
			@session_start();
		}

	}

	// Get discount JSON
	public function discount_json() {

		if( !array_key_exists('validate_promo_code', $_REQUEST) ) return;
		if( !array_key_exists('del_count', $_REQUEST) ) return;
		$code = $_REQUEST['validate_promo_code'];
		$del_count = $_REQUEST['del_count'];

		// Do a lookup against the API to find all discounts
		$discounts = Administrate::api()->raw_call('getAllDiscounts');

		$ret_discounts = array();


		// Find if we've got a valid discount
		foreach($discounts->AllDiscounts as $discount)
		{
			if($discount->Type == 'partner') {
				if((strtolower($discount->Code) == strtolower($code) && $discount->PromoType != 'percentage_delegates') || ($discount->PromoType == 'percentage_delegates' && $del_count >= $discount->Delegates)) {

					// Found one - build an object response and return it
					$validated = new stdClass();
					$validated->type = $discount->PromoType;
					$validated->discount_code = $discount->Code;
					$validated->discount_amount = $discount->Rate;
					$ret_discounts[] = $validated;
				}
			}
		}

		// If no discount, return false
		echo json_encode($ret_discounts);
		exit;
	}

	//  Run the checkout
	public function run() {

		// MSIE 9 workaround for Ajax form in checkout
		if( array_key_exists('msie_form', $_GET) ) {
			$args = explode("&", $_GET["msie_form"] );

			$order_attendee_details_first_name = array();
			$order_attendee_details_last_name = array();
			$order_attendee_details_email = array();

			foreach( $args as $a ) {
				$pair  = explode("=", $a);
				$key   = urldecode($pair[0]);
				$value = urldecode($pair[1]);

				if( $key == "order_attendee_details_first_name[]" ) {
					$order_attendee_details_first_name[] = $value;
				}
				else if( $key == "order_attendee_details_last_name[]" ) {
					$order_attendee_details_last_name[] = $value;
				}
				else if( $key == "order_attendee_details_email[]" ) {
					$order_attendee_details_email[] = $value;
				}
				else {
					$_POST[$key] = $_REQUEST[$key] = $value;
				}
			}
			$_POST["order_attendee_details_first_name"] = $_REQUEST["order_attendee_details_first_name"]  = $order_attendee_details_first_name;
			$_POST["order_attendee_details_last_name"] = $_REQUEST["order_attendee_details_last_name"]  = $order_attendee_details_last_name;
			$_POST["order_attendee_details_email"] = $_REQUEST["order_attendee_details_email"]  = $order_attendee_details_email;
		}

		/* This function:
		 *      Loads the payment processor if required
		 *      Attempts to fetch an order
		 *      If we don't have an order:
		 *          Show the events table and STOP
		 *      If we are processing an IPN response:
		 *          Process it and STOP
		 *      Validates any new information
		 *      Performs some step-specific logic, including ensuring the T's & C's were accepted
		 *      Runs the appropriate checkout step
		 */
		if( !array_key_exists('currency_list', $this->params) ) {
			$this->params['currency_list'] = false;
		}

		// If currency list was passed as string, JSON decode it
		if( is_string( $this->params['currency_list'] ) ) {
			$this->params['currency_list'] = AdministrateEventWidget2::decode_currency_list( $this->params['currency_list'] );
		}

		// Currency list empty or not parsing
		if( !is_array($this->params['currency_list']) || empty($this->params['currency_list']) ) {
			$this->params['currency_list'] = false;
		}

		//  Extract parameters
		$this->params = array_merge(
			array(
				'step'                      => 1,
				'event_id'                  => null,
				'track_id'                  => null,
				'objectives'                => null,
				'show_fields'               => 'code,summary,schedule,location,inclusions,method,prerequisites,topics,benefits,duration',
				'payment_by_cc'             => 0,
				'currency'                  => 'GBP',
				'step1_course_fields'       => '',
				'step4_course_fields'       => '',
				'pricing_basis'             => 'net',
				'tax_label'                 => 'VAT',
				'currency_indicator'        => 'symbol', // symbol / currency
				'show_currency_indicator'   => true,
				'order_id'                  => false,
			),
			$this->params
		);

		//  Set param types
		if(!is_null($this->params['event_id'])) {
			$this->params['event_id'] = intval($this->params['event_id']);
		}

		if(!is_null($this->params['track_id'])) {
			$this->params['track_id'] = intval($this->params['track_id']);
		}

		if (array_key_exists('step',$_REQUEST)) {
			$this->params['step'] = $_REQUEST['step'];
		}
		$this->params['step'] = intval($this->params['step']);
		$this->params[$this->stepKey] = $this->params['step'];
		$this->params[$this->eventKey] = $this->params['event_id'];
		$this->params[$this->orderKey] = $this->params['order_id'];
		if (!$this->params[$this->orderKey] && isset($_POST[$this->orderKey])) {
			$this->params[$this->orderKey] = $_POST[$this->orderKey];
		}
		$this->params['show_fields'] = array_map('trim', explode(',', $this->params['show_fields']));

		// Include order and log DAO's
		require_once($this->get_path('/AdministrateWidgetCheckoutOrder.php'));
		require_once($this->get_path('/AdministrateWidgetCheckoutLog.php'));

		$log = new AdministrateWidgetCheckoutLog($this->service);

		//  Figure out what step we're on
		if (!$this->params[$this->stepKey]) {
			$current_step = constant('ADM_CHECKOUT_STEP_COURSE');
		} else {
			$current_step = intval($this->params[$this->stepKey]);
		}

		// Get order currency
		//
		if( isset($_SESSION[$this->orderKey]) ) {
			$order_id = $_SESSION[$this->orderKey];
		}
		else if( $this->params[$this->orderKey] ) {
			$order_id = $this->params[$this->orderKey];
		}
		else {
			$order_id = false;
		}

		$previous_order = false;

		// Get previous order for the session key
		if( $order_id !== false ) {
			$previous_order = new AdministrateWidgetCheckoutOrder(
				$this->service,
				array( $this->strip_namespace($this->orderKey) => $order_id )
			);

			$order_currency = $previous_order->get_currency();
		}
		else {
			if( array_key_exists('event_currency', $_REQUEST) ) {
				$order_currency = $_REQUEST['event_currency'];
			}
			elseif(!$this->params['currency']) {
				$order_currency = $this->service->get_option('currency');
			}
			// Have a last-gasp attempt at grabbing the currency from the paypal IPN post
			elseif(array_key_exists('mc_currency', $_REQUEST)) {
				$order_currency = $_REQUEST['mc_currency'];
			}
		}

		// If checkout for the same event in a different currency,
		//
		if( array_key_exists('event_currency', $_REQUEST) ) {
			$this->params['currency'] = $this->params['event_currency'];

			if( $order_currency!=$_REQUEST['event_currency'] ) {
				// Reset the order
				unset($_SESSION[$this->orderKey]);
				$this->params[$this->orderKey] = false;
			}
		}
		else if( isset($order_currency) ) {
			$this->params['currency'] = $this->params['event_currency'] = $order_currency;
		}

		// If we're enabling pay by CC and we're passed the terms step
		if( $this->service->to_boolean($this->service->get_option('payment_by_cc')) )
		{
			// If multi-currency enabled,
			if( array_key_exists('currency_list',$this->params) && is_array($this->params['currency_list']) ) {
				if( AdministrateWidgetCheckoutPaymentProcessorAuthorizeNet::has_order_key() ) {
					$processor = "AuthorizeNet";
				}
				else if( AdministrateWidgetCheckoutPaymentProcessorSagePay::has_order_key() ) {
					$processor = "SagePay";
				}
				else if( AdministrateWidgetCheckoutPaymentProcessorPayPal::has_order_key() ) {
					$processor = "PayPal";
				}
				else {
					// Get processor associated with this currency
					$processor = $this->params['currency_list'][$order_currency];
				}
			}
			else {
				// Defualt processor
				$processor = $this->service->get_option('payment_processor');
			}

			// Now load the class specific to the processor we're using
			$class = __CLASS__ . 'PaymentProcessor' . $processor;

			if( !class_exists($class) ) {
				$processor = "PayPal";
				$class = __CLASS__ . 'PaymentProcessorPayPal';
			}

			$this->paymentProcessorName = $processor;

			if( $current_step >= constant('ADM_CHECKOUT_STEP_CONFIRM') ) {
				$this->paymentProcessor = new $class($this->service, $this);
				if ($this->paymentProcessor->order && $this->paymentProcessor->order->exists()) {
					$this->order = $this->paymentProcessor->get_order();
				}
			}
		}

		// Try and fetch order
		if (!$this->order) {

			// If a zero event ID was passed, reset the session
			if(
				$this->params[$this->eventKey] === 0
				|| $this->params[$this->trackKey] === 0
			) {
				$this->order = false;
				if (isset($_SESSION[$this->orderKey])) {
					unset($_SESSION[$this->orderKey]);
				}

			// Otherwise try to bring up the order in the session
			} else {

				// Take order key from session if we can, otherwise check if it's set in postdata
				$where = array();
				if (isset($_SESSION[$this->orderKey])) {
					$where[$this->strip_namespace($this->orderKey)] = $_SESSION[$this->orderKey];
				} elseif($this->params[$this->orderKey]) {
					$where[$this->strip_namespace($this->orderKey)] = $this->params[$this->orderKey];
				}

				//  If we have anything to query for the order, query for it now
				if (!empty($where)) {
					$this->order = new AdministrateWidgetCheckoutOrder($this->service, $where);
					if (!$this->order->exists()) {
						$this->order = false;

					}
				}

				// Was an event ID passed in?
				if(
					$this->params[$this->eventKey] > 0
					|| $this->params[$this->trackKey] > 0
				) {
					// If we've not got an order, or the order we've got doesn't match the requested key
					if(
						!$this->order
						|| (isset($this->params[$this->eventKey]) && ($this->order->get_event_id() != $this->params[$this->eventKey]))
						|| (isset($this->params[$this->trackKey]) && ($this->order->get_track_id() != $this->params[$this->trackKey]))
					) {
						// Build a new order
						$this->order = new AdministrateWidgetCheckoutOrder($this->service);

						// Use post/get currency, or read from settings
						if( array_key_exists('event_currency', $_REQUEST) ) {

							$this->params['currency'] = $_REQUEST['event_currency'];
						}
						else if( !$this->params['currency'] ) {
							$this->params['currency'] = $this->service->get_option('currency');
						}

						if(!is_null($this->params[$this->eventKey])) {
							$event = Administrate::api()->get_event( $this->params[$this->eventKey] );

							// If the event is not available in the desired currency,
							if( ! $event->get_price_by_currency( $this->params['currency'] ) ) {
								// Start the order in the event default currency
								$this->params['currency'] = $event->get_default_currency();
							}

							$this->order->create($this->params[$this->eventKey], session_id(), $this->params['currency'] );
						} else {
							$this->order->create(null, session_id(), $this->params['currency'], $this->params[$this->trackKey]);
						}

						// Build the new order and assign to the session
						$_SESSION[$this->orderKey] = $this->order->get_id();
					}
				}
			}

		}

		// If we *still* don't have an order - just show the event table
		if(!$this->order) {
			return $this->service->display_widget('event_table', $this->params);
		}
		$this->setObjectives();

		if ($this->registeringInitialInterests($current_step)){
			$this->setOrderItemIds();

		}

		$log->create('Found order, beginning step ' . ($current_step-1) . '.', $this->order->get_id());

		if($this->order->get_event_id()) {
			// Get a fresh event (caching disabled)
			$fresh_event = $this->order->get_event(false);

			// No price available
			if (!$fresh_event || !$fresh_event->get_price_by_currency($this->params['currency'])) {
				return $this->service->display_widget('event_table');
			}
		}

		// If this is an IPN response, handle the payment and stop
		if($current_step == constant('ADM_CHECKOUT_STEP_IPN')) {
			$log->create('Received IPN response, as step is 6 and order is not complete.', $this->order->get_id());
			$this->order->receive_ipn();
			return;
		}

		// At this point, we must have an order - rejoice!
		// Read and use some defaults from the options.
		$this->widgetTitle = $this->_generate_title();

		// Default $errors to empty array, we'll populate it if we have any.
		$errors = array();

		/* Max step is the highest step that we've completed and
		 * validated - this determines what steps are available
		 * in the checkout widget.
		 */
		if($current_step > $this->order->get_max_step()) {
			// Set it in the post, so that we can save it with
			// the rest of the post data from the order.
			$_POST['order_max_step'] = $current_step;
		}

		if($current_step > constant('ADM_CHECKOUT_STEP_COURSE')) {
			//first buyer fields were disabled
			if (array_key_exists('buyer_is_attendee', $_POST) && $_POST['buyer_is_attendee'] == 'on') {
				//more than one buyer
				if (array_key_exists('order_attendee_details_first_name', $_POST)) {
					array_unshift($_POST['order_attendee_details_first_name'], $_POST['order_buyer_details_first_name']);
					array_unshift($_POST['order_attendee_details_last_name'], $_POST['order_buyer_details_last_name']);
					array_unshift($_POST['order_attendee_details_email'], $_POST['order_buyer_details_email']);
				//only one buyer
				} else {
					$_POST['order_attendee_details_first_name'] = array($_POST['order_buyer_details_first_name']);
					$_POST['order_attendee_details_last_name'] = array($_POST['order_buyer_details_last_name']);
					$_POST['order_attendee_details_email'] = array($_POST['order_buyer_details_email']);
				}
			}
			$errors = $this->order->save($this->_prepare_api_fields($current_step - 1, $_POST));
		}

		// #### EXTRA VALIDATION FOR SPECIFIC STEPS ####

		// If we're in an order and there's no currency, reset to first step
		$currency = $this->order->get_currency();
		if($current_step > constant('ADM_CHECKOUT_STEP_COURSE') && empty($currency)) {
			$log->create('Tried to progress past step course with no currency set, resetting to step 1.', $this->order->get_id());
			$current_step = 1;
		}

		// If we're on the confirmation step and the T's & C's
		// weren't accepted, add the validation error.
		if($current_step == constant('ADM_CHECKOUT_STEP_CONFIRM') && !isset($_REQUEST['tac']) && !isset($_REQUEST[$this->service->add_namespace(array('processor', 'cancel'))])) {
			$log->create('Tried to progress past T&C step without ticking checkbox.', $this->order->get_id());
			$errors[] = $this->service->get_text('You must accept the Terms &amp; Conditions.');
		}

		$log->create('Completed logic checks for step ' . ($current_step-1) . '.', $this->order->get_id());
		// If we've errors revert to previous step to allow correction.
		if($current_step > constant('ADM_CHECKOUT_STEP_COURSE') && !empty($errors) ) {
			$current_step--;
		}

		// Form action is the 'next' step - e.g. where the 'submit' button goes.
		$formAction = $this->get_step_url($current_step + 1);

		// Run the appropriate step
		$step_content = array();
		$complete_content = false;
		if($current_step == constant('ADM_CHECKOUT_STEP_PLACE')) {
			// _run_completion will place the order with Administrate
			$complete_content = $this->_run_completion($errors);
		} elseif($current_step == constant('ADM_CHECKOUT_BEGIN_STRIPE')) {
			$payment_processor  = $this->paymentProcessorName;
			if($payment_processor == 'Stripe') {
				// If paying by invoice
				if(!Administrate::get_is_invoice_payment_from_post() && $this->paymentProcessor->get_token()) {
					// This is the response from the Stripe server, just return it as this should be an AJAX call.
					echo $this->paymentProcessor->card_transaction();
					exit;
				}
			}
		} else {
			if(
				(!isset($_REQUEST['ajax']) && ($current_step >= constant('ADM_CHECKOUT_STEP_COURSE')))
				|| ($current_step == constant('ADM_CHECKOUT_STEP_COURSE'))
			) {
				$step_content[constant('ADM_CHECKOUT_STEP_COURSE')] = $this->_run_step1($errors);
			}

			if(
				(!isset($_REQUEST['ajax']) && ($current_step >= constant('ADM_CHECKOUT_STEP_ATTENDEE')))
				|| ($current_step == constant('ADM_CHECKOUT_STEP_ATTENDEE'))
			) {
				$step_content[constant('ADM_CHECKOUT_STEP_ATTENDEE')] = $this->_run_step2($errors);
			}

			if(
				(!isset($_REQUEST['ajax']) && ($current_step >= constant('ADM_CHECKOUT_STEP_TERMS')))
				|| ($current_step == constant('ADM_CHECKOUT_STEP_TERMS'))
			) {
				$step_content[constant('ADM_CHECKOUT_STEP_TERMS')] = $this->_run_step3($errors);
			}

			if(
				(!isset($_REQUEST['ajax']) && ($current_step >= constant('ADM_CHECKOUT_STEP_CONFIRM')))
				|| ($current_step == constant('ADM_CHECKOUT_STEP_CONFIRM'))
			) {
				$step_content[constant('ADM_CHECKOUT_STEP_CONFIRM')] = $this->_run_step4($errors);
			}
		}

		// If the request was made via AJAX, just spit out the requested step and exit
		if(isset($_REQUEST['ajax']) && ( ($current_step < constant('ADM_CHECKOUT_STEP_PLACE')) )) {

			$log->create('Echoing out json for step ' . ($current_step) . ' of checkout.', $this->order->get_id());
			echo $this->jsonFlag;
			$json = array(
				"step"		=>	$current_step,
				"content"	=>	$step_content[$current_step],
				"formAction"	=> 	$formAction
			);
			echo json_encode($json);
			echo $this->jsonFlagEnd;
			exit;

		}

		// Otherwise show the whole checkout interface
		else {

			$log->create('Rendering out complete form for ' . ($current_step) . ' of checkout.', $this->order->get_id());
			// If there is content to display, display it in the checkout template
			if ((count($step_content) > 0) || $complete_content) {

				// Include the template
				ob_start();
				require_once($this->get_path('/views/checkout.php'));
				return ob_get_clean();

			}
		}
	}

	//  Run step 1
	private function _run_step1($errors = array()) {

		if($this->order->get_event_id()) {

			$possible_mapped_items = Administrate::api()->get_mapped_items(
				$this->order->get_event()->get_course()->get_id(),
				$this->order->get_event()->get_location_id()
			);

			$content = $this->get_include_contents(
				$this->get_path('/views/checkout_step1.php'),
				array(
					'course' => $this->widgetTitle,
					'course_fields' => $this->service->get_event_fields($this->order->get_event(), $this->service->get_option('step1_course_fields')),
					'num_attendees' => $this->order->get_num_attendees(),
					'prices' => $this->order->get_event_prices(),
					'pricing_basis' => $this->service->get_option('basis'),
					'tax_label' => $this->service->get_option('tax_label'),
					'default_currency' => $this->order->get_currency(),
					'show_currency_indicator' => $this->service->to_boolean($this->service->get_option('show_currency_indicator')),
					'currency_indicator' => $this->service->get_option('currency_indicator'),
					'event' => $this->order->get_event(),
					'show_promo_codes' => $this->service->to_boolean($this->service->get_option('show_promo_codes')),
					'errors' => $errors,
					'is_event_free' => $this->order->is_free(),
					'show_sessions' => $this->service->get_bool_option('show_sessions'),
					'show_session_titles' => $this->service->get_bool_option('show_session_titles'),
					'show_times' => $this->service->get_bool_option('show_times'),
					'possible_mapped_items' => $possible_mapped_items,
					'item_ids' => $this->order->get_item_ids()
				)
			);
		} else {
			$content = $this->get_include_contents(
				$this->get_path('/views/checkout_step1_track.php'),
				array(
					'track' => $this->order->get_track(),
					'num_attendees' => $this->order->get_num_attendees(),
					'promo_code' => $this->order->get_promo_code(),
					'pricing_basis' => $this->service->get_option('basis'),
					'show_currency_indicator' => $this->service->get_option('show_currency_indicator', 'pricing'),
					'currency' => $this->order->get_currency(),
					'errors' => $errors,
				)
			);
		}

		//  Return the content to display
		return $content;

	}

	//  Run step 2
	private function _run_step2($errors = array()) {

		//  Save reference to table fields
		$tableFields = $this->service->get_table_fields('orders');

		//  Set the buyer fields
		$buyerValues = $this->_get_serialized_field_values($tableFields['order_buyer_details']['serialize'], $this->order->get_buyer_details(), 'order_buyer_details_');

		//  Set the invoice address fields
		$invoiceValues = $this->_get_serialized_field_values($tableFields['order_invoice_address']['serialize'], $this->order->get_invoice_address(), 'order_invoice_address_');
		if (empty($invoiceValues['country'])) {
			$invoiceValues['country'] = $this->service->get_option('default_country');
		}

		//  Set the attendee fields
		$attendeeValues = $this->_get_serialized_field_values($tableFields['order_attendee_details']['serialize'], $this->order->get_attendee_details(), 'order_attendee_details_', $this->order->get_num_attendees());

		//  Includes the step 2 template
		$countries = new AdministrateCountriesList();
		$content = $this->get_include_contents(
			$this->get_path('/views/checkout_step2.php'),
			array(
				'buyer_fields' => $tableFields['order_buyer_details']['serialize'],
				'buyer_values' => $buyerValues,
				'invoice_fields' => $tableFields['order_invoice_address']['serialize'],
				'invoice_values' => $invoiceValues,
				'attendee_fields' => $tableFields['order_attendee_details']['serialize'],
				'attendee_values' => $attendeeValues,
				'num_attendees' => $this->order->get_num_attendees(),
				'countries' => $countries->get_list(),
				'errors' => $errors,
				'is_event_free' => $this->order->is_free()
			)
		);

		//  Return the content to display
		return $content;

	}

	//  Run step 3
	private function _run_step3($errors = array()) {

		//  Includes the step 3 template
		$content = $this->get_include_contents(
			$this->get_path('/views/checkout_step3.php'),
			array(
				'errors'	=>	$errors
			)
		);

		//  Return the content to display
		return $content;

	}

	//  Run step 4
	private function _run_step4($errors = array()) {

		//  Save reference to table fields
		$tableFields = $this->service->get_table_fields('orders');

		//  Set the buyer fields
		$buyerValues = $this->_get_serialized_field_values($tableFields['order_buyer_details']['serialize'], $this->order->get_buyer_details(), 'order_buyer_details_');

		//  Set the invoice address fields
		$invoiceValues = $this->_get_serialized_field_values($tableFields['order_invoice_address']['serialize'], $this->order->get_invoice_address(), 'order_invoice_address_');

		//  Set the attendee fields
		$attendeeValues = $this->_get_serialized_field_values($tableFields['order_attendee_details']['serialize'], $this->order->get_attendee_details(), 'order_attendee_details_', $this->order->get_num_attendees());

		//  If there is a payment processor, set the order to it
		if ($this->paymentProcessor) {
			$this->paymentProcessor->set_order($this->order);
		}

		if($this->order->get_event_id()) {
			$event = $this->order->get_event();

			// Tax label
			$tax_label_custom = $event->get_tax_name();
			$tax_label = $event->is_eu_digital_good() && !empty($tax_label_custom) ?
				$tax_label_custom : $this->service->get_option('tax_label', 'pricing');

			//  Includes the step 4 template
			$content = $this->get_include_contents(
				$this->get_path('/views/checkout_step4.php'),
				array(
					'buyer_fields' => $tableFields['order_buyer_details']['serialize'],
					'buyer_values' => $buyerValues,
					'invoice_fields' => $tableFields['order_invoice_address']['serialize'],
					'invoice_values' => $invoiceValues,
					'attendee_fields' => $tableFields['order_attendee_details']['serialize'],
					'attendee_values' => $attendeeValues,
					'num_attendees' => $this->order->get_num_attendees(),
					'course_fields' => $this->service->get_event_fields($this->order->get_event(), $this->service->get_option('step4_course_fields')),
					'order_id' => $this->order->get_id(),
					'billing_address' => $this->service->get_option('billing_address'),
					'course' => $this->widgetTitle,
					'prices' => $this->order->get_event_prices(),
					'pricing_basis' => $this->service->get_option('basis', 'pricing'),
					'tax_label' => $tax_label,
					'default_currency' => $this->order->get_currency(),
					'show_currency_indicator' => $this->service->to_boolean($this->service->get_option('show_currency_indicator')),
					'currency_indicator' => $this->service->get_option('currency_indicator'),
					'event' => $event,
					'errors' => $errors,
					'is_event_free' => $this->order->is_free(),
					'mapped_items' => $this->order->get_mapped_items_and_prices()
				)
			);
		} else {
			$content = $this->get_include_contents(
				$this->get_path('/views/checkout_step4_track.php'),
				array(
					'track' => $this->order->get_track(),
					'num_attendees' => $this->order->get_num_attendees(),
					'promo_code' => $this->order->get_promo_code(),
					'pricing_basis' => $this->service->get_option('basis'),
					'billing_address' => $this->service->get_option('billing_address'),
					'show_currency_indicator' => $this->service->to_boolean($this->service->get_option('show_currency_indicator')),
					'currency' => $this->order->get_currency(),
					'buyer_fields' => $tableFields['order_buyer_details']['serialize'],
					'buyer_values' => $buyerValues,
					'invoice_fields' => $tableFields['order_invoice_address']['serialize'],
					'invoice_values' => $invoiceValues,
					'attendee_fields' => $tableFields['order_attendee_details']['serialize'],
					'attendee_values' => $attendeeValues,
					'errors' => $errors,
				)
			);
		}

		//  Return the content to display
		return $content;

	}

	//  Run step 5 (ie, completion)
	private function _run_completion($errors) {
		$payment_processor  = $this->paymentProcessorName;
		$order_status_ok = true;

		if($payment_processor == 'Stripe') {
			// If paying by invoice
			if(Administrate::get_is_invoice_payment_from_post()) {
				$order_status_ok = true;
			} else {
				// Error handling is now handled by Stripe on the client side.
				$this->order->complete(true);
				$order_status_ok = true;
			}
		}

		// If the order isn't complete, process it
		if(!$this->order->is_complete() && $order_status_ok ) {
			/* If we're doing a SagePay order, there is no differentiation
			 * between SuccessURL and IPN - it only accepts one URL. For this
			 * reason we ensure that the order is processed and not submitted
			 * as 'pending.'
			 */
			$skip_pending = ($this->paymentProcessorName == 'SagePay') || isset($_GET['crypt']);
			$order_status_ok = $this->order->complete($skip_pending);
		}

		//  Kill the session variable
		unset($_SESSION[$this->orderKey]);

		if( is_object($this->paymentProcessor) && property_exists($this->paymentProcessor, "display_name") ) {
			$processor_display_name = $this->paymentProcessor->display_name;
		}
		else {
			$processor_display_name = $this->paymentProcessorName;
		}
		//  Includes the step 5 template
		$content = $this->get_include_contents(
			$this->get_path('/views/checkout_complete.php'),
			array(
				'payment_type'		=>	$this->order->get_payment_type(),
				'payment_processor'	=>	$processor_display_name,
				'errors'			=>	$errors,
				'order_status_ok'		=>	$order_status_ok,
				'order_num'			=>	$this->order->get_api_invoice_id(),
				'order_currency'			=>	$this->order->get_currency()
			)
		);


		//  Return the content to display
		return $content;

	}

	//  Prepare fields
	private function _prepare_api_fields($step, $inputFields) {
		$outputFields = array();
		foreach ($inputFields as $key=>$value) {
			if (isset($this->stepFields[$step])) {
				foreach ($this->stepFields[$step] as $allowedField) {
					if (substr($key, 0, strlen($allowedField)) == $allowedField) {
						$outputFields[$key] = $value;
					}
				}
			}
		}
		return $outputFields;
	}

	protected function registeringInitialInterests($current_step) {
		return $current_step - 1 == 1;
	}

	protected function setOrderItemIds() {
		// If we have item id's set for mapped items, save them using our setter then nuke them from the POST
		// so that they don't overwrite themselves with a serialized out PHP Array() later.
		if (array_key_exists('order_item_ids', $_POST)) {
			$this->order->set_item_ids($_POST['order_item_ids']);
			unset($_POST['order_item_ids']);
		} else {
			$this->order->set_item_ids(array());
		}
	}

	protected function setObjectives() {
		if (array_key_exists('objectives', $_GET)) {
			$objectives = array();
			$split = explode(',', $_GET['objectives']);
			foreach($split as $item) {
				$objective = explode('-', $item);
				$objective = array(
					"objective_id" => $objective[0],
					"event_id" => $objective[1],
				);
				$objectives[] = $objective;
			}
			$this->order->set_objective_ids($objectives);
		}
	}

	//  Validate the payment
	private function _validate_payment() {

		//  Initialize errors array
		$errors = array();

		//  If the payment type is credit card, validate using processor specific routines
		if (!isset($_REQUEST['order_payment_type'])) {
			$paymentError = $this->paymentProcessor->validate_payment();
			if ($paymentError) {
				$errors['payment'] = $paymentError;
			}
		}

		//  Return the errors
		return $errors;

	}

	//  Process the payment
	private function _process_payment() {
		return $this->paymentProcessor->process_payment();
	}

	//  Get field values for a serialized set of fields
	private function _get_serialized_field_values($fields, $savedValues, $fieldPrefix, $num = false) {
		$savedValues = unserialize(base64_decode($savedValues));
		$values = array();
		foreach ($fields as $field) {
			if (isset($_POST[$fieldPrefix.$field])) {
				$values[$field] = $_POST[$fieldPrefix.$field];
			} else if (is_array($savedValues) && isset($savedValues[$field])) {
				$values[$field] = $savedValues[$field];
			} else {
				if ($num) {
					$values[$field] = array();
					for ($i = 1; $i <= $num; ++$i) {
						array_push($values[$field], '');
					}
				} else {
					$values[$field] = '';
				}
			}
		}
		return $values;
	}

	//  Generate the widget title
	private function _generate_title() {
		if($this->order->get_event_id()) {
			$title = $this->order->get_course_title() . '<br />';

			$date_range = $this->order->get_event_dates($this->service->get_bool_option('show_classroom_dates'));
			$online_event = $this->order->get_event()->is_online();
			if ($this->order->get_event()->is_persistent()) {
				$title .= 'Access Duration: ' . $this->order->get_event()->get_access_duration() . ' days.';
			} else {
				$title .= $this->service->t_dates($date_range, $online_event, $this->order->get_event()->is_persistent());
			}

			if ($this->_is_true($this->service->get_option('show_times'))) {
				$title .= '<br />' . $this->service->t_times($this->order->get_event_times());
			}
		} else {
			$title = $this->order->get_track()->name . '<br />';
		}
		return $title;
	}

	//  Get the step URL
	public function get_step_url($num=0, $includeHost = true) {
		$href = $this->get_base_url($includeHost);
		if (strpos($href, '?') > -1) {
			$href .= '&';
		} else {
			$href .= '?';
		}
		//$href .= $this->stepKey . '=' . $num;
		return $href;
	}

	//  Get the base URL
	public function get_base_url($includeHost = false) {

		//  If the base URL isn't set yet, set it now
		if (!property_exists($this, 'baseUrl')) {
			$params = array();
			foreach ($_GET as $key=>$val) {
				if (in_array($key, $this->validParams)) {
					$params[$key] = $val;
				}
			}
			if (isset($params[$this->stepKey])) {
				unset($params[$this->stepKey]);
			}
			$this->baseUrl = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);
			if (!empty($params)) {
				$this->baseUrl .= '?' . http_build_query($params);
			}
		}

		//  If we're including the host ...
		if ($includeHost) {

			//  If the full base URL isn't set, set it now
			if (!property_exists($this, 'baseUrlFull')) {
				$tmp = 'http';

				$use_https = (
					array_key_exists('HTTPS',$_SERVER)
					|| (array_key_exists('HTTP_X_FORWARDED_PROTO', $_SERVER) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https')
				);

				if($use_https) {
					$tmp .= 's';
				}
				$tmp .= '://'.$_SERVER['SERVER_NAME'];
				if (($_SERVER["SERVER_PORT"] != 80) && ($_SERVER['SERVER_PORT'] != 443)) {
					$tmp .= ':'.$_SERVER['SERVER_PORT'];
				}
				$this->baseUrlFull = $tmp . $this->baseUrl;
			}

			//  Return the full base URL
			return $this->baseUrlFull;

		//  Or else return the base URL
		} else {
			return $this->baseUrl;
		}

	}

	//  Get the reset URL
	private function _get_reset_url() {
		return $this->service->get_abs_url('widgets/checkout/?event_id=0');
	}

	//  Display the payment form
	public function display_payment_form() {
		if (is_object($this->paymentProcessor)) {
			$this->paymentProcessor->display_form();
		}
	}

	// Get the order
	public function get_order() {
			return $this->order;
	}

	//  Get the order event
	public function get_order_event() {
		return $this->order->get_event();
	}

}
