<?php

//  SagePay Administrate Widget Checkout Payment Processor
class AdministrateWidgetCheckoutPaymentProcessorSagePay extends AdministrateWidgetCheckoutPaymentProcessor {

	//  Properties
	protected $key = 'sagepay';
	protected $apiUrl = '';
	protected $decryptedParams = array();
	protected $config = array();

	//  Constructor
	public function __construct(&$service, &$widget, &$order = false) {
		$this->display_name = "SagePay";

		//  Call parent class
		parent::__construct($service, $widget, $order);

		// Get the config
		$this->config = Administrate::api()->get_payment_config('sagepay', false);

		//  Parse the params
		if (isset($_GET['crypt'])) {
			parse_str($this->_decrypt_str($_GET['crypt']), $this->decryptedParams);
		}

		//  If the order isn't defined yet and an item number is in the request, initialize order
		if (!$this->order && isset($this->decryptedParams['VendorTxCode'])) {
			$this->set_order($this->get_order());
		}

		if($this->service->get_option('sagepay_mode') == 'Test') {
			$this->apiUrl = 'https://test.sagepay.com/gateway/service/vspform-register.vsp';
		} else {
			$this->apiUrl = 'https://live.sagepay.com/gateway/service/vspform-register.vsp';
		}

		$this->protocolVer = "3.00";

		if($this->service->get_option('sagepay_mode') == 'Simulator') {
			// Testing simulator is only available with v2.23
			$this->protocolVer = "2.23";
			$this->apiUrl = 'https://test.sagepay.com/Simulator/VSPFormGateway.asp';
		}
	}

	//  Display payment form
	public function display_form() {

		//  Get the required options
		$vendor = $this->config->username;
		$password = $this->config->password;
		$emailMessage = $this->service->get_option('sagepay_email_message');
		$adminEmail = $this->service->get_option('order_notify_email');
		$notifyUser = $this->service->to_boolean($this->service->get_option('cc_send_email'));
		
		//  Use the session ID for the transaction Code
		$txCode    = $this->service->get_namespace(). '-' . $this->order->get_id(). '-' . time();
		//  Figure out what emails to send
		if (!$notifyUser) {
			$sendEmail = 0;	
		} else {
			$sendEmail = 1;
		}

		$track_total = 0;
		$track = null;

		//  Generate the basket string
		if($this->order->get_event_id()) {
			$basketParts = array(
				1,                                                                            // always 1 line of detail
				str_replace(":", " ", $this->order->get_course_title()),                    // item name
				$this->order->get_num_attendees(),                                            // quantity
				$this->order->get_event_net_price(),                                        // price excluding tax
				$this->order->get_event_gross_price() - $this->order->get_event_net_price(),    // tax
				$this->order->get_event_gross_price(),                                        // price including tax
				$this->order->get_event_gross_price() * $this->order->get_num_attendees()        // total
			);
		} else {

			$track = $this->order->get_track();

			list(
				$tax_amount,
				$tax_name,
				$net_price,
				$discounted_net_price,
				$gross_price,
				$discounted_gross_price,
				) = AdministrateLearningTrackWidget::price_detail(
				$track,
				$this->service->get_option('currency', 'pricing'),
				$this->order->get_num_attendees(),
				$this->get_order()->get_discount_type(),
				$this->get_order()->get_discount_rate()
			);

			$basketParts = array(
				1,
				str_replace(":", " ", str_replace("&", " ", $track->name)),
				$this->order->get_num_attendees(),
				$discounted_net_price / $this->order->get_num_attendees(),
				$tax_amount / $this->order->get_num_attendees(),
				$discounted_gross_price / $this->order->get_num_attendees(),
				$discounted_gross_price
			);

			$track_total = $discounted_gross_price;
		}

		$basket = implode(':', $basketParts); 

		//  If the billing country is UK, change to GB
		if ($this->order->get_invoice_country() == 'UK') {
			$country = 'GB';
		} else {
			$country = $this->order->get_invoice_country();
		}

		// If the billing country is US, SagePay requires the state abbreviation to be 2 characters (sigh)
		$territory = '';
		if ($this->order->get_invoice_country() == 'US') {

			//  Strip everything but characters
			$territory = preg_replace("/[^A-Za-z ]/", '', $this->order->get_invoice_territory());

			//  If we don't have a match already, search for the full state name
			if (!isset($this->usStates[strtoupper($territory)])) {
				if ($tmpKey = array_search(strtolower($territory), $this->usStates)) {
					$territory = $tmpKey;
				}

			//  Otherwise just make sure the state is upper case
			} else {
				$territory = strtoupper($territory);
			}

		}

		//  Set the parameters to pass
		$params = array(

			//  Transaction fields
			'VendorTxCode'		=>	$txCode,
			'SuccessURL'		=>	str_replace(' ', '%20', preg_replace('/step=[0-9]{1}/', 'step=5', urldecode($_REQUEST['cancel_base_url']))),
			'FailureURL'		=>	str_replace(' ', '%20', preg_replace('/step=[0-9]{1}/', 'step=1', $_REQUEST['cancel_base_url'])),
			//  Order info
			'Currency'			=>	$this->order->get_currency(),
			'CustomerName'		=>	$this->order->get_buyer_first_name() . ' ' . $this->order->get_buyer_last_name(),
			'Basket'			=>	$basket,

			//  Email options
			'SendEMail'			=>	$sendEmail,
			'CustomerEMail'		=>	$this->order->get_buyer_email(),
			//'VendorEMail'		=>	$adminEmail,
			'eMailMessage'		=>	$emailMessage,

			//  Billing fields
			'BillingFirstnames'	=>	$this->order->get_buyer_first_name(),
			'BillingSurname'	=>	$this->order->get_buyer_last_name(),
			'BillingAddress1'	=>	$this->order->get_invoice_street(),
			//'BillingAddress2'	=>	'',
			'BillingCity'		=>	$this->order->get_invoice_city(),
			'BillingPostCode'	=>	$this->order->get_invoice_postal_code(),
			'BillingCountry'	=>	$country,
			'BillingState'		=>	$territory,
			'BillingPhone'		=>	$this->order->get_buyer_phone(),

			//  Shipping fields
			'DeliveryFirstnames'=>	$this->order->get_buyer_first_name(),
			'DeliverySurname'	=>	$this->order->get_buyer_last_name(),
			'DeliveryAddress1'	=>	$this->order->get_invoice_street(),
			//'DeliveryAddress2'	=>	'',
			'DeliveryCity'		=>	$this->order->get_invoice_city(),
			'DeliveryPostCode'	=>	$this->order->get_invoice_postal_code(),
			'DeliveryCountry'	=>	$country,
			'DeliveryState'		=>	$territory,
			'DeliveryPhone'		=>	$this->order->get_buyer_phone(),

			//  Payment options
			'AllowGiftAid'		=>	0,
			'ApplyAVSCV2'		=>	1,
			'Apply3DSecure'		=>	1
		);

		if($this->order->get_event_id()) {
			$mapped_items = $this->order->get_mapped_items_and_prices();
			$item_amount = 0;
			foreach($mapped_items as $item) {
				$item_amount += $item['subtotal'] + $item['tax'] - $item['discount'];
			}
			$params['Amount'] = floatval(($this->order->get_event_gross_price() * $this->order->get_num_attendees()) + $item_amount);
			$params['Description'] = substr($this->order->get_course_title() . ' ' . $this->service->get_text('Course Registration'), 0, 99);
		} else {
			$params['Amount'] = floatval($track_total);
			$params['Description'] = substr($track->description, 0, 99);
		}

		// Encrypt the parameters -- NOTE: SagePay blows up if we encode the parameters, so decode them before passing them along
		$encryptedStr = $this->_encrypt_str(urldecode(http_build_query($params)));

		//  Include the form
		require_once($this->widget->get_path('/views/checkout_step4_payment_sagepay.php'));
	}

	//  Encrypt a string
	protected function _encrypt_str($str) {

		// add PKCS5 padding to the text to be encypted
			$blockSize = 16;
		$padLength = $blockSize - (strlen($str) % $blockSize);
		for($i = 1; $i <= $padLength; $i++) {
			$str .= chr($padLength);
		}

			// perform encryption with PHP's MCRYPT module
		$encryptedStr = mcrypt_encrypt(MCRYPT_RIJNDAEL_128, $this->_get_password(), $str, MCRYPT_MODE_CBC, $this->_get_password());

		// perform hex encoding and return
		return "@" . bin2hex($encryptedStr);
	}

	//  Decrypt a string
	protected function _decrypt_str($str) {

		//  Remove the first char which is @ to flag this is AES encrypted
		$str = substr($str, 1); 

		//  HEX decoding
		$str = pack('H*', $str);

		//  Perform decryption with PHP's MCRYPT module
		$decryptedStr = mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $this->_get_password(), $str, MCRYPT_MODE_CBC, $this->_get_password());
		$padChar = ord($decryptedStr[strlen($decryptedStr) - 1]);
		return substr($decryptedStr, 0, -$padChar); 

	}

	//  Get the encryption password
	protected function _get_password() {
		if (!property_exists($this, 'config') || !array_key_exists('password', $this->config)) {
			$this->config = Administrate::api()->get_payment_config('sagepay', false);
		}
		return $this->config->password;
	}

	//  Process the payment
	protected function _process_payment($txnId=null) {

		//  Save the transaction ID
		parent::_process_payment($this->decryptedParams['VPSTxId']);

		//  Redirect to step 5 (completion)
		wp_redirect($this->widget->get_step_url(5, true));
		exit;
	}

	//  Set the order
	public function get_order() {
		if (!$this->order || !property_exists($this, 'order')) {
			$txParts = explode('-', $this->decryptedParams['VendorTxCode']);
			$where = array(
				'order_id'	=>	$txParts[count($txParts)-1]
			);
			$this->order = new AdministrateWidgetCheckoutOrder($this->service, $where);	
		}
		return $this->order;
	}

	// Is order key present in the request
	public static function has_order_key() {
		return isset($_GET['crypt']);
	}

	private function _get_protocol_ver() {
		return $this->protocolVer;
	}
}
