<?php

Class SimpleXMLElementExtended extends SimpleXMLElement {
	public function addChildWithCDATA($name, $value = NULL) {
		$new_child = $this->addChild($name);

		if ($new_child !== NULL) {
			$node = dom_import_simplexml($new_child);
			$no   = $node->ownerDocument;
			$node->appendChild($no->createCDATASection($value));
		}

		return $new_child;
	}
}

//  WPM Administrate Widget Checkout Payment Processor
class AdministrateWidgetCheckoutPaymentProcessorWPM extends AdministrateWidgetCheckoutPaymentProcessor {

	//  Properties
	protected $key = '';
	protected $apiUrl = '';

	//  Constructor
	public function __construct(&$service, &$widget, &$order = false) {
		$this->display_name = "WPM";

		//  Call parent class
		parent::__construct($service, $widget, $order);

		//  If the order isn't defined yet and an item number is in the request, initialize order
		if (!$this->order && isset($_REQUEST['item_number'])) {
			$this->set_order($this->get_order());
		}
	}

	//  Display payment form
	public function display_form() {

		// Get the config
		$config = Administrate::api()->get_payment_config('wpm');
		$this->apiUrl = $config->url;

		$https = (
			array_key_exists('HTTPS',$_SERVER)
			|| (
				array_key_exists('HTTP_X_FORWARDED_PROTO', $_SERVER)
				&& $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https'
			)
		);

		$description = '';
		$amount = '';
		if($this->order->get_event_id()) {
			$mapped_items = $this->order->get_mapped_items_and_prices();
			$item_amount = 0;
			foreach($mapped_items as $item) {
				$item_amount += $item['subtotal'] + $item['tax'] - $item['discount'];
			}
			$description =  $this->order->get_course_title();
			$amount_net_of_tax = number_format($this->order->get_event_net_price() * $this->order->get_num_attendees(), 2, '.', '');
			$amount_of_tax = number_format(($this->order->get_event_gross_price() - $this->order->get_event_net_price()) * $this->order->get_num_attendees(), 2, '.', '');
			$amount_gross_of_tax = number_format($this->order->get_event_gross_price() * $this->order->get_num_attendees(), 2, '.', '');
		} else {
			$track = $this->order->get_track();
			$params["L_BUTTONVAR2"] = "item_name=" . $track->name;

			list(
				$tax_amount,
				$tax_name,
				$net_price,
				$discounted_net_price,
				$gross_price,
				$discounted_gross_price,
				) = AdministrateLearningTrackWidget::price_detail(
				$track,
				$this->service->get_option('currency', 'pricing'),
				$this->order->get_num_attendees(),
				$this->get_order()->get_discount_type(),
				$this->get_order()->get_discount_rate()
			);

			$amount = $discounted_gross_price / $this->order->get_num_attendees();
		}
		$msgid = md5($config->client_id . $this->order->get_id() . $amount_gross_of_tax . $config->shared_key);

		$wpmpaymentrequest = new SimpleXMLElementExtended('<wpmpaymentrequest/>');

		$wpmpaymentrequest->addAttribute('msgid', $msgid);

		$wpmpaymentrequest->addChild('clientid', $config->client_id);
		$wpmpaymentrequest->addChild('requesttype', 1);
		$wpmpaymentrequest->addChild('pathwayid', $config->pathway_id);
		$wpmpaymentrequest->addChild('departmentid', $config->department_id);
		$wpmpaymentrequest->addChild('staffid', '');
		$wpmpaymentrequest->addChild('customerid', 12345678);  // @todo: Spec says account_id but we don't have this info yet.
		$wpmpaymentrequest->addChild('title', '');
		$wpmpaymentrequest->addChildWithCDATA('firstname', $this->order->get_buyer_first_name());
		$wpmpaymentrequest->addChild('middlename', '');
		$wpmpaymentrequest->addChildWithCDATA('lastname', $this->order->get_buyer_last_name());

		$emailFrom = $config->email_from;
		if (empty($emailFrom)) {
			$emailFrom = $this->order->get_buyer_email();
		}
		$wpmpaymentrequest->addChildWithCDATA('emailfrom', $emailFrom);

		$wpmpaymentrequest->addChild('toemail', '');
		$wpmpaymentrequest->addChild('ccemail', '');
		$wpmpaymentrequest->addChild('emailfooter', '');
		$wpmpaymentrequest->addChild('transactionreference', $this->order->get_id());
		$wpmpaymentrequest->addChildWithCDATA('redirecturl', preg_replace('/step=[0-9]{1}/', 'step=5', $_REQUEST['cancel_base_url']));
		$wpmpaymentrequest->addChildWithCDATA('callbackurl', 'http' . ($https ? 's' : '') . '://' . $_SERVER['HTTP_HOST'] . '/common/processors/wpm/wpm_ipn_handler.php');
		$wpmpaymentrequest->addChildWithCDATA('cancelurl', preg_replace('/step=[0-9]{1}/', 'step=1', $_REQUEST['cancel_base_url']));
		$wpmpaymentrequest->addChildWithCDATA('billaddress1', $this->order->get_invoice_street());
		$wpmpaymentrequest->addChild('billaddress2', '');
		$wpmpaymentrequest->addChild('billaddress3', '');
		$wpmpaymentrequest->addChildWithCDATA('billtown', $this->order->get_invoice_city());
		$wpmpaymentrequest->addChildWithCDATA('billcounty', $this->order->get_invoice_territory());
		$wpmpaymentrequest->addChildWithCDATA('billpostcode', $this->order->get_invoice_postal_code());
		$wpmpaymentrequest->addChildWithCDATA('billcountry', $this->order->get_invoice_country());

		$wpmpaymentrequest->addChildWithCDATA('customfield1', $this->order->get_buyer_company());
		$wpmpaymentrequest->addChildWithCDATA('customfield2', $description);
		$wpmpaymentrequest->addChildWithCDATA('customfield3', ($this->order->get_event_id() ? $this->order->get_event()->get_start_date() : ''));
		$wpmpaymentrequest->addChildWithCDATA('customfield4', $this->order->get_buyer_email());

		$wpmpaymentrequest->addChild('customfield8', strtolower($this->service->get_option('use_sales_opportunities')));
		$wpmpaymentrequest->addChild('customfield9', strtolower($this->service->get_option('api_key')));
		$wpmpaymentrequest->addChild('customfield10', $this->order->get_id());

		$payments = $wpmpaymentrequest->addChild('payments');

		$payments->addAttribute('id', 1);
		$payments->addAttribute('type', 'PN');
		$payments->addAttribute('payoption', 'ADM');

		$payments->addChildWithCDATA('description', $description);

		$payment = $payments->addChild('payment');

		$payment->addAttribute('payid', 1);

		$payment->addChild('amounttopay', $amount_gross_of_tax);
		$payment->addChild('amounttopayvat', $amount_of_tax);
		$payment->addChild('amounttopayexvat', $amount_net_of_tax);
		$payment->addChild('vatdesc', '');
		$payment->addChild('vatcode', '');
		$payment->addChild('vatrate', '');
		$payment->addChildWithCDATA('customfield5', $this->order->get_event()->get_finance_code());

		$payment->addChild('dateofpayment', date('Y-m-d H:i:s'));
		$payment->addChild('mandatory', 1);

		require_once($this->widget->get_path('/views/checkout_step4_payment_wpm.php'));
	}


	//  Process the payment
	protected function _process_payment($txnId=null) {
		return parent::_process_payment($_REQUEST['txn_id']);
	}

	//  Set the order
	public function get_order() {
		if (!$this->order || !property_exists($this, 'order')) {
			$where = array(
				'order_id'			=>	$_REQUEST['item_number'],
				'order_session_id'	=>	$_REQUEST['custom']
			);
			$this->order = new AdministrateWidgetCheckoutOrder($this->service, $where);
		}
		return $this->order;
	}

	// Is order key present in the request
	public static function has_order_key() {
		return isset($_REQUEST["item_number"]);
	}
}
