<?php extract($params); ?>
<h3><?php $this->service->echo_text('Register for this Course'); ?></h3>

<?php
// Disable checkout if the event is sold out or no places available
// The function get_num_places() returns 0 only when max_places is set, and course is not full 
if( $event->is_sold_out() || ($event->get_num_places()===0) ) {
	$errors[] = "Unfortunately, the course you have selected is now sold out. Please choose another event.";
}

// NOTE: Courses affacted by bug #17750 get negative tax and 0 total amount
// No course should have negative tax, check for this until the bug is resloved
if( $this->get_order()->get_event_gross_price() < $this->get_order()->get_event_net_price() ) {
	$errors[] = "Checkout for this event is not available.";
}

// Checkout is not possible if there are any errors
$has_errors = (bool) ( count($errors) != 0 );

?>

<?php $this->service->display_errors($errors); ?>

<?php include($this->service->get_path('/widgets/views/course_fields.php')); ?>

<table>
	<thead>
		<tr>
			<th><?php $this->service->echo_text('Course'); ?></th>
			<th class="checkout-price-header">
				<?php $this->service->echo_text('Price per Person'); ?>
				<span class="<?php echo  $this->add_namespace(array('price', 'qualifier'), '-'); ?>">
					<?php if (!empty($tax_label) && !$is_event_free) { ?>
						<?php echo  $default_currency; ?>
						<?php include("checkout_tax_label.php"); ?>
					<?php } ?>
				</span>
			</th>
			<th><?php $this->service->echo_text('Attendees'); ?></th>
		</tr>
	</thead>
	<tbody>
		<tr>
			<td>
				<?php echo  $course; ?>
				<div class="administrate-order-price-mobile">
					<?php $this->service->echo_text('Price per Person', 'administrate'); ?>:
					<?php 
					if($is_event_free) {
						echo "Free";
					} else {
						$this->service->display_currency_prices(
							$this->get_order_event(),
							$pricing_basis, $default_currency, $show_currency_indicator, $currency_indicator
						);
						include("checkout_tax_label.php");
					}
					?>
				</div>
			</td>
			
			<td class="checkout-price">
				<?php 
				if($has_errors) {
					// Don't display price if booking not possible
					echo "-";
				} else if($is_event_free) {
					echo "Free";
				} else {
					$this->service->display_currency_prices(
						$this->get_order_event(), 
						$pricing_basis, $default_currency, $show_currency_indicator, $currency_indicator);
				}
				?>
			</td>
			<td>
				<?php if($has_errors) {
					// Don't display available places if booking not possible
					echo "-";
				} else {
				?>
				<select name="order_num_attendees" id="order_num_attendees">
					<?php
					$maxAttendees = intval($this->service->get_option('max_attendees', 'checkout'));

					// if maxAttendees == 0 (i.e. dropdown in settings is # number for event), then
					// lets start with a big number and reduce it if less places are available
					if($maxAttendees == 0) {
						$maxAttendees = 100; // arbitrary large number for case where event has no max AND max_attendees option is event max
					}
					if(!is_null($event->get_num_places()) && $event->get_num_places() < $maxAttendees) {
						$maxAttendees = $event->get_num_places();
					}
					?>
					<?php for ($i = 1; $i <= $maxAttendees; ++$i) { ?>
						<option
							value="<?php echo  $i; ?>"
							<?php if ($i == $num_attendees) { ?>
								selected="selected"
							<?php } ?>
						><?php echo  $i; ?></option>
					<?php } ?>
				</select>
				<?php } ?>
			</td>
		</tr>
		<?php if(count($possible_mapped_items)) { ?>
			<tr <?php if(!$this->service->get_option('use_sales_opportunities', 'checkout')) echo 'style="display: none"'; ?>>
				<th class="administrate-mapped-items" colspan="3">
					Mapped Items Available with your Order
				</th>
			</tr>
			<?php foreach($possible_mapped_items as $item) { ?>
				<?php
				$prices = array_filter($item->item->prices, function($price) use($default_currency) {
					return $price->currency == $default_currency;
				});

				if(count($prices) == 0) {
					continue;
				}

				$best_price = reset($prices);

				foreach($prices as $price) {
					if($price->currency == $default_currency && $price->amount < $best_price->amount) {
						$best_price = $price;
					}
				}
				?>
				<tr <?php if(!$this->service->get_option('use_sales_opportunities', 'checkout')) echo 'style="display: none"'; ?>>
					<td>
						<input type="checkbox"
						       name="order_item_ids[]"
						       class="order_item_ids"
						       value="<?php echo $item->id; ?>"
							<?php echo (array_search($item->id, $item_ids) !== false ? 'checked' : ''); ?>
						       data-is_discountable="<?php echo ($best_price->is_discountable) ? 1 : 0; ?>"
						       data-is_taxable="<?php echo ($best_price->is_taxable) ? 1 : 0; ?>"
						       data-price="<?php echo $best_price->amount; ?>"
							> <?php echo $item->item->name; ?>
						<?php
						$non_things = array();
						if(!$best_price->is_discountable) $non_things[] = 'Not Discountable';
						if(!$best_price->is_taxable) $non_things[] = 'Not Taxable';
						if(count($non_things)) echo '<small>' . implode(', ', $non_things) . '</small>';
						?>
					</td>
					<td>
						<?php echo $item->item->description; ?>
					</td>
					<td>
						<?php echo $this->service->get_currency_symbol($default_currency) . number_format($best_price->amount, 2); ?>
					</td>
				</tr>
			<?php } ?>
		<?php } ?>
		<tr <?php if( (!$show_promo_codes) || ($is_event_free) || $has_errors ) echo 'style="display: none;"; '?>>
			<td class="administrate-order-promo">
				<?php /* Promo code form */ ?>
				<div id="promo_form" >
					<input type="hidden" id="net_price" value="<?php echo  floatval($event->get_price_by_currency($default_currency)->get_net()); ?>" />
					<input type="hidden" id="gross_price" value="<?php echo  floatval($event->get_price_by_currency($default_currency)->get_gross()); ?>" />
					<input type="hidden" id="include_tax" value="<?php echo  ($pricing_basis == 'gross') ? 1 : 0 ?>" />
					<input type="hidden" id="show_currency_indicator" value="<?php echo  $show_currency_indicator; ?>" />
					<input type="hidden" id="currency_symbol" value="<?php echo  $this->service->get_currency_symbol($default_currency); ?>" />
					<input type="hidden" name="order_discount" id="order_discount">

					<?php echo  $this->service->echo_text('Promo Code'); ?>:<br/>
					<input type="text" id="promo_code">
					<input type="button" id="promo_code_apply" value="<?php $this->service->echo_text('Apply'); ?>" class="<?php echo  $this->service->add_namespace('btn', '-'); ?>" style="float:none;" >
				</div>
				
				<div class="promo-status-container">
					<p class="promo_status" id="promo_correct" style="display: none;">Promo Code Accepted</p>
					<p class="promo_status" id="promo_incorrect" style="display: none;">Promo Code Invalid</p>
				</div>
				
			</td>
			<td class="checkout-price">
				<?php /* Promo code discount prices */ ?>
				<div id="promo_prices">

					<div>
						<span class="promo_label"><?php $this->service->echo_text('Subtotal'); ?>:</span>
						<span id="order_subtotal">-</span>
					</div>

					<div class="order_promo_code_row">
						<span class="promo_label"><?php  $this->service->echo_text('Promo Code'); ?><br><span id="order_promo_rate"></span></span>
						<span class="order_promo_code_row" id="order_promo_code_discount">-</span>
					</div>

					<div class="order_tax_row">
						<span class="promo_label"><?php if($pricing_basis == 'gross') $this->service->echo_text('Tax:'); ?></span>
						<span class="order_tax_row" id="order_tax" >-</span>
					</div>

					<div>
						<span class="promo_label"><?php $this->service->echo_text( ($pricing_basis == 'gross') ? 'TOTAL:' : "Net Total:"); ?></span>
						<span id="order_total">-</span>
					</div>

				</div>
			</td>
			
			<td>&nbsp;</td>
			
		</tr>
	</tbody>
</table>

<?php if( $event->is_multi_session() ) { ?>
<p class="administrate-session-label">Event Dates:</p>
<?php include( $this->service->get_widget_dir(). "/event_sessions.php" ); ?>
<?php } ?>

<?php if( $event->is_eu_digital_good() && !$is_event_free ) { ?>
<p class="administrate_tax_note">
	(*) Depending on your location different EU tax rates may apply for this course. 
</p>
<?php } ?>

<?php if(!$has_errors) { ?>
<input type="submit" value="<?php $this->service->echo_text('Register Now'); ?>" class="<?php echo  $this->service->add_namespace('btn', '-'); ?>">
<?php } ?>

<script>
	// Promo Codes
	(function($){

		// Turns 8123.14 into #8,123.14
		function format_currency(total) {
			var currency = (($('#show_currency_indicator').val() == '1') ? $('#currency_symbol').val() : '');
			var figure = parseFloat(total, 10).toFixed(2).replace(/(\d)(?=(\d{3})+\.)/g, "$1,").toString();
			return currency + figure;
		}

		$(document).ready(function() {

			var include_tax = ($('#include_tax').val() == 1);

			var event_net_price = $('#net_price').val();
			var event_gross_price = $('#gross_price').val();

			var quantity = 1;
			var active_promo_discount = false;
			var active_promo_type = false;

			update_figures();

			function calculate_discount(discount_type, discount_amount) {
				if(discount_type == 'fixed') {
					return {
						discount_amount: (
							(parseFloat(discount_amount) > parseFloat(event_net_price * quantity))
								? (event_net_price * quantity)
								: discount_amount
						),
						discount_rate_display: format_currency(discount_amount)
					};
				} else {
					return {
						discount_amount: (event_net_price * discount_amount).toFixed(2) * quantity,
						discount_rate_display: (parseFloat((discount_amount * 100).toFixed(5))) + '%'
					};
				}
			}

			function calculate_mapped_items_price(tax_pct, discount_pct, qty) {
				var subtotal = 0;
				var discount = 0;
				var tax = 0;

				$('.order_item_ids').each(function(i, item) {
					if(!$(item).is(':checked')) {
						return;
					}

					var price = $(item).data('price');
					subtotal += price;;
					if($(item).data('is_taxable')) {
						tax += (price * tax_pct) - price;
					}
					if($(item).data('is_discountable')) {
						discount += price * discount_pct
					}

				});

				return {
					'subtotal': subtotal * qty,
					'discount': discount * qty,
					'tax': tax * qty
				};
			}

			// Updates the data as displayed on the first step of checkout.
			function update_figures() {
				var subtotal = event_net_price * quantity;
				var discount = 0;
				var tax = 0;

				$('#event_net').html(format_currency(subtotal));

				// Update display
				$('#order_subtotal').html(format_currency(subtotal));

				// Work out price, tax and discount information for event
				var discount_calc = null;
				if (active_promo_discount !== false) {

					var discount_calc = calculate_discount(active_promo_type, active_promo_discount);
					discount = parseFloat(discount_calc.discount_amount);
					$('#order_promo_rate').html(discount_calc.discount_rate_display);
					subtotal -= discount;
					if (subtotal < 0) {
						subtotal = 0;
					}
				}

				var tax_pct = 1;
				if ($('#include_tax').val() == 1) {
					tax_pct = event_gross_price / event_net_price;
					var tax = ((subtotal * tax_pct) - (subtotal));
				}

				// Update prices to include mapped items
				var mapped_price = calculate_mapped_items_price(tax_pct, ((active_promo_type != 'fixed') ? active_promo_discount : 0), quantity);

				// parseFloat all the things so that £25 + £0 is not £250
				subtotal += parseFloat(mapped_price['subtotal']) - parseFloat(mapped_price['discount']);
				tax += parseFloat(mapped_price['tax']);
				discount += parseFloat(mapped_price['discount']);

				if ($('#include_tax').val() == 1) {
					$('#order_tax').html(format_currency(tax));
				}

				if(active_promo_discount) {
					$('#order_promo_code_discount').html(format_currency(discount));
					$('#order_promo_rate').html(discount_calc.discount_rate_display);
				} else {
					$('#order_promo_code_discount').html('');
					$('#order_promo_rate').html('');
				}

				$('#order_subtotal').html(format_currency(subtotal + discount));
				$('#order_total').html(format_currency(subtotal + tax))
			}

			function validate_promo() {
				$.ajax(
					// Post to first checkout step - our param will force a promo code validation to occur.
					'<?php echo  $this->get_step_url(1); ?>',
					{
						data: {
							'validate_promo_code': $('#promo_code').val(),
							'del_count': quantity
						},
						method: 'POST',
						dataType: 'json',
						success: function(data) {
							if (!data.length && !$('#promo_code').val()) {
								return;
							}

							if (!data.length) {
								$('#promo_correct').hide();
								$('#promo_incorrect').show();
								$('#order_discount').val('');
								active_promo_discount = false;
								active_promo_type = false;
								$('#promo_code').val('');
								update_figures();
							} else {
								var discount = false;
								var discount_amount = 0;

								// Ascertain the best discount available
								$.each(data, function(k, this_discount) {

									// get discount for this particular one
									var this_discount_amount = calculate_discount(
										this_discount.type,
										this_discount.discount_amount
									);

									// update if it's a better deal
									if(this_discount_amount.discount_amount > discount_amount) {
										discount_amount = this_discount_amount.discount_amount;
										discount = this_discount;
									}
								});

								if($('#promo_code').val() != discount.discount_code) {
									if(discount.type == 'percentage_delegates') {
										$('#promo_correct').text('Bulk Delegates Discount Added');
									}
								} else {
									$('#promo_correct').text('Promo Code Accepted');
								}

								$('#promo_correct').show();
								$('#promo_incorrect').hide();
								$('#order_discount').val(discount.discount_code);
								$('#promo_code').val(discount.discount_code);

								active_promo_discount = discount.discount_amount;
								active_promo_type = discount.type;
								update_figures();
							}
						}
					}
				);
			}

			$('.order_item_ids').change(function() {
				update_figures();
			});

			$('#order_num_attendees').change(function() {
				quantity = $(this).val();
				validate_promo();
				update_figures();
			});

			$('#promo_code_apply').click(function() {
				validate_promo();
			});

		});
	})(jQuery);
</script>
