<?php
//  Administrate course table widget
class AdministrateWidgetCourseList extends AdministrateWidget {

	//  Configuration
	protected $key = 'course_list';
	protected $category = false;
	protected $subcategory = false;
	protected $course_code = false;
	
	//  Run the checkout
	public function run() {
		
		//  Extract parameters
		extract(array_merge(
			array(
				'category_id'		=> 0,
				'subcategory_id'	=> 0,
				'show_codes'    	=> true,
				'show_current_online' => $this->service->get_option('show_current_online'),
				'course_code'       => false
			), 
			$this->params
		));

		//  Set param types
		$show_codes = $this->_is_true($show_codes);

		$category_ids = $subcategory_ids = array();

		if($category_id) {
			$category_ids = array_map('intval', explode(',', $category_id));
		}

		if($subcategory_id) {
			$subcategory_ids = array_map('intval', explode(',', $subcategory_id));
		}

		// If no course parameter was passed, see if we can get a request parameter
		if ((empty($category_ids)) && (empty($subcategory_ids))) {

			if (isset($_REQUEST[$this->service->add_namespace(array('subcategory', 'id'))])) {
				$subcategory_ids = array_map('intval', explode(',', $_REQUEST[$this->service->add_namespace(array('subcategory', 'id'))]));
			}

			else if (isset($_REQUEST[$this->service->add_namespace(array('category', 'id'))])) {
				$category_ids = array_map('intval', explode(',', $_REQUEST[$this->service->add_namespace(array('category', 'id'))]));
			}
		}

		//  Initialize SOAP params
		$params = array(
			'IncludeCurrentOnline' => $show_current_online
		);

		if($course_code) {
			$params['CourseCode'] = $course_code;
		}

		if(!empty($subcategory_ids)) {
			$this->_set_courses($params, true, $subcategory_ids);
		}
		else if(!empty($category_ids)) {
			$this->_set_courses($params, false, $category_ids);
		}
		else {
			$this->_set_courses($params);
		}

		ob_start();
		include($this->get_path('/views/course_list.php'));	
		return ob_get_clean();
		
	}

	//  Set courses
	private function _set_courses($params, $is_subcategory=false, $id=null) {
		$show_empty = $this->service->to_boolean($this->service->get_option('show_empty_courses'));

		if ($is_subcategory) {
			// We make different API call when the show_empty flag is set,
			// it expect a different parameter name.
			$id_param = 'CourseSubCategoryID';
		}
		else {
			$id_param = 'CourseCategoryID';
		}

		// Add ID parameter to API call
		if ( !is_null($id) ) {
			$params[$id_param] = $id;
		}

		if ($show_empty) {
			$this->courses = Administrate::api()->get_course_list($params);
		} else {
			$this->courses = array();

			if(array_key_exists('CourseCode', $params)) {
				$params['CourseCode'] = array_map('trim', explode(',', $params['CourseCode']));
			}

			$events = Administrate::api()->get_event_list($params);
			foreach ($events as $event) {
				$key = $event->get_course_code();
				if (!isset($this->courses[$key])) {
					$course = Administrate::api()->get_course( $event->get_course_code());
					if ($course->exists()) {
						$this->courses[$key] = $course;
					}
				}
			}
			ksort($this->courses);
		}
	}

	//  Get courses
	public function get_courses() {
		return $this->courses;
	}

}
