<?php
//  Include the parent class
require_once(dirname(__FILE__) . '/WPPluginPattern/WPPluginPattern.php');

//  Administrate plugin controller class
class AdministratePlugin extends WPPluginPattern {
	protected	$_db = null;
	protected $_api_cache = null;

	//  Properties
	protected $namespace = "administrate";
	protected $tables = array(

		//  Orders table
		'orders' => array(
			'order_id' => array(
				'type'		=>	'INT',
				'length'	=>	11,
				'null'		=>	false
			),
			'order_status' => array(
				'type'		=>	'ENUM',
				'options'	=>	array('I', 'P', 'C'), // I = incomplete, P = pending (invoiced), C = complete
				'null'		=>	false,
				'default'	=>	'I'
			),
			'order_payment_type' => array(
				'type'		=>	'ENUM',
				'options'	=>	array('I', 'C'), // I = invoice, C = credit card
				'null'		=>	false,
				'default'	=>	'I'
			),
			'order_time_started' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false
			),
			'order_time_completed' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false,
				'default'	=>	0
			),
			'order_event_id' => array(
				'type'		=>	'INT',
				'length'	=>	11,
				'null'		=>	true
			),
			'order_track_id' => array(
				'type'      => 'INT',
				'length'    => 11,
				'null'      => true
			),
			'order_session_id' => array(
				'type'		=>	'CHAR',
				'length'	=>	32,
				'null'		=>	false
			),
			'order_max_step' => array(
				'type'		=>	'TINYINT',
				'length'	=>	1,
				'null'		=>	false,
				'default'	=>	1
			),
			'order_currency' => array(
				'type'		=>	'CHAR',
				'length'	=>	3,
				'null'		=>	false,
				'default'	=>	'GBP'
			),
			'order_discount' => array(
				'type'		=>	'TEXT',
				'null'		=>	true
			),
			'order_buyer_details' => array(
				'type'		=>	'TEXT',
				'null'		=>	true,
				'serialize'	=>	array(
					'first_name',
					'last_name',
					'company',
					'email',
					'phone',
					'notes'
				)
			),
			'order_invoice_address' => array(
				'type'		=>	'TEXT',
				'null'		=>	true,
				'serialize'	=>	array(
					'address',
					'city',
					'territory',
					'postal_code',
					'country'
				)
			),
			'order_num_attendees' => array(
				'type'		=>	'TINYINT',
				'length'	=>	1,
				'null'		=>	false,
				'default'	=>	1
			),
			'order_attendee_details' => array(
				'type'		=>	'TEXT',
				'null'		=>	true,
				'serialize'	=>	array(
					'first_name',
					'last_name',
					'email',
					'notes'
				),
				'multiple'	=>	true
			),
			'order_processor_transaction_id' => array(
				'type'		=>	'VARCHAR',
				'length'	=>	64,
				'null'		=>	true
			),
			'order_api_invoice_id' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false,
				'default'	=>	0
			),
			'order_item_ids' => array(
				'type'		=>	'VARCHAR',
				'length'	=> 255,
				'null'		=> true,
				'default'	=> null
			)
		),

		//  Logs table
		'logs' => array(
			'log_id' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false
			),
			'log_msg' => array(
				'type'		=>	'TEXT',
				'null'		=>	false
			),
			'log_time' => array(
				'type'		=>	'BIGINT',
				'length'	=>	20,
				'null'		=>	false
			),
			'log_order_id' => array(
				'type'		=>	'INT',
				'length'	=>	11,
				'null'		=>	false,
				'default'	=>	0
			)
		),

		//  Filters table
		'filters' => array(
			'filter_id' => array(
				'type'		=>	'INT',
				'length'	=>	11,
				'null'		=>	false
			),
			'filter_object_type' => array(
				'type'		=>	'VARCHAR',
				'length'	=>	11,
				'null'		=>	false
			),
			'filter_object_id' => array(
				'type'		=>	'VARCHAR',
				'length'	=>	10,
				'null'		=>	false
			),
			'filter_hidden' => array(
				'type'		=>	'TINYINT',
				'length'	=>	4,
				'null'		=>	false,
				'default'	=>	0
			),
			'filter_url_string' => array(
				'type'		=>	'VARCHAR',
				'length'	=>	128,
				'null'		=>	false
			),
			'filter_object_path' => array(
				'type'		=>	'VARCHAR',
				'length'	=>	256
			),
			'filter_object_keywords' => array(
				'type'		=>	'VARCHAR',
				'length'	=>	256
			),
			'filter_object_description' => array(
				'type'		=>	'VARCHAR',
				'length'	=>	256
			)
		)

	);
	protected $oldTables = array(
		'sessions'	=>	'administrateSession',
		'orders'	=>	'administrateOrders',
		'logs'		=>	'administrateLogs'
	);
	protected $subdomainPlaceholder = '<subdomain>';
	protected $domainPlaceholder = '<domain>';
	protected $cacheFlusherLaunched = false;

	//  Constructor
	public function __construct($config, &$wpdb, $debug = false) {
		parent::__construct($config, $wpdb, $debug);

		//  Setup database wrapper for the Wordpress database object
		$this->_db = new AdministrateWordpressDB($wpdb);
		$this->_db_prefix = $wpdb->prefix . "administrate_";

		Administrate::log()->set_debug($debug);
		$this->init_api();

		//  If running in the admin, include and initialize admin controller
		if (is_admin()) {

			//  Activate just in case
			if ($this->is_admin_page()) {
				$this->activate();
			}

			//  Initialize the admin controller
			require_once($this->get_path('/AdministratePluginAdmin.php'));
			$this->controller = new AdministratePluginAdmin($this);

		//  Or else include and initialize the public controller
		} else {
			require_once($this->get_path('/AdministratePluginPublic.php'));
			$this->controller = new AdministratePluginPublic($this);
		}

		// Construct plugin monitor
		$this->monitor = new AdministrateWordpressMonitor($this, $this->_db);

		//  Register activation / deactivation hooks
		register_activation_hook($config['plugin']['file'], array($this, 'activate'));
		register_deactivation_hook($config['plugin']['file'], array($this, 'deactivate'));
	}

	public function setup_tables() {

		// Default this option to 0 if it doesn't exist
		if(get_option('adm_db_version') == false) {
			add_option('adm_db_version', 0);
		}

		// If the version we specify doesn't match the saved version, updates are ready.
		// Documentation for this method:
		// http://codex.wordpress.org/Creating_Tables_with_Plugins#Creating_or_Updating_the_Table
		// NOTE:
		// The Wordpress function dbDelta requires that there is double space between
		// "PRIMARY KEY" and the key name, otherwise primary keys are not updated
		require_once(ABSPATH.'wp-admin/includes/upgrade.php');

		$sql = "CREATE TABLE {$this->_db_prefix}filters (
						filter_id int(11) NOT NULL AUTO_INCREMENT,
						filter_object_type varchar(11) NOT NULL,
						filter_object_id varchar(64) NOT NULL,
						filter_hidden tinyint(4) NOT NULL DEFAULT '0',
						filter_url_string varchar(128) NOT NULL,
						filter_object_path varchar(256) DEFAULT NULL,
						filter_object_title varchar(256) DEFAULT NULL,
						filter_object_keywords varchar(256) DEFAULT NULL,
						filter_object_description varchar(256) DEFAULT NULL,
						PRIMARY KEY  (filter_id)
					);";

		// Check the new schema against the present one and migrate.
		dbDelta($sql);

		$sql = "CREATE TABLE {$this->_db_prefix}logs (
						log_id bigint(20) NOT NULL AUTO_INCREMENT,
						log_msg text NOT NULL,
						log_time bigint(20) NOT NULL,
						log_order_id int(11) NOT NULL DEFAULT '0',
						PRIMARY KEY  (log_id)
					)";

		// Check the new schema against the present one and migrate.
		dbDelta($sql);

		$sql = "CREATE TABLE {$this->_db_prefix}live_monitor (
						id bigint(20) NOT NULL AUTO_INCREMENT,
						log_date timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
						log text NOT NULL,
						PRIMARY KEY  (id)
		)";
		dbDelta($sql);


		$sql = "CREATE TABLE {$this->_db_prefix}orders (
						order_id int(11) NOT NULL AUTO_INCREMENT,
						order_status enum('I','P','C') NOT NULL DEFAULT 'I',
						order_payment_type enum('I','C') NOT NULL DEFAULT 'I',
						order_time_started bigint(20) NOT NULL,
						order_time_completed bigint(20) NOT NULL DEFAULT '0',
						order_event_id int(11) NULL DEFAULT NULL,
						order_track_id int(11) NULL DEFAULT NULL,
						order_item_ids varchar(255) NULL DEFAULT NULL,
						order_session_id char(32) NOT NULL,
						order_max_step tinyint(1) NOT NULL DEFAULT '1',
						order_currency char(3) NOT NULL DEFAULT 'GBP',
						order_discount text,
						order_buyer_details text,
						order_invoice_address text,
						order_num_attendees tinyint(1) NOT NULL DEFAULT '1',
						order_attendee_details text,
						order_processor_transaction_id varchar(64) DEFAULT NULL,
						order_api_invoice_id bigint(20) NOT NULL DEFAULT '0',
						order_being_processed int(1) NOT NULL DEFAULT '0',
						order_path_objectives mediumtext,
						PRIMARY KEY  (order_id)
					)";

		// Check the new schema against the present one and migrate.
		dbDelta($sql);

		// Drop old caching table if present
		$this->_db->query("DROP TABLE IF EXISTS {$this->_db_prefix}administrate_caches;");

		// Create caching tables
		foreach( AdministrateCache::get_table_structure($this->_db_prefix) as $table_sql) {
			dbDelta($table_sql);
		}

		if( isset($this->db_version) ) {
			update_option('adm_db_version', $this->db_version);
		}
	}

	//  Display the price widget
	public function display_price_widget($event, $fieldName, $defaultCurrency = '', $showCurrencyIndicator = true, $currencyIndicator = 'symbol') {
		$this->controller->display_price_widget($event, $fieldName, $defaultCurrency, $showCurrencyIndicator);
	}

	//  Display the currency selector
	public function display_currency_selector($events, $defaultCurrency = '', $fieldName = '') {
		$this->controller->display_currency_selector($events, $defaultCurrency, $fieldName);
	}

	//  Display the currency prices
	public function display_currency_prices($event, $pricingBasis = false, $defaultCurrency = '', $showCurrencyIndicator = true, $currencyIndicator = 'symbol') {
		$this->controller->display_currency_prices($event, $pricingBasis, $defaultCurrency, $showCurrencyIndicator, $currencyIndicator);
	}

	//  Get currency symbol
	public function get_currency_symbol($currency) {
		return $this->controller->get_currency_symbol($currency);
	}

	//  Format a currency
	public function format_currency($amount, $currency, $showCurrencyIndicator = true, $currencyIndicator = 'symbol') {
		return $this->controller->format_currency($amount, $currency, $showCurrencyIndicator, $currencyIndicator);
	}

	//  Format a datetime
	public function format_datetime($ts, $format = 'M j @ g:ia') {
		return date_i18n($format, $ts);
	}

	public function get_docs_url() {
		$urls_conf = $this->get_config("plugin", "urls");
		return $urls_conf["docs"];
	}

	//  Get the plugin update URL
	public function get_update_url() {
		$urls_conf = $this->get_config("plugin", "urls");
		return $urls_conf["update"];
	}

	//  Get the registration URL of an event
	public function get_registration_url($eventId=0, $currency=false, $track_id=null) {
		$url = get_permalink($this->get_option('checkout_page', 'checkout'));
		if (strpos($url, '?') > -1) {
			$url .= '&';
		} else {
			$url .= '?';
		}

		if(is_null($track_id)) {
			$url .= $this->add_namespace(array('checkout', 'event', 'id')) . '=' . $eventId;
		} else {
			$url .= $this->add_namespace(array('checkout', 'track', 'id')) . '=' . $track_id;
		}

		if($currency) {
			$url .=  "&event_currency=" . $currency;
		}

		// $url is now a complete URL - began with get_permalink
		return $url;
	}

	//  Get the course url structure
	public function get_course_url_structure() {
		if (!property_exists($this, 'courseUrlStructure')) {
			$this->courseUrlStructure = $this->get_option('course_url_structure', 'course');
		}
		return $this->courseUrlStructure;
	}

	//  Get the course URL structure parts
	public function get_course_url_parts() {
		if (!property_exists($this, 'courseUrlParts')) {
			$this->courseUrlParts = explode('-', $this->get_course_url_structure());
		}
		return $this->courseUrlParts;
	}

	//  Get the course page
	public function get_course_page() {
		if (!property_exists($this, 'coursePage')) {
			$this->coursePage = $this->get_option('course_page', 'course');
		}
		return $this->coursePage;
	}

	//  Get the course page URL
	public function get_course_page_base_url() {
		if (!property_exists($this, 'coursePageBaseUrl')) {

			// Detect presence of http/https. get_permalink always uses url as defined
			// in settings, however, home_url() will return url being used (if overridden
			// by .htaccess for example)
			if(substr(get_permalink($this->get_course_page()), 0, 5) == 'https') {
				$site_url = home_url('', 'https');
			} else {
				$site_url = home_url('', 'http');
			}
			$this->coursePageBaseUrl = substr(get_permalink($this->get_course_page()), strlen($site_url));
		}
		return $this->coursePageBaseUrl;
	}

	//  Whether or not to use pretty course URLs
	public function use_custom_course_urls() {
		if (!property_exists($this, 'customCourseUrls')) {
			$this->customCourseUrls = ((strlen($this->get_course_url_structure()) > 0) && (strlen(get_option('permalink_structure')) > 0));
		}
		return $this->customCourseUrls;
	}

	//  Get the info URL of a course
	public function get_course_url($course, $subcategory = false, $category = false, $forceGenerate = false) {

		//  If there is a custom structure, generate it
		if ($this->use_custom_course_urls()) {

			//  If the course object wasn't passed in, query for it
			if (!is_object($course)) {
				$course = Administrate::api()->get_course( $course);
			}

			//  See if there is already a saved filter path
			require_once($this->get_path('/AdministrateCourseFilter.php'));
			$filter = new AdministrateCourseFilter(
				$this,
				array(
					'filter_object_type'	=>	'course',
					'filter_object_id'		=>	$course->get_code(),
					'filter_hidden'			=>	0
				)
			);

			//  If the filter exists, return the cached path
			if ($filter->exists() && !$forceGenerate) {
				// return the full URL built from the filter
				return home_url($filter->get_object_path());

			//  Otherwise continue to generate it
			} else {

				//  Generate the URL
				$params = array();
				if ($category) {
					$params['category'] = $category;
				} else if (isset($_REQUEST[$this->add_namespace(array('category', 'id'))])) {
					$params['category'] = Administrate::api()->get_category($_REQUEST[$this->add_namespace(array('category', 'id'))]);
				} else {
					$categories = $course->get_categories();
					if (count($categories) > 0) {
						$params['category'] = $categories[0];
					}
				}
				if ($subcategory) {
					$params['subcategory'] = $subcategory;
				} else if (isset($_REQUEST[$this->add_namespace(array('subcategory', 'id'))])) {
					$params['subcategory'] = Administrate::api()->get_subcategory($_REQUEST[$this->add_namespace(array('subcategory', 'id'))]);
				} else if (isset($params['category'])) {
					$subcategories = $params['category']->get_subcategories();
					if (count($subcategories) > 0) {
						$params['subcategory'] = $subcategories[0];
					}
				}
				$params['course'] = $course;
				// get_course_page_url returns a full URL
				return $this->get_course_page_url($params);

			}

		//  Otherwise just pass the course ID
		} else {
			// get_course_page_url returns a full URL
			return $this->get_course_page_url(array('course'=>$course));
		}

	}

	//  Get the info URL of a subcategory
	public function get_subcategory_url($subcategory, $category = false, $forceGenerate = false) {

		//  If there is a custom structure, generate it
		if ($this->use_custom_course_urls()) {

			//  If the subcategory object wasn't passed in, query for it
			if (!is_object($subcategory)) {
				$subcategory = Administrate::api()->get_subcategory($subcategory);
			}

			//  See if there is already a saved filter path
			require_once($this->get_path('/AdministrateCourseFilter.php'));
			$filter = new AdministrateCourseFilter(
				$this,
				array(
					'filter_object_type'	=>	'subcategory',
					'filter_object_id'		=>	$subcategory->get_id(),
					'filter_hidden'			=>	0
				)
			);

			//  If the filter exists, return the cached path
			if ($filter->exists() && !$forceGenerate) {
				// return the full URL built from the filter
				return home_url($filter->get_object_path());

			//  Otherwise continue to generate it
			} else {

				//  Set the params
				$params = array();
				if ($category) {
					$params['category'] = $category;
				} else if (isset($_REQUEST[$this->add_namespace(array('category', 'id'))])) {
					$params['category'] = Administrate::api()->get_category($_REQUEST[$this->add_namespace(array('category', 'id'))]);
				} else {
					$params['category']	= $subcategory->get_parent_category();
				}
				$params['subcategory'] = $subcategory;

				//  Return the subcategory URL
				return $this->get_course_page_url($params);

			}

		//  Otherwise just pass the course ID
		} else {
			return $this->get_course_page_url(array('subcategory'=>$subcategory));
		}

	}

	//  Get the info URL of a category
	public function get_category_url($category, $forceGenerate = false) {

		//  If there is a custom structure, generate it
		if ($this->use_custom_course_urls()) {

			//  If the category isn't an object, query for it
			if (!is_object($category)) {
				$category = Administrate::api()->get_category($category);
			}

			//  See if there is already a saved filter path
			require_once($this->get_path('/AdministrateCourseFilter.php'));
			$filter = new AdministrateCourseFilter(
				$this,
				array(
					'filter_object_type'	=>	'category',
					'filter_object_id'		=>	$category->get_id(),
					'filter_hidden'			=>	0
				)
			);

			//  If the filter exists, return the cached path
			if ($filter->exists() && !$forceGenerate) {
				// return the full URL built from the filter
				return home_url($filter->get_object_path());

			//  Or else generate the URL
			} else {
				return $this->get_course_page_url(array('category'=>$category));
			}

		//  Or else just return the normal URL
		} else {
			return $this->get_course_page_url(array('category'=>$category));
		}

	}

	//  Get a course page URL
	public function get_course_page_url($params = array()) {

		//  Set the base URL
		$url = $this->get_course_page_base_url();

		//  If there is a custom structure, generate it
		if ($this->use_custom_course_urls()) {

			//  Include the course filter class
			require_once($this->get_path('/AdministrateCourseFilter.php'));

			//  Split the structure into parts
			$urlStructureParts = $this->get_course_url_parts();

			//  Loop through URL structure backwards to find the first identifier available
			$objectSuffix = '';
			for ($i = count($urlStructureParts)-1; $i >= 0; --$i) {

				//  If the part matches the passed key, generate the URL from here
				if (isset($params[$urlStructureParts[$i]])) {

					//  Get the associated filter
					if ($urlStructureParts[$i] == 'course') {
						$objectId = $params[$urlStructureParts[$i]]->get_code();
					} else {
						$objectId = $params[$urlStructureParts[$i]]->get_id();
					}
					$filter = new AdministrateCourseFilter(
						$this,
						array(
							'filter_object_type'	=>	$urlStructureParts[$i],
							'filter_object_id'		=>	$objectId,
							'filter_hidden'			=>	0
						)
					);

					if ($filter->exists()) {
						// Get filter url data
						$url_string = $filter->get_url_string();

						if($urlStructureParts[$i] == 'course') {
								// If filtered by course, encode the course ID
								// to ensure & and other special characters don't break the filter
								$url_string = urlencode($url_string);
						}
						// Concatenate filter parts
						$objectSuffix = $url_string . '/' . $objectSuffix;
					}

				}

			}

			// Add the suffix to the URL.
			// Dependant on config, $url may or may not have a trailing slash at this point, so we make sure it does.
			if(substr($url, -1) != '/') {
				$url .= '/';
			}

			$url .= $objectSuffix;

		//  Otherwise just add the query string
		} else {

			//  Add the appropriate query string prefix
			if (strpos($url, '?') > -1) {
				$url .= '&';
			} else {
				$url .= '?';
			}

			//  Loop through params and add them to the query string
			foreach ($params as $key=>$val) {
				if( !is_object($params[$key]) ) {
					$objectId = $params[$key];
				} else if ($key == 'course') {
					$objectId = $params[$key]->get_code();
				} else {
					$objectId = $params[$key]->get_id();
				}
				$url .= $this->add_namespace(array($key, 'id')) . '=' . urlencode($objectId);
			}

		}

		// Return the full site url for the relative URL we have generated
		return home_url($url);

	}

	//  Whether or not the filter object is supposed to be hidden
	public function is_hidden($objectType, $objectId) {

		//  Include the course filter class
		require_once($this->get_path('/AdministrateCourseFilter.php'));

		//  Get the associated filter
		$filter = new AdministrateCourseFilter(
			$this,
			array(
				'filter_object_type'	=>	$objectType,
				'filter_object_id'		=>	$objectId,
				'filter_hidden'			=>	1
			)
		);

		//  Return a boolean
		return $filter->exists();

	}

	public function get_learning_track_page() {
		return $this->controller->get_learning_track_page();
	}

	//  Get event field values based on passed list (array) or field names
	public function get_event_fields($event, $fields) {

		//  If an array of fields wasn't passed, try unserializing it
		if (!is_array($fields)) {
			$fields = unserialize($fields);
		}
		if (empty($fields)) {
			$fields = array();
		}

		//  Add the requested fields to the values
		$values = array();
		if (in_array('code', $fields)) {
			$values['code'] = $event->get_course_code();
		}
		if (in_array('summary', $fields)) {
			$values['summary'] = $event->get_course_summary();
		}
		if (in_array('schedule', $fields)) {
			$values['schedule'] = $event->get_course_schedule();
		}
		if (in_array('location', $fields)) {
			$values['location'] = $event->get_location();
		}
		if (in_array('inclusions', $fields)) {
			$values['inclusions'] = $event->get_course_inclusions();
		}
		if (in_array('method', $fields)) {
			$values['method'] = $event->get_course_method();
		}
		if (in_array('prerequisites', $fields)) {
			$values['prerequisites'] = $event->get_course_prerequisites();
		}
		if (in_array('topics', $fields)) {
			$values['topics'] = $event->get_course_topics();
		}
		if (in_array('benefits', $fields)) {
			$values['benefits'] = $event->get_course_benefits();
		}
		if (in_array('duration', $fields)) {
			$values['duration'] = $event->get_course_duration();
		}

		//  Return the values
		return $values;

	}

	//  Get course field values based on passed list (array) or field names
	public function get_course_fields(&$course, $fields) {

		//  If an array of fields wasn't passed, try unserializing it
		if (!is_array($fields)) {
			$fields = unserialize($fields);
		}
		if (empty($fields)) {
			$fields = array();
		}

		//  Add the requested fields to the values
		$values = array();
		if (in_array('code', $fields)) {
			$values['code'] = $course->get_code();
		}
		if (in_array('summary', $fields)) {
			$values['summary'] = $course->get_summary();
		}
		if (in_array('schedule', $fields)) {
			$values['schedule'] = $course->get_schedule();
		}
		if (in_array('inclusions', $fields)) {
			$values['inclusions'] = $course->get_inclusions();
		}
		if (in_array('method', $fields)) {
			$values['method'] = $course->get_method();
		}
		if (in_array('prerequisites', $fields)) {
			$values['prerequisites'] = $course->get_prerequisites();
		}
		if (in_array('topics', $fields)) {
			$values['topics'] = $course->get_topics();
		}
		if (in_array('benefits', $fields)) {
			$values['benefits'] = $course->get_benefits();
		}
		if (in_array('custom_text_8', $fields)) {
			$values['custom_text_8'] = $course->get_custom_text_8();
		}
		if (in_array('duration', $fields)) {
			$values['duration'] = $course->get_duration();
		}

		//  Return the values
		return $values;

	}

	/**
	 * @param bool $where - false or an array of 'where' clause, e.g. 'order_id' => 4
	 * @param int $start - start results at N'th value - default 0
	 * @param int $limit - show only X results - defaults to MAX_INT (all results)
	 * @return array - array of AdministrateWidgetCheckoutOrder objects
	 */
	public function get_orders($where = false, $start = 0, $limit = PHP_INT_MAX) {

		// Always sort orders by time started
		$order = array('order_time_started' => 'DESC');
		require_once($this->get_path('/widgets/checkout/AdministrateWidgetCheckoutOrder.php'));
		$dao = new AdministrateWidgetCheckoutOrder($this);
		return $dao->get_all($where, $order, $start, $limit);
	}

	/**
	 * @param bool $where - false or an array of 'where' clause, e.g. 'order_id' => 4
	 * @return int - number of items matching this condition
	 */
	public function count_orders($where = false) {
		if(!$where) {

			// No where specified - return all orders
			$sql = "SELECT COUNT(order_id) AS Count FROM " . $this->get_orders_table() . " WHERE order_max_step >= 3;";
		} else {

			// Where specified - build parameterized query string
			$where_string = " WHERE ";
			foreach($where as $field => $value) {
				$where_items[] = $field . ' = %s';
				$params[] = $value;
			}
			$where_items[] = 'order_max_step >= 3';
			$where_string .= implode(' AND ', $where_items);

			// Prepare the statement
			$sql = $this->db->prepare(
				"SELECT COUNT(order_id) AS Count FROM " . $this->get_orders_table() . $where_string,
				$params
			);
		}

		$count = $this->db->get_row($sql, 'ARRAY_A');

		return $count['Count'];
	}

	/**
	 * Deletes orders from the database which are sat at step 1 or 2 and older than the age specified.
	 * @param $purge_age - orders over than this age in seconds should be purged
	 */
	public function purge_orders($purge_age) {

		// Clean up order
		$sql = $this->db->prepare(<<<SQL

DELETE
	wp_orders.*,
	wp_logs.*
FROM
	{$this->get_orders_table()} wp_orders
	JOIN {$this->get_logs_table()} wp_logs ON wp_orders.order_id = wp_logs.log_order_id
WHERE
	order_time_started < UNIX_TIMESTAMP() - %s
	AND order_max_step <= 2;

SQL
		, array($purge_age));

		$this->db->query($sql);
	}

	/**
	 * @return array - list of event ID's which an order exists for
	 */
	public function get_ordered_event_ids() {
		// No need to paramaterize - get_orders_table is specified
		// in code - user's can't determine this.
		$sql = "SELECT DISTINCT(order_event_id) FROM " . $this->get_orders_table();
		return $this->db->get_results($sql, 'ARRAY_A');
	}

	//  Get the number of places available for an event
	public function get_event_num_places(&$event) {
		return Administrate::api()->get_num_places($event);
	}

	//  Get API URL
	public function get_api_url($mode, $domain) {
		$serviceConfig = $this->get_config('service');
		//return str_replace($this->subdomainPlaceholder, $subdomain, $serviceConfig['urls'][$mode]);
		return str_replace($this->domainPlaceholder, $domain, $serviceConfig['urls'][$mode]);
	}

	/**
	 * Initialise the API, set up credentials and cache.
	 */
	public function init_api() {

		// Fetch api config from the WP config.
		$apiOptions = get_option($this->add_namespace('api_options'));

		// If running in live mode then grab the real credentials, otherwise load service config and grab demo creds.
		if($apiOptions['mode'] == 'live') {
			$user = $apiOptions['user'];
			$password = $apiOptions['password'];
		} else {
			$service_config = $this->get_config('service');
			$user = $service_config['demoUser'];
			$password = $service_config['demoPassword'];
		}

		$soap_url = $this->get_api_url($apiOptions['mode'], $apiOptions['domain']);

		// REST URL
		// TODO (Database migration)
		// Database record only stores SOAP url,
		// Add record for REST instead of this hard-coded suffix.
		$url_parts = parse_url($soap_url);
		$rest_url = $url_parts['scheme'] . "://" . $url_parts['host'] . "/api/v2/weblink/";
		$internal_api_url = $url_parts['scheme'] . "://" . $url_parts['host'] . "/beta/api/weblink/";

		// Create API credentials
		$api_credentials = new AdministrateCredentials(
			$rest_url, $soap_url,
			$user, $password,
			false, $internal_api_url
		);

		Administrate::api()->set_credentials($api_credentials);

		// Create cache and perform cache sweep
		//
		$cache_time = intval($this->get_option('cache_timeout', 'api'));

		if ($cache_time) {
			$this->_api_cache = new AdministrateCache($this->_db, $this->_db_prefix, "wp", $cache_time);
			Administrate::api()->set_cache($this->_api_cache);

			$lcs = get_option('last_cache_sweep');

			if( !$lcs ) {
				add_option('last_cache_sweep', time() );
				$lcs = time();
			}

			if( $this->_api_cache->delete_expired($lcs) ) {
				update_option('last_cache_sweep', time() );
			}
		}

	}

	// Disable plugin cache
	public function disable_cache() {
		Administrate::api()->set_cache(null);
	}

	// Enable plugin cache
	public function enable_cache() {
		Administrate::api()->set_cache($this->_api_cache);
	}

	//  Purge the cache
	public function purge_cache() {
		AdministrateCache::purge($this->_db, $this->_db_prefix);

		if (isset($_POST['purge_cache'])) {
			$this->show_notice(__('You have successfully purged the cache.', 'administrate'));
		}
	}

	//  Build the cache
	public function build_cache() {

		//  Increase execution time limit
		set_time_limit(1800);

		//  Get unique categories
		$categories = Administrate::api()->get_category_list();

		//  Loop through categories
		foreach ($categories as $category) {

			//  Get the subcategories
			$subcategories = $category->get_subcategories();
		}

		//  Get all courses
		$courses = Administrate::api()->get_course_list();

		//  Get all locations
		$locations = Administrate::api()->get_location_list();

		//  Get all events in the next 12 months
		$events = Administrate::api()->get_event_list();

		//  Loop through events
		$courseCodes = array();
		foreach ($events as $event) {

			//  If we haven't queried for the course by code already, do it now
			if ( in_array($event->get_course_code(), $courseCodes) ) {
				continue;
			}

			$course = Administrate::api()->get_course($event->get_course_code());
			$courseCodes[] = $course;
		}
	}

	//  Launch cache flusher via AJAX
	public function launch_cache_flusher() {
		?>
		<script>
		administrateCacheFlusher = '<?php echo  wp_login_url(); ?>';
		</script>
		<?php
	}

	//  Refresh course page URLs
	public function refresh_urls() {

		//  Only proceed if there is a custom URL structure defined
		if ($this->use_custom_course_urls()) {

			//  Increase execution time limit
			set_time_limit(1800);

			//  Include the course filter object
			require_once($this->get_path('/AdministrateCourseFilter.php'));

			//  Loop through categories
			foreach (Administrate::api()->get_category_list() as $category) {

				//  Attempt to query for the filter
				$filter = new AdministrateCourseFilter(
					$this,
					array(
						'filter_object_type'	=>	'category',
						'filter_object_id'		=>	$category->get_id()
				));

				//  If the filter doesn't exist, create it
				if (!$filter->exists()) {
					$filter->create(array(
						'filter_object_type'	=>	'category',
						'filter_object_id'		=>	$category->get_id(),
						'filter_url_string'		=>	$category->get_name()
					));
				}

				// Get full URL for this category
				$full_category_url = $this->get_category_url($category, true);

				// We save only the relative part of the URL - when we render it
				// we'll calculate the full URL including hostname.
				$relative_category_url = substr($full_category_url, strlen(home_url()));

				// Don't to try save url's which would overwrite the root page
				if($relative_category_url != '/') {
					$filter->save(array(
						'filter_object_path' => $relative_category_url
					));
				} else {
					$filter->delete();
				}

				//  Loop through subcategories
				foreach ($category->get_subcategories() as $subcategory)  {

					//  Attempt to query for the filter
					$filter = new AdministrateCourseFilter(
						$this,
						array(
							'filter_object_type'	=>	'subcategory',
							'filter_object_id'		=>	$subcategory->get_id()
						)
					);

					//  If the filter doesn't exist, create it
					if (!$filter->exists()) {
						$filter->create(array(
							'filter_object_type'	=>	'subcategory',
							'filter_object_id'		=>	$subcategory->get_id(),
							'filter_url_string'		=>	$subcategory->get_name()
						));
					}

					// Get full URL for this subcategory
					$full_subcategory_url = $this->get_subcategory_url($subcategory, $category, true);

					// We save only the relative part of the URL - when we render it
					// we'll calculate the full URL including hostname.
					$relative_subcategory_url = substr($full_subcategory_url, strlen(home_url()));

					// Don't to try save url's which would overwrite the root page
					if($relative_subcategory_url != '/') {
						$filter->save(array(
							'filter_object_path' => $relative_subcategory_url
						));
					} else {
						$filter->delete();
					}

				}

			}

			//  Loop through courses
			foreach (Administrate::api()->get_course_list() as $course) {

				//  Attempt to query for the filter
				$filter = new AdministrateCourseFilter(
					$this,
					array(
						'filter_object_type'	=>	'course',
						'filter_object_id'		=>	$course->get_code()
					)
				);

				//  If the filter doesn't exist, create it
				if (!$filter->exists()) {
					$filter->create(array(
						'filter_object_type'	=>	'course',
						'filter_object_id'		=>	$course->get_code(),
						'filter_url_string'		=>	$course->get_code() . ' ' . $course->get_title(),
					));
				}

				// Get full URL for this course
				$full_course_url = $this->get_course_url($course, false, false, true);

				// We save only the relative part of the URL - when we render it
				// we'll calculate the full URL including hostname.
				$relative_course_url = substr($full_course_url, strlen(home_url()));

				// Don't to try save url's which would overwrite the root page
				if($relative_course_url != '/') {
					$filter->save(array(
						'filter_object_path' => $relative_course_url
					));
				} else {
					$filter->delete();
				}

			}

			$this->show_notice(__('You have successfully refreshed the custom course URLs.', 'administrate'));

		//  Otherwise notify the user that there are no custom URLs
		} else if (isset($_POST['refresh_urls'])) {

			$this->show_notice(__('You have not set a custom course URL structure, so there is nothing to do.', 'administrate'));

		}

	}

	//  Save course URLs
	public function save_urls($params = array()) {
		update_option("administrate_seo_title_mode", $params['seo_title_mode']);
		update_option("administrate_seo_title_separator", $params['seo_title_separator']);

		//  Increase execution time limit
		set_time_limit(1800);

		//  Include the course filter
		require_once($this->get_path('/AdministrateCourseFilter.php'));

		//  Loop through the submitted IDs in reverse order to set categories/subcategories first
		for ($i = count($params['ids'])-1; $i >= 0; --$i) {

			//  Initialize the filter
			$filter = new AdministrateCourseFilter($this, $params['ids'][$i]);

			//  Figure out the value for the hidden field
			$hidden = 0;
			if (isset($params['hidden_ids']) && is_array($params['hidden_ids']) && in_array(strval($filter->get_id()), $params['hidden_ids'])) {
				$hidden = 1;
			}

			//  Set the fields to be saved
			$fields = array(
				'filter_hidden'				=>	$hidden,
				'filter_object_keywords'	=>	$params['keywords'][$i],
				'filter_object_description'	=>	$params['descriptions'][$i],
				'filter_object_title'		=>	$params['titles'][$i]
			);

			//  Only update the URL string if it's not empty
			$urlString = $params['url_strings'][$i];
			if (empty($urlString)) {
				if ($filter->get_object_type() == 'course') {
					$course = $course = Administrate::api()->get_course( $filter->get_object_id());
					$fields['filter_url_string'] = $course->get_code() . ' ' . $course->get_title();
				} else if ($filter->get_object_type() == 'subcategory') {
					$subcategory = Administrate::api()->get_subcategory($filter->get_object_id());
					$fields['filter_url_string'] = $subcategory->get_name();
				} else if ($filter->get_object_type() == 'category') {
					$category = Administrate::api()->get_category($filter->get_object_id());
					$fields['filter_url_string'] = $category->get_name();
				}
			} else {
				$fields['filter_url_string'] = $urlString;
			}

			//  Save the filter
			$filter->save($fields);

		}

		//  Refresh all the object paths
		$this->refresh_urls();

		//  Show a message
		$this->show_notice(__('You have successfully updated SEO settings &amp; filters.', 'administrate'));

	}

	//  Install the plugin
	public function activate() {

		//  Call parent
		$result = parent::activate();

		//  Update the service mode to 'demo' if it is currently 'test'
		$mode = $this->get_option('mode', 'api');
		if ($mode == 'test') {
			$this->update_option('mode', 'api', 'demo');
		}

		//  If the domain API option doesn't exist but the subdomain does, automatically add the domain option
		$domain = $this->get_option('domain', 'api');
		$subdomain = $this->get_option('subdomain', 'api');
		if (empty($domain) && !empty($subdomain)) {
			$this->update_option('domain', 'api', $subdomain . '.administrateapp.com');
		}

		$this->setup_tables();

		//  Return the result
		return $result;

	}

	//  Migrate log data
	protected function _migrate_data() {

		//  Set the old tables
		foreach ($this->oldTables as $key=>$table) {
			$this->oldTables[$key] = $this->db->prefix.$table;
		}

		//  Check to see if the old tables exist
		$migrate = false;
		foreach ($this->oldTables as $table) {
			$tableExists = $this->db->query("SHOW TABLES LIKE '" . $table . "'");
			if ($tableExists) {
				$migrate = true;
				break;
			}
		}

		//  If any of the old tables exist, migrate
		if ($migrate) {

			//  Migrate the sessions / orders first because we'll need some data for the logs table
			$sessions = $this->db->get_results("SELECT * FROM " . $this->oldTables['sessions'] . " ORDER BY orderReference ASC");

			//  Loop through sessions
			if ($sessions) {
				$maxOrderId = 0;
				foreach ($sessions as $session) {

					//  Only proceed if there is an order reference
					if (!empty($session->orderReference)) {

						//  Extract the order ID
						$tmp = explode(":", $session->orderReference);
						$orderId = intval($tmp[1]);

						//  Get the order associated with the session
						$order = $this->db->get_row("SELECT * FROM " . $this->oldTables['orders'] . " WHERE orderId = " . $orderId . " LIMIT 1");

						//  Extract the buyer info and serialize
						$tmp = explode('^', $session->buyer);
						$buyerDetails = serialize(array(
							'first_name'	=>	$tmp[0],
							'last_name'		=>	$tmp[1],
							'company'		=>	$tmp[2],
							'email'			=>	$tmp[3],
							'phone'			=>	$tmp[4]
						));

						//  Extract the invoice address info and serialize
						$tmp = explode('^', $session->invoiceAddress);
						$invoiceAddress = serialize(array(
							'address'		=>	$tmp[0],
							'city'			=>	$tmp[1],
							'territory'		=>	$tmp[2],
							'postal_code'	=>	$tmp[3]
						));

						//  Extract the attendee info and serialize
						$tmp = explode('^', $session->attendees);
						$attendees = array();
						foreach ($tmp as $attendee) {
							if (!empty($attendee)) {
								$tmp2 = explode(':', $attendee);
								array_push(
									$attendees,
									array(
										'name'	=>	$tmp2[0],
										'email'	=>	$tmp2[1]
									)
								);
							}
						}
						$attendeeDetails = serialize($attendees);

						//  Insert the data into the new orders table
						$this->db->insert(
							$this->get_orders_table(),
							array(
								'order_id'						=>	$orderId,
								'order_status'					=>	$order->orderStatus,
								'order_payment_type'			=>	$order->orderType,
								'order_time_started'			=>	0,
								'order_time_completed'			=>	strtotime($order->orderDate),
								'order_event_id'				=>	$order->eventId,
								'order_session_id'				=>	$session->sessionKey,
								'order_max_step'				=>	$session->phmax,
								'order_currency'				=>	$session->currency,
								'order_discount'				=>	$session->discount,
								'order_buyer_details'			=>	$buyerDetails,
								'order_invoice_address'			=>	$invoiceAddress,
								'order_num_attendees'			=>	$session->numberOfAttendees,
								'order_attendee_details'		=>	$attendeeDetails,
								'order_paypal_transaction_id'	=>	$order->paypalTransactionId
							)
						);

						//  Keep track of max order ID
						if ($orderId > $maxOrderId) {
							$maxOrderId = $orderId;
						}

					}

				}

				//  Set the auto increment start to the maximum order ID
				$this->db->query('ALTER TABLE ' . $this->get_orders_table() . ' AUTO_INCREMENT = ' . $maxOrderId);

			}

			//  Get all the data from the previous log table
			$logs = $this->db->get_results("SELECT * FROM " . $this->oldTables['logs'] . " ORDER BY logDate ASC");

			//  Loop through logs and insert them into the new logs table
			if ($logs) {
				$maxLogId = 0;
				foreach ($logs as $log) {

					//  Extract the order ID
					if (empty($log->orderReference)) {
						$orderId = 0;
					} else {
						$tmp = explode(":", $log->orderReference);
						$orderId = intval($tmp[1]);
					}

					//  Insert the data into the new logs table
					$this->db->insert(
						$this->get_logs_table(),
						array(
							'log_id'		=>	$log->logId,
							'log_msg'		=>	$log->logMsg,
							'log_time'		=>	strtotime($log->logDate),
							'log_order_id'	=>	$orderId
						)

					);

					//  Keep track of the max log ID
					if ($log->logId > $maxLogId) {
						$maxLogId = $log->logId;
					}

				}

				//  Set the auto increment based on max log ID
				$this->db->query('ALTER TABLE ' . $this->get_logs_table() . ' AUTO_INCREMENT = ' . $maxLogId);

			}

			//  Finally, delete all the old tables
			foreach ($this->oldTables as $table) {
				$this->db->query('DROP TABLE ' . $table);
			}

		}

	}

	//  Test API connection
	public function test_api_connection() {

		$start_t = microtime(true);
		$success = Administrate::api()->raw_call("getEvents", array(), false);
		$end_t = microtime(true);
		$ms = number_format($end_t - $start_t, 4);

		if ( $success )
		{
			$numMs = number_format($ms, 4);
			$this->show_notice(__('Connection <strong>successful</strong> in <strong>' . $numMs . ' seconds</strong>.', 'administrate'));
		} else {
			$this->show_error(__('Connection <strong>failed</strong>.', 'administrate'));
		}
	}

	//  Test API performance
	public function test_api_performance() {
		$methods = array(
			'get_event_list',
			'get_category_list',
			'get_course_list',
			'get_location_list'
		);
		$msg = 'Performance tests: ';
		foreach ($methods as $method) {
			$startTime = microtime(true);
			$result = $this->call($method, array(), true);
			$endTime = microtime(true);
			$numMs = number_format($endTime - $startTime, 4);
			$msg .= '<strong>' . $method . '</strong> in <strong>' . $numMs . '</strong> seconds, ';
		}
		$msg = substr($msg, 0, -2) . '.';
		$this->show_notice(__($msg, 'administrate'));
	}

	//  Test API consistency
	public function test_api_consistency() {
		set_time_limit(1800);
		$consistent = true;
		$methods = array(
			'get_event_list',
			'get_category_list',
			'get_course_list',
			'get_location_list'
		);
		foreach ($methods as $method) {
			if (isset($result)) {
				unset($result);
			}
			for ($i = 0; $i < 10; ++$i) {
				$tmpResult = count($this->call($method, array(), true));
				if (isset($result) && ($tmpResult != $result)) {
					$consistent = false;
				}
				$result = $tmpResult;
			}
		}
		if ($consistent) {
			$this->show_notice(__('API consistency test <strong>successful</strong>.', 'administrate'));
		} else {
			$this->show_error(__('API consistency test <strong>failed</strong>.', 'administrate'));
		}
	}

	//  Get the orders table
	public function get_orders_table() {
		return $this->get_table('orders');
	}

	//  Get the logs table
	public function get_logs_table() {
		return $this->get_table('logs');
	}

	//  Get the filters table
	public function get_filters_table() {
		return $this->get_table('filters');
	}

	//  Filter text output
	public function filter_text_output($str) {
		$str = trim($str);
		if ($str != strip_tags($str)) {
			return $str;
		} else {
			return nl2br($str);
		}
	}

	//  Encode base64/serialize a string
	protected function _encode($str) {
		return base64_encode(serialize($str));
	}

	//  Decode base64/unserialize a string
	protected function _decode($str) {
		return unserialize(base64_decode($str));
	}

	//  Whether the current page is generated
	public function page_is_generated() {
		return $this->controller->page_is_generated();
	}

	// Echo prices
	public function t_prices($event, $basis = false, $currency='', $show_currency_indicator=true, $currency_indicator='', $currency_list=false) {
		ob_start();
			$this->controller->display_currency_prices($event, $basis, $currency, $show_currency_indicator, $currency_indicator, $currency_list);
			$contents = ob_get_contents();
		ob_end_clean();
		return $contents;
	}

	// Course widget url
	public function get_widget_url_course($course_code) {
		// Get course url for Wordpress
		return $this->get_course_url($course_code);
	}

	// Checkout widget url
	public function get_widget_url_checkout($event_id, $currency=false, $track_id=null) {
		// Get url for Weblink
		return $this->get_registration_url($event_id, $currency, $track_id);
	}

	// Checkout widget url
	public function get_widget_url_track($track_id = false) {
		// Get url for Weblink
		return $this->controller->get_learning_track_page() . '/' . $track_id;
	}

	public function get_widget_url_track_list() {
		return $this->get_learning_track_page();
	}

	// Resource url
	public function get_resource_url($resource_path) {
		// Get resource url for Wordpress
		return plugin_dir_url( __FILE__ )."resources/".$resource_path;
	}

	public function get_product() {
		return 'wordpress';
	}
}
