<?php

// Database access for the Wordpress plugin
//
// Wordpress plugins can access the database database through the $wpdb object
// This class is a wrapper around $wpdb.
//
class AdministrateWordpressDB extends AdministrateDB {
	public function __construct($wpdb) {
		$this->_wpdb = $wpdb;
	}

	//  Executes a query
	public function query($qry) {
		$qry = trim($qry);
		$this->_set_last_query($qry);
		return $this->_wpdb->query($qry);
	}

	//  Executes a query and get result set
	public function query_fetch($qry) {
		$qry = trim($qry);
		$this->_set_last_query($qry);
		$results_array = $this->_wpdb->get_results($qry, ARRAY_A);
		return count($results_array)==1 ? $results_array[0] : $results_array;
	}

	//  Prepare statement
	public function prepare($qry, $param_types="") {

		// Convert MySQLi syntax to $wpdb type specifiers
		$qry_format = $qry;
		foreach (str_split($param_types) as $type) {
			switch($type) {
				case "i":
					$param_str = "%d";
					break;
				case "s":
					$param_str = "%s";
					break;
				default:
					$param_str = "%s";
			}
			$qry_format = preg_replace('/\(\?\)/', $param_str, $qry_format, 1);
		}

		// Wordpress requires parameter values to be bound
		// at the time of calling $wpdb::prepare(), return
		// the formated query for now.
		return $qry_format;
	}

	//  Execute prepared
	public function prepared_exec($prep_statement, $params = array()) {
		return $this->query( $this->_wpdb->prepare($prep_statement, $params) );
	}

	//  Execute prepared and get result set
	public function prepared_fetch($prep_statement, $params = array()) {
		return $this->query_fetch($this->_wpdb->prepare($prep_statement, $params));
	}

	//  Gets the number of records returned by a query
	public function get_last_num_records() {
		return $this->_wpdb->num_rows;
	}

	//  Get last auto-generated insert id
	public function get_last_insert_id() {
		return $this->_wpdb->insert_id;
	}

	//  Get last error
	public function get_last_error() {
		return $this->_wpdb->last_error;
	}

	// Value Formatters
	public function format_string($val) {
		return esc_sql($val);
	}

	public function format_date($val) {
		return date('Y-m-d', $val);
	}

	public function get_table_prefix() {
		return $this->_wpdb->prefix;
	}
}
