// Plugin settings pages interface
//
(function( $ ) {
	var plugin = $('#AdministratePlugin');

	if ( ! plugin.length ) {
		// Exit if selector is empty
		return;
	}

	// Plugin mode
	//
	var mode = $('#mode');
	var liveMode = $('#domain, #user, #password');

	if( mode.length ) {
		mode.on("change", togglePluginMode);
		togglePluginMode();
	}

	function togglePluginMode() {
		if (mode.val() == 'demo') {
			liveMode.closest("tr").hide();
		}
		else {
			liveMode.closest("tr").show();
		}
	}

	// Construct monitoring report object
	this.monitoring_report = new AdministrateMonitoringReport($);

	//	Registration
	//
	var invoiceCheckbox = $("#payment_by_invoice");
	var invoiceFields   = $("#invoice_instructions, #invoice_send_email");

	var ccCheckbox = $("#payment_by_cc");
	var ccFields   = $("#payment_processor, #cc_send_email, #cc_instructions");

	var ccProviderSelect = $('#payment_processor');
	var ccProviders = {
		PayPal: $("#paypal_user, #paypal_password, #paypal_signature, #paypal_mode"),
		SagePay: $("#sagepay_vendor_username, #sagepay_encryption_password, #sagepay_email_message, #sagepay_mode"),
		AuthorizeNet: $("#authorizenet_user, #authorizenet_key, #authorizenet_mode, #authorizenet_md5"),
		Stripe: $("#stripe_public_key, #stripe_secret_key"),
		WPM: $('#wpm_url, #wpm_client_id, #wpm_pathway_id, #wpm_department_id, #wpm_shared_key')
	};

	ccProviderSelect.closest("td").append("<span class='stripe_connection_type'></span>")

	if( invoiceCheckbox.length ) {

		invoiceCheckbox.on("change", toggleInvoice);
		ccCheckbox.on("change", toggleCC);
		ccProviderSelect.on("change", toggleProvider);

		toggleCC();
		toggleInvoice();
	}

	function toggleInvoice() {
		if( invoiceCheckbox.prop('checked') ) {
			invoiceFields.closest("tr").show();
		}
		else {
			invoiceFields.closest("tr").hide();
			ccCheckbox.prop('checked',true);
			toggleCC();
		}
	}

	function toggleCC() {
		if( ccCheckbox.prop('checked') ) {
			ccProviderSelect.closest("tr").show();
			ccFields.closest("tr").show();

			toggleProvider();
		}
		else {
			ccProviderSelect.closest("tr").hide();
			ccFields.closest("tr").hide();

			$.each(ccProviders, function(id, provider) {
				provider.closest("tr").hide();
			});

			invoiceCheckbox.prop('checked',true);
			toggleInvoice();
		}
	}

	function toggleProvider() {
		$.each(ccProviders, function(id, provider) {
			if( id == ccProviderSelect.val() ) {
				provider.closest("tr").show();
			}
			else {
				provider.closest("tr").hide();
			}
		});

		var is_https = ( window.location.protocol == "https:" );

		if( ccProviderSelect.val() == "Stripe" ) {
			if( is_https ) {
				$(".stripe_connection_type").html("SSL detected.");
				$(".stripe_connection_type").addClass("stripe_connection_ssl");
				$(".stripe_connection_type").removeClass("stripe_connection_no_ssl");
			}
			else {
				$(".stripe_connection_type").html("It is highly recommended to use Stripe on a secure connection. No SSL detected.");
				$(".stripe_connection_type").removeClass("stripe_connection_ssl");
				$(".stripe_connection_type").addClass("stripe_connection_no_ssl");
			}
		}
		else {
			$(".stripe_connection_type").html( "" );
			$(".stripe_connection_type").removeClass("stripe_connection_ssl");
			$(".stripe_connection_type").removeClass("stripe_connection_no_ssl");
		}
	}

	// Order logs
	//
	var orderLogTable = $('#administrate-order-logs');

	if ( orderLogTable.length ) {
		orderLogTable.administrate_log_table();
	}

	/*
		Multi-currency list admin
	*/
	function add_currency(e) {
		var currency_list  = get_currency_list();
		var new_currency   = $("#administrate_currency").val();
		var new_processor  = $("#administrate_processor").val();

		currency_list[ new_currency ] = new_processor;

		update_currency_list(currency_list);
		e.preventDefault();
	};

	function remove_currency(e) {
		var currency_list    = get_currency_list();
		var removed_currency = $(this).attr("data-currency");

		console.log(removed_currency);
		delete currency_list[ removed_currency ];

		update_currency_list(currency_list);
		e.preventDefault();
	};

	function get_currency_list() {
		var currency_str = $("#currency_list").val();
		var list = {};

		if( !currency_str || currency_str[0]!='{' || currency_str[currency_str.length-1]!='}' ) {
			return list;
		}

		currency_str = currency_str.slice(1, currency_str.length-1);
		console.log(currency_str);

		$.each(currency_str.split(","), function(i,pair){
			if( !pair ) {
				return;
			}

			var split = pair.split(":");
			list[ split[0] ] = split[1];
		});
		return list;
	};

	// Update DOM
	function update_currency_list (currency_list) {
		$("#currency_list").val( serialize_list(currency_list) );

		var dom_list = $("#administrate_currency_list_preview");
		var is_empty = true;

		dom_list.html("");

		$.each( currency_list, function(currency,processor) {
			var new_item = $('<li>Prices in <span>'+currency+'</span> are enabled and associated with <span>'+processor+'</span>.&nbsp;</li>');
			var new_link = $('<a href="#" title="Remove currency" data-currency="'+currency+'">[remove]</a>');

			new_link.on("click", remove_currency);
			new_item.append(new_link);
			dom_list.append(new_item);

			is_empty = false;
		});

		if( is_empty ) {
			dom_list.hide();
			$("#administrate_multicurrency_tip").hide();
			$("#administrate_no_multicurrency").show();
		}
		else {
			dom_list.show();
			$("#administrate_multicurrency_tip").show();
			$("#administrate_no_multicurrency").hide();
		}

		function serialize_list(list) {
			var serialized_str = "";
			var first = true;

			$.each(list, function(currency, processor) {
				if( !first ) {
					serialized_str = serialized_str+",";
				}

				serialized_str = serialized_str + currency + ":" + processor;
				first = false;
			});
			serialized_str = "{" + serialized_str + "}";
			return serialized_str;
		};
	};

	// Initial update
	update_currency_list( get_currency_list() );

	// Setup event handlers
	$("#administrate_add_currency").on("click", add_currency );
	$(".administrate_remove_currency").on("click", remove_currency );


	// Confirmation to save changes
	//
	var changed = false;
	var strSave = "There are unsaved changes on this page.";

	plugin.find("input,select,textarea").on( "change", function(e) {changed = true;} );
	plugin.find("input[type=submit]").on( "click", function(e) {changed = false;} );

	$(window).on("beforeunload", function(e) {
		if( changed ) {
			return strSave;
		}
	});
}) (jQuery);
